<?php
    
    
    class Affiliates extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'affiliates' => affiliates()
            ];
            $this->view('affiliates/index', $data);
        }
        
        public function overview()
        {
            $this->view('affiliates/overview');
        }
        
        public function payouts()
        {
            if (isSuperAdmin()) {
                $payouts = payouts();
            } else {
                $payouts = payouts(user_id());
            }
            
            $data = [
                'payouts' => $payouts
            ];
            $this->view('affiliates/payouts', $data);
        }
        
        public function pay($id)
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $payout = $this->payoutModel->getPayoutById($id);
            if (empty($payout)) {
                redirect(404);
                die();
            }
            if (payout($payout->payout_id)->payout_status_id != '1') {
                flash('user_message', 'payouts', 'payout_already_processed', 'danger');
                redirect('affiliates/payouts');
                die();
            }
            if ($payout->user_id == user_id()) {
                flash('user_message', 'payouts', 'you_cannot_do_self_payment', 'danger');
                redirect('affiliates/payouts');
                die();
            }
            
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'payout_id' => $id,
                    'user_id' => $payout->user_id,
                    'payout_amount' => cleanString($_POST['payout_amount']),
                    'payout_transaction_id' => cleanString($_POST['payout_transaction_id']),
                    'payout_note' => isset($_POST['payout_note']) ? cleanString($_POST['payout_note']) : NULL,
                    'payout_status_id' => 2,
                    'total_earnings' => user_payouts_total($payout->user_id)->payout_amount,
                    'payout' => $payout
                ];
                if (empty($data['payout_amount'])) {
                    $data['input_error'][] = 'please_enter_payout_amount';
                }
                if ($data['payout_amount'] != $data['payout']->payout_amount) {
                    $data['input_error'][] = 'payout_amount_should_be_equal_to_requested_amount';
                }
                if (empty($data['payout_transaction_id'])) {
                    $data['input_error'][] = 'please_enter_payout_transaction_id';
                }
                if (payout_transaction_id_exists($data['payout_transaction_id'])) {
                    $data['input_error'][] = 'payout_transaction_id_already_exists';
                }
                if (empty($data['input_error'])) {
                    if ($this->payoutModel->payRequestedPayout($data)) {
                        sendEmail('PayoutSent', user($payout->user_id)->email, 'payouts', $id);
                        notificationToUser('requested_payout_paid', $payout->user_id, 'affiliates/show/' . $id);
                        addActivityLog('payout', $id, 'requested_payout_paid');
                        flash('user_message', 'payouts', 'requested_payout_paid');
                        redirect('affiliates/payouts');
                    } else {
                        flash('user_message', 'payouts', 'paying_requested_payout_failed', 'danger');
                        redirect('affiliates/pay');
                    }
                    die();
                }
                $this->view('affiliates/pay', $data);
            }
            $data = [
                'user_id' => $payout->user_id,
                'total_earnings' => user_payouts_total($payout->user_id)->payout_amount,
                'payout' => $payout
            ];
            $this->view('affiliates/pay', $data);
        }
        
        public function request_payout()
        {
            $user = $this->userModel->getUserById(user_id());
            if (empty($user->payout_mode_id) || empty($user->payout_address)) {
                flash('user_message', 'payouts', 'set_payout_mode_and_payout_address_to_request_payout', 'danger');
                redirect('users/profile');
                die();
            }
            
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'payout_name' => getOption('payout_prefix') . '-' . str_pad(next_payout_id(), 5, "0", STR_PAD_LEFT),
                    'user_id' => user_id(),
                    'payout_mode_id' => user(user_id())->payout_mode_id,
                    'payout_address' => user(user_id())->payout_address,
                    'payout_amount' => cleanString($_POST['payout_amount']),
                    'total_earnings' => affiliates_total()
                ];
                if (empty($data['payout_mode_id']) || empty($data['payout_address'])) {
                    $data['input_error'] = 'please_set_your_payout_mode_and_address_under_profile_page';
                }
                if (empty($data['payout_amount'])) {
                    $data['input_error'] = 'please_enter_payout_amount';
                }
                if (!empty($data['payout_mode_id']) && !empty($data['payout_address'])) {
                    if ($data['payout_amount'] < payout_mode($data['payout_mode_id'])->payout_mode_minimum) {
                        $data['input_error'] = 'your_requested_payout_amount_below_minimum_payout_limit';
                    }
                }
                
                if ($data['payout_amount'] > ($data['total_earnings'] - user_payouts_total($data['user_id'], '1')->payout_amount - user_payouts_total($data['user_id'], '2')->payout_amount)) {
                    $data['input_error'] = 'payout_amount_cannot_be_greater_than_due_earnings';
                }
                if (user_payouts_total($data['user_id'], 1)->payout_amount > 0) {
                    $data['input_error'] = 'you_have_another_pending_payout';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->payoutModel->requestPayout($data)) {
                        sendEmail('PayoutRequestedForAdmin', user('1')->email, 'payouts', $id);
                        notificationToUser('payout_requested', '1', 'affiliates/payouts');
                        addActivityLog('payout', $id, 'payout_requested');
                        flash('user_message', 'payouts', 'payout_requested');
                    } else {
                        flash('user_message', 'payouts', 'requesting_payout_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'payouts', $data['input_error'], 'danger');
                }
                redirect('affiliates/payouts');
                die();
            }
            $data = [
                'user_id' => user_id(),
                'total_earnings' => user_payouts_total(user_id())->payout_amount
            ];
            $this->view('affiliates/payouts', $data);
        }
        
        public function show($id = NULL)
        {
            $payout = $this->payoutModel->getPayoutById($id);
            if ($payout->user_id != user_id() && !isSuperAdmin()) {
                redirect(404);
                die();
            }
            if (empty($payout)) {
                redirect(404);
            }
            $data = [
                'payout' => $payout
            ];
            $this->view('affiliates/show', $data);
        }
        
        public function decline($id = NULL)
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $payout = $this->payoutModel->getPayoutById($id);
            if (empty($payout)) {
                redirect(404);
            }
            if (isSuperAdmin() && payout($payout->payout_id)->payout_status_id != '1') {
                flash('user_message', 'payouts', 'payout_already_processed', 'danger');
                redirect('affiliates/payouts');
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (empty($payout)) {
                    redirect(404);
                }
                $data = [
                    'payout_id' => $id,
                    'payout_status_id' => '3',
                    'payout' => $payout
                ];
                if ($this->payoutModel->updatePayoutStatus($data)) {
                    sendEmail('PayoutDeclined', user($payout->user_id)->email, 'payouts', $id);
                    notificationToUser('payout_declined', $payout->user_id, 'affiliates/show/' . $id);
                    addActivityLog('user', $id, 'payout_declined');
                    flash('user_message', 'payouts', 'payout_declined');
                } else {
                    flash('user_message', 'payouts', 'payout_declining_failed', 'danger');
                }
            }
            redirect('affiliates/payouts');
            die();
        }
        
        public function cancel($id = NULL)
        {
            $payout = $this->payoutModel->getPayoutById($id);
            if ($payout->user_id != user_id()) {
                redirect(404);
                die();
            }
            if (empty($payout)) {
                redirect(404);
            }
            if ($payout->user_id != user_id() && payout($payout->payout_id)->payout_status_id != '1') {
                flash('user_message', 'payouts', 'payout_already_processed', 'danger');
                redirect('affiliates/payouts');
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (empty($payout)) {
                    redirect(404);
                }
                $data = [
                    'payout_id' => $id,
                    'payout_status_id' => '4',
                    'payout' => $payout
                ];
                if ($this->payoutModel->updatePayoutStatus($data)) {
                    sendEmail('PayoutDeclined', user($payout->user_id)->email, 'payouts', $id);
                    notificationToUser('payout_declined', $payout->user_id, 'affiliates/show/' . $id);
                    addActivityLog('user', $id, 'payout_declined');
                    flash('user_message', 'payouts', 'payout_declined');
                } else {
                    flash('user_message', 'payouts', 'payout_declining_failed', 'danger');
                }
            }
            redirect('affiliates/payouts');
            die();
        }
        
        public function send_referral_link()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'email' => cleanString($_POST['email'])
                ];
                if (empty($data['email'])) {
                    $data['input_error'] = 'please_enter_email_address';
                }
                if (empty($data['input_error'])) {
                    if (sendEmail('InviteFriend', $data['email'], 'affiliates', NULL)) {
                        flash('user_message', 'payouts', 'affiliate_link_sent');
                    } else {
                        flash('user_message', 'payouts', 'sending_affiliate_link_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'payouts', $data['input_error'], 'danger');
                }
                redirect('affiliates/overview');
                die();
            }
        }
        
        
    }