<?php
    
    
    class Assistants extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'assistants' => chat_assistants(1)
            ];
            $this->view('assistants/grid', $data);
        }
        
        public function my_assistants()
        {
            $data = [
                'assistants' => chat_assistants(user_id())
            ];
            $this->view('assistants/grid', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $chat_assistant_logo = !empty($_FILES['chat_assistant_logo']['name']) ? fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['chat_assistant_logo']['name'])) : NULL;
                
                $data = [
                    'chat_assistant_name' => cleanString($_POST['chat_assistant_name']),
                    'chat_assistant_profession' => cleanString($_POST['chat_assistant_profession']),
                    'chat_assistant_logo' => $chat_assistant_logo,
                    'chat_assistant_prompt' => cleanString($_POST['chat_assistant_prompt']),
                    'chat_assistant_status_id' => !empty($_POST['chat_assistant_status_id']) ? cleanString($_POST['chat_assistant_status_id']) : 1
                ];
                if (empty($data['chat_assistant_name'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_name';
                }
                if (chat_assistant_exists($data['chat_assistant_name'])) {
                    $data['input_error'][] = 'chat_assistant_name_already_exists';
                }
                if (empty($data['chat_assistant_profession'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_profession';
                }
                if (empty($data['chat_assistant_prompt'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_prompt';
                }
                if (!empty($chat_assistant_logo)) {
                    upload_image($data['chat_assistant_logo'], 'media', $_FILES['chat_assistant_logo']['tmp_name']);
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->assistantModel->addChatAssistant($data)) {
                        addActivityLog('assistant', $id, 'chat_assistant_added');
                        flash('user_message', 'assistants', 'chat_assistant_added');
                        redirect('assistants/show/' . $id);
                    } else {
                        flash('user_message', 'assistants', 'adding_chat_assistant_failed', 'danger');
                        redirect('assistants/add');
                    }
                    die();
                }
                $this->view('assistants/add', $data);
            }
            $this->view('assistants/add');
        }
        
        public function edit($id = NULL)
        {
            $assistant = chat_assistant($id);
            if (empty($assistant) || ($assistant->chat_assistant_added_by != user_id() && !isSuperAdmin())) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'chat_assistant_id' => $id,
                    'chat_assistant_name' => cleanString($_POST['chat_assistant_name']),
                    'chat_assistant_profession' => cleanString($_POST['chat_assistant_profession']),
                    'chat_assistant_prompt' => cleanString($_POST['chat_assistant_prompt']),
                    'chat_assistant_status_id' => cleanString($_POST['chat_assistant_status_id']),
                    'assistant' => $assistant
                ];
                if (empty($data['chat_assistant_name'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_name';
                }
                if (empty($data['chat_assistant_profession'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_profession';
                }
                if (empty($data['chat_assistant_prompt'])) {
                    $data['input_error'][] = 'please_enter_chat_assistant_prompt';
                }
                if (empty($data['input_error'])) {
                    if ($this->assistantModel->updateChatAssistant($data)) {
                        addActivityLog('chat_assistant', $id, 'chat_assistant_updated');
                        flash('user_message', 'chat_assistants', 'chat_assistant_updated');
                    } else {
                        flash('user_message', 'chat_assistants', 'chat_assistant_update_failed', 'danger');
                    }
                    redirect('assistants/show/' . $id);
                    die();
                }
                $this->view('assistants/edit', $data);
            }
            $data = [
                'assistant' => $assistant
            ];
            $this->view('assistants/edit', $data);
        }
        
        public function search_assistant()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'query' => cleanString($_POST['query'])
                ];
                $result = $this->assistantModel->searchChatAssistants($data['query']);
                if ($result) {
                    foreach ($result as $assistant) {
                        $attributes = assistant_attributes($assistant->chat_assistant_id, user_id());
                        echo '<a href="' . URLROOT . '/assistants/show/' . $assistant->chat_assistant_id . '" class="list-group-item list-group-item-action border-' . $attributes->border . ' mb-3 assistant-box">' . $assistant->chat_assistant_name . '<p class="text-muted">' . $assistant->chat_assistant_profession . '</p></a>';
                    }
                } else {
                    echo '<p class="list-group-item border-1">' . no_records('chat_assistants') . '</p>';
                }
            }
        }
        
        public function show($id = NULL)
        {
            $assistant = chat_assistant($id);
            if (empty($assistant) || ($assistant->chat_assistant_added_by != 1 && $assistant->chat_assistant_added_by != user_id())) {
                redirect(404);
            }
            $data = [
                'assistant' => $assistant
            ];
            $this->view('assistants/show', $data);
        }
        
        public function chat_assistant($id = NULL)
        {
            $assistant = chat_assistant($id);
            if (empty($assistant)) {
                redirect(404);
            }
            redirect('chats/assistant/' . $id);
            die();
        }
        
        public function image($id = NULL)
        {
            $assistant = chat_assistant($id);
            if (empty($assistant) || $assistant->chat_assistant_added_by != user_id()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'chat_assistant_id' => $id,
                    'chat_assistant_logo' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['chat_assistant_logo']['name'])),
                    'current_image' => $assistant->chat_assistant_logo,
                    'assistant' => $assistant
                ];
                if (!upload_image($data['chat_assistant_logo'], 'media', $_FILES['chat_assistant_logo']['tmp_name'])) {
                    flash('user_message', 'assistants', 'update_failed', 'danger');
                } else {
                    if ($this->assistantModel->updateChatAssistantImage($data)) {
                        unlinkUploads('media', $data['current_image']);
                        addActivityLog('assistant', user_id(), 'chat_assistant_logo_updated');
                        flash('user_message', 'assistants', 'chat_assistant_logo_updated');
                    } else {
                        flash('user_message', 'assistants', 'chat_assistant_logo_update_failed', 'danger');
                    }
                }
            }
            redirect('assistants/show/' . $id);
            die();
        }
        
        public function activate($id = NULL)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $assistant = chat_assistant($id);
                if (empty($assistant) || ($assistant->chat_assistant_added_by != user_id() && !isSuperAdmin())) {
                    redirect(404);
                }
                $data = [
                    'chat_assistant_status_id' => 1,
                    'chat_assistant_id' => $id
                ];
                if ($id == 1) {
                    flash('user_message', 'assistants', 'general_chat_bot_cannot_be_activated', 'danger');
                } elseif ($this->assistantModel->updateChatAssistantStatus($data)) {
                    addActivityLog('assistant', $id, 'chat_assistant_activated');
                    flash('user_message', 'assistants', 'chat_assistant_activated');
                } else {
                    flash('user_message', 'assistants', 'chat_assistant_activation_failed', 'danger');
                }
                redirect('assistants/show/' . $id);
                die();
            } else {
                redirect('assistants');
            }
        }
        
        public function deactivate($id = NULL)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $assistant = chat_assistant($id);
                if (empty($assistant) || ($assistant->chat_assistant_added_by != user_id() && !isSuperAdmin())) {
                    redirect(404);
                }
                $data = [
                    'chat_assistant_status_id' => 3,
                    'chat_assistant_id' => $id
                ];
                if ($id == 1) {
                    flash('user_message', 'assistants', 'general_chat_bot_cannot_be_deactivated', 'danger');
                } elseif ($this->assistantModel->updateChatAssistantStatus($data)) {
                    addActivityLog('assistant', $id, 'chat_assistant_deactivated');
                    flash('user_message', 'assistants', 'chat_assistant_deactivated');
                } else {
                    flash('user_message', 'assistants', 'chat_assistant_deactivation_failed', 'danger');
                }
                redirect('assistants/show/' . $id);
                die();
            } else {
                redirect('assistants');
            }
        }
        
        public function delete($id = NULL)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $assistant = chat_assistant($id);
                if (empty($assistant) || ($assistant->chat_assistant_added_by != user_id() && !isSuperAdmin())) {
                    redirect(404);
                }
                if ($assistant->chat_assistant_added_by == 1) {
                    flash('user_message', 'assistants', 'inbuilt_chat_assistant_cannot_be_deleted', 'danger');
                } elseif ($this->assistantModel->deleteChatAssistant($id)) {
                    addActivityLog('assistant', $id, 'chat_assistant_deleted');
                    flash('user_message', 'assistants', 'chat_assistant_deleted');
                } else {
                    flash('user_message', 'assistants', 'chat_assistant_deletion_failed', 'danger');
                }
            }
            redirect('assistants');
            die();
        }
        
        
    }