<?php
    
    
    class Faqs extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'faqs' => faqs()
            ];
            $this->view('faqs/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'question' => cleanString($_POST['question']),
                    'answer' => cleanString($_POST['answer']),
                    'faq_status_id' => !empty($_POST['faq_status_id']) ? cleanString($_POST['faq_status_id']) : 1,
                ];
                if (empty($data['question'])) {
                    $data['input_error'][] = 'please_enter_question';
                }
                if (empty($data['answer'])) {
                    $data['input_error'][] = 'please_enter_answer';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->faqModel->addFaq($data)) {
                        addActivityLog('faq', $id, 'faq_added');
                        flash('user_message', 'faq', 'faq_added');
                        redirect('faqs/show/' . $id);
                    } else {
                        flash('user_message', 'faq', 'adding_faq_failed', 'danger');
                        redirect('faqs/add');
                    }
                    die();
                }
                $this->view('faqs/add', $data);
            }
            $this->view('faqs/add');
        }
        
        public function edit($id = NULL)
        {
            $faq = $this->faqModel->getFaqById($id);
            if (empty($faq)) {
                redirect(404);
            }
            
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'faq_id' => $id,
                    'question' => cleanString($_POST['question']),
                    'answer' => cleanString($_POST['answer']),
                    'faq_status_id' => !empty($_POST['faq_status_id']) ? cleanString($_POST['faq_status_id']) : 0,
                    'faq' => $faq
                ];
                if (empty($data['question'])) {
                    $data['input_error'][] = 'please_enter_question';
                }
                if (empty($data['answer'])) {
                    $data['input_error'][] = 'please_enter_answer';
                }
                if (empty($data['input_error'])) {
                    if ($this->faqModel->updateFaq($data)) {
                        addActivityLog('faq', $id, 'faq_updated');
                        flash('user_message', 'faq', 'faq_updated');
                    } else {
                        flash('user_message', 'faq', 'faq_update_failed', 'danger');
                    }
                    redirect('faqs/show/' . $id);
                    die();
                }
                $this->view('faqs/edit', $data);
            }
            $data = [
                'faq' => $faq
            ];
            $this->view('faqs/edit', $data);
        }
        
        public function show($id = NULL)
        {
            $faq = $this->faqModel->getFaqById($id);
            if (empty($faq)) {
                redirect(404);
            }
            
            $data = [
                'faq' => $faq
            ];
            $this->view('faqs/show', $data);
        }
        
        public function delete($id = NULL)
        {
            $faq = $this->faqModel->getFaqById($id);
            if (empty($faq)) {
                redirect(404);
            }
            
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->faqModel->deleteFaq($id)) {
                    addActivityLog('faq', $id, 'faq_deleted');
                    flash('user_message', 'faq', 'faq_deleted');
                    redirect('faqs');
                } else {
                    flash('user_message', 'faq', 'faq_delete_failed', 'danger');
                    redirect('faqs/show/' . $id);
                }
                die();
            } else {
                redirect('faqs');
            }
        }
        
        
    }
