<?php
    
    
    class Finetuning extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
                die();
            }
            
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                flash('user_message', 'chats', 'you_do_not_have_an_active_subscription', 'danger');
                redirect('packages/all');
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'finetunes' => finetune_files(user_id())
            ];
            $this->view('finetuning/index', $data);
        }
        
        public function files()
        {
            if ((int)user(user_id())->user_remaining_trained_models == 0) {
                $flash = 'maximum_number_of_trained_models_in_your_subscription_package_reached';
            }
            if (isset($flash)) {
                flash('user_message', 'finetune', $flash, 'danger');
                redirect('finetuning');
                die();
            }
            $this->view('finetuning/files');
        }
        
        public function upload()
        {
            $target_dir = APPROOT . MEDIA_FOLDER;
            $file = fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['file']['name']));
            if (move_uploaded_file($_FILES['file']['tmp_name'], $target_dir . $file)) {
                $data = [
                    'finetune_file' => $file,
                    'finetune_file_size' => filesize(APPROOT . MEDIA_FOLDER . $file)
                ];
                $file_id = $this->finetuneModel->addFineTuneFile($data);
                echo json_encode(array(
                    "file_id" => $file_id,
                    "finetune_file" => URLROOT . '/app' . MEDIA_FOLDER . $file,
                    "status" => true,
                    "message" => label('file_uploaded_successfully')
                ));
            } else {
                echo json_encode(array(
                    "file_id" => false,
                    "finetune_file" => false,
                    "status" => false,
                    "message" => label('failed_to_upload_file')
                ));
            }
        }
        
        public function upload_for_finetuning($id = NULL)
        {
            $finetune_file = finetune_file($id);
            if (empty($finetune_file) || $finetune_file->finetune_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if ((int)user(user_id())->user_remaining_trained_models == 0) {
                $flash = 'maximum_number_of_trained_models_in_your_subscription_package_reached';
            }
            if (isset($flash)) {
                flash('user_message', 'finetune', $flash, 'danger');
                redirect('finetuning');
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (is_null($finetune_file->finetune_file_output_id)) {
                    $response = ai_upload_finetune_file(URLROOT . '/app' . MEDIA_FOLDER . $finetune_file->finetune_file);
                    
                    if (!empty($response) && $response['status'] == 'error') {
                        $data['input_error'] = $response['message'];
                    }
                } else {
                    $data['input_error'] = "file_already_uploaded";
                }
                
                if (empty($data['input_error']) && isset($response)) {
                    $datax = [
                        'finetune_id' => $id,
                        'finetune_file_output_id' => $response['file_id'],
                        'finetune_file_output_filename' => $response['filename'],
                        'finetune_file_output_size' => $response['bytes'],
                        'finetune_file_output_status' => $response['file_status']
                    ];
                    if ($this->finetuneModel->updateFineTuneFileOutput($datax)) {
                        
                        $available_trained_models_prepaid = user(user_id())->user_remaining_trained_models;
                        $response['old_tokens'] = $available_trained_models_prepaid;
                        $response['current_tokens'] = ($available_trained_models_prepaid == -1) ? $available_trained_models_prepaid : ($available_trained_models_prepaid - 1);
                        
                        $dataw = [
                            'user_id' => user_id(),
                            'user_remaining_words' => (int)user(user_id())->user_remaining_words,
                            'user_remaining_chat_assistants' => (int)user(user_id())->user_remaining_chat_assistants,
                            'user_remaining_trained_models' => (int)$response['current_tokens'],
                            'user_remaining_chat_files' => (int)user(user_id())->user_remaining_chat_files,
                            'user_remaining_chat_links' => (int)user(user_id())->user_remaining_chat_links,
                            'user_remaining_chat_sitemaps' => (int)user(user_id())->user_remaining_chat_sitemaps
                        ];
                        
                        $dataz = [
                            'user_id' => user_id(),
                            'type' => 'trained_models',
                            'transaction_type' => 'out',
                            'package_id' => NULL,
                            'openai_template_id' => NULL,
                            'old' => (int)$response['old_tokens'],
                            'current' => $dataw['user_remaining_trained_models'],
                            'user_package_history_comment' => 'Finetune ID: ' . $id,
                            'user_package_history_added_by' => user_id()
                        ];
                        $this->userModel->addUserPackageHistory($dataz);
                        $this->userModel->editUserRemaining($dataw);
                        
                        addActivityLog('finetune', $id, 'file_uploaded_for_finetuning');
                        flash('user_message', 'finetune', 'file_uploaded_for_finetuning');
                    } else {
                        flash('user_message', 'finetune', 'uploading_file_for_finetuning_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'finetune', $data['input_error'], 'danger');
                }
            }
            redirect('finetuning/show/' . $id);
            die();
        }
        
        public function finetune($id = NULL)
        {
            $finetune_file = finetune_file($id);
            if (empty($finetune_file) || $finetune_file->finetune_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if (isset($flash)) {
                flash('user_message', 'finetune', $flash, 'danger');
                redirect('finetuning');
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {                
                if (!is_null($finetune_file->finetune_file_output_id)) {
                    $response = ai_create_finetune_job($finetune_file->finetune_file_output_id);
                    
                    if (!empty($response) && $response['status'] == 'error') {
                        $data['input_error'] = $response['message'];
                    }
                } else {
                    $data['input_error'] = "file_not_uploaded";
                }
                
                if (empty($data['input_error']) && isset($response)) {
                    $datax = [
                        'finetune_id' => $id,
                        'finetune_job_output_id' => $response['finetune_job_id'],
                        'finetune_job_output_status' => $response['finetune_job_status'],
                        'finetune_job_output_model' => $response['finetune_job_model'],
                        'finetune_job_output_tokens' => NULL
                    ];
                    if ($this->finetuneModel->updateFineTuneJobOutput($datax)) {
                        addActivityLog('finetune', $id, 'finetune_job_submitted');
                        flash('user_message', 'finetune', 'finetune_job_submitted');
                    } else {
                        flash('user_message', 'finetune', 'finetune_job_submission_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'finetune', $data['input_error'], 'danger');
                }
            }
            redirect('finetuning/show/' . $id);
            die();
        }
        
        public function check_finetune_status($id = NULL)
        {
            $finetune_file = finetune_file($id);
            if (empty($finetune_file) || $finetune_file->finetune_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'finetune_id' => $id
                ];
                
                $finetune_file = finetune_file($data['finetune_id']);
                
                if (!is_null($finetune_file->finetune_job_output_id)) {
                    $response = ai_retrieve_finetune_job($finetune_file->finetune_job_output_id);
                    
                    if (!empty($response) && $response['status'] == 'error') {
                        $data['input_error'] = $response['message'];
                    }
                } else {
                    $data['input_error'] = "finetune_job_not_started";
                }
                
                if (empty($data['input_error']) && isset($response) && $response['status'] == 'success') {
                    $datax = [
                        'finetune_id' => $id,
                        'finetune_job_output_id' => $response['finetune_job_id'],
                        'finetune_job_output_status' => $response['finetune_job_status'],
                        'finetune_job_output_model' => $response['finetune_job_model'],
                        'finetune_job_output_tokens' => $response['trained_tokens'],
                        'finetune_job_output_model_used' => $response['model']
                    ];
                    if ($this->finetuneModel->updateFineTuneJobOutput($datax)) {
                        addActivityLog('finetune', $id, 'finetune_job_submitted');
                        flash('user_message', 'finetune', 'finetune_job_submitted');
                    } else {
                        flash('user_message', 'finetune', 'finetune_job_submission_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'finetune', $data['input_error'], 'danger');
                }
            }
            redirect('finetuning/show/' . $id);
            die();
        }
        
        public function show($id = NULL)
        {
            $finetune_file = finetune_file($id);
            if (empty($finetune_file) || $finetune_file->finetune_file_added_by != user_id()) {
                redirect(404);
            }            
            $data = [
                'finetune' => $finetune_file
            ];
            $this->view('finetuning/show', $data);
        }
        
        public function chat_trained_model($id = NULL)
        {
            $finetune_file = finetune_file($id);
            if (empty($finetune_file) || $finetune_file->finetune_file_added_by != user_id()) {
                redirect(404);
            }
            redirect('chats/trained_model/' . $id);
            die();
        }
        
        
    }