<?php
    
    
    class Links extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
                die();
            }
            
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                flash('user_message', 'chats', 'you_do_not_have_an_active_subscription', 'danger');
                redirect('packages/all');
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'chat_links' => chat_links(user_id())
            ];
            $this->view('links/index', $data);
        }
        
        public function submit()
        {
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                $data['input_error'] = 'you_do_not_have_an_active_subscription';
            }
            if ($subscription && (int)user(user_id())->user_remaining_chat_links == 0) {
                $data['input_error'] = 'maximum_number_of_chat_links_in_your_subscription_package_reached';
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $url = cleanString($_POST['chat_link']);
                if (!validate_url($url)) {
                    $data['input_error'] = "invalid_url";
                }
                if (chat_link_exists($url)) {
                    $data['input_error'] = 'chat_link_already_exists';
                }
                if (empty($data['input_error'])) {
                    $content = @file_get_contents($url);
                    
                    if ($content !== false) {
                        $file_name = time() . uid(5) . '.txt';
                        $file = fopen(APPROOT . DIRECTORY_SEPARATOR . MEDIA_FOLDER . DIRECTORY_SEPARATOR . $file_name, "w") or die("Unable to open file!");
                        fwrite($file, $content);
                        fclose($file);
                        $data = [
                            'chat_file' => $file_name,
                            'chat_file_size' => filesize(APPROOT . MEDIA_FOLDER . $file_name),
                            'chat_link' => $url
                        ];
                        if ($chat_file_id = $this->chatfileModel->addChatFile($data)) {
                            
                            $dataw = [
                                'user_id' => user_id(),
                                'user_remaining_words' => (int)user(user_id())->user_remaining_words,
                                'user_remaining_chat_assistants' => (int)user(user_id())->user_remaining_chat_assistants,
                                'user_remaining_trained_models' => (int)user(user_id())->user_remaining_trained_models,
                                'user_remaining_chat_files' => (int)user(user_id())->user_remaining_chat_files,
                                'user_remaining_chat_links' => (user(user_id())->user_remaining_chat_links > 0) ? (int)user(user_id())->user_remaining_chat_links - 1 : -1,
                                'user_remaining_chat_sitemaps' => (int)user(user_id())->user_remaining_chat_sitemaps
                            ];
                            
                            $dataz = [
                                'user_id' => user_id(),
                                'type' => 'chat_links',
                                'transaction_type' => 'out',
                                'package_id' => NULL,
                                'openai_template_id' => NULL,
                                'old' => ($dataw['user_remaining_chat_links'] == -1) ? -1 : $dataw['user_remaining_chat_links'] + 1,
                                'current' => $dataw['user_remaining_chat_links'],
                                'user_package_history_comment' => 'Chat Link: ' . $chat_file_id,
                                'user_package_history_added_by' => user_id()
                            ];
                            $this->userModel->addUserPackageHistory($dataz);
                            $this->userModel->editUserRemaining($dataw);
                            
                            addActivityLog('link', $chat_file_id, 'link_submitted');
                            flash('user_message', 'link', 'link_submitted');
                        } else {
                            flash('user_message', 'link', 'submitting_link_failed', 'danger');
                        }
                    } else {
                        flash('user_message', 'link', 'submitting_link_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'link', $data['input_error'], 'danger');
                }
            }
            redirect('links');
            die();
        }
        
        public function upload_for_chat($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                $data['input_error'] = 'you_do_not_have_an_active_subscription';
            }
            if ($subscription && (int)user(user_id())->user_remaining_chat_links == 0) {
                $data['input_error'] = 'maximum_number_of_chat_links_in_your_subscription_package_reached';
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (empty($data['input_error'])) {
                    $data = [
                        'chat_file_id' => $id
                    ];
                    
                    $chat_file = chat_file($data['chat_file_id']);
                    
                    if (is_null($chat_file->chat_file_output_id)) {
                        $response = ai_upload_assistant_file(URLROOT . '/app' . MEDIA_FOLDER . $chat_file->chat_file);
                        
                        if (!empty($response) && $response['status'] == 'error') {
                            $data['input_error'] = $response['message'];
                        }
                    } else {
                        $data['input_error'] = "link_already_uploaded";
                    }
                    
                    if (empty($data['input_error'])) {
                        $datax = [
                            'chat_file_id' => $id,
                            'chat_file_output_id' => $response['file_id'],
                            'chat_file_output_filename' => $response['filename'],
                            'chat_file_output_size' => $response['bytes'],
                            'chat_file_output_status' => $response['file_status']
                        ];
                        if ($this->chatfileModel->updateChatFileOutput($datax)) {
                            addActivityLog('link', $id, 'link_uploaded_for_chatting');
                            flash('user_message', 'link', 'link_uploaded_for_chatting');
                        } else {
                            flash('user_message', 'link', 'uploading_link_for_chatting_failed', 'danger');
                        }
                    } else {
                        flash('user_message', 'link', $data['input_error'], 'danger');
                    }
                } else {
                    flash('user_message', 'link', $data['input_error'], 'danger');
                }
            }
            redirect('links/show/' . $id);
            die();
        }
        
        public function show($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            
            $data = [
                'chat_link' => $chat_link
            ];
            $this->view('links/show', $data);
        }
        
        public function chat_link($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            redirect('chats/link/' . $id);
            die();
        }
        
        public function delete($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->chatfileModel->deleteChatFile($id)) {
                    unlinkUploads('media', $chat_link->chat_file);
                    if (!empty($chat_link->chat_file_output_id)) {
                        ai_delete_file($chat_link->chat_file_output_id);
                    }
                    addActivityLog('file', $id, 'file_deleted');
                    flash('user_message', 'file', 'file_deleted');
                } else {
                    flash('user_message', 'file', 'file_delete_failed', 'danger');
                }
            }
            redirect('links');
            die();
        }
        
        public function sitemaps()
        {
            $data = [
                'sitemaps' => sitemaps(user_id())
            ];
            $this->view('links/sitemaps', $data);
        }
        
        public function add_sitemap()
        {
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                $data['input_error'] = 'you_do_not_have_an_active_subscription';
            }
            if ($subscription && (int)user(user_id())->user_remaining_chat_sitemaps == 0) {
                $data['input_error'] = 'maximum_number_of_chat_sitemaps_in_your_subscription_package_reached';
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $url = cleanString($_POST['chat_link']);
                if (!validate_url($url)) {
                    $data['input_error'] = "invalid_url";
                }
                if (!validate_xml_sitemap($url)) {
                    $data['input_error'] = "invalid_sitemap_url";
                }
                if (chat_link_exists($url)) {
                    $data['input_error'] = 'chat_sitemap_already_exists';
                }
                if (empty($data['input_error'])) {
                    $data = [
                        'chat_file' => $url,
                        'chat_file_size' => NULL,
                        'chat_link' => $url,
                        'is_sitemap' => 1
                    ];
                    if ($chat_file_id = $this->chatfileModel->addChatFile($data)) {
                        foreach (sitemap_urls($url) as $link) {
                            $content = @file_get_contents($link->loc);
                            if ($content !== false) {
                                $file_name = time() . uid(5) . '.txt';
                                $file = fopen(APPROOT . DIRECTORY_SEPARATOR . MEDIA_FOLDER . DIRECTORY_SEPARATOR . $file_name, "w") or die("Unable to open file!");
                                fwrite($file, $content);
                                fclose($file);
                                $dataX = [
                                    'chat_file_id' => $chat_file_id,
                                    'chat_sitemap_link' => $link->loc,
                                    'chat_sitemap_file_name' => $file_name,
                                    'chat_sitemap_link_size' => filesize(APPROOT . MEDIA_FOLDER . $file_name)
                                ];
                                $this->chatfileModel->addChatSitemapLink($dataX);
                            } else {
                                $data['input_error'] = "link_content_could_not_be_read";
                            }
                            addActivityLog('sitemap', $chat_file_id, 'sitemap_submitted');
                            flash('user_message', 'sitemap', 'sitemap_submitted');
                        }
                        
                        $dataw = [
                            'user_id' => user_id(),
                            'user_remaining_words' => (int)user(user_id())->user_remaining_words,
                            'user_remaining_chat_assistants' => (int)user(user_id())->user_remaining_chat_assistants,
                            'user_remaining_trained_models' => (int)user(user_id())->user_remaining_trained_models,
                            'user_remaining_chat_files' => (int)user(user_id())->user_remaining_chat_files,
                            'user_remaining_chat_links' => (int)user(user_id())->user_remaining_chat_links,
                            'user_remaining_chat_sitemaps' => (user(user_id())->user_remaining_chat_sitemaps > 0) ? (int)user(user_id())->user_remaining_chat_sitemaps - 1 : -1
                        ];
                        
                        $dataz = [
                            'user_id' => user_id(),
                            'type' => 'chat_sitemaps',
                            'transaction_type' => 'out',
                            'package_id' => NULL,
                            'openai_template_id' => NULL,
                            'old' => ($dataw['user_remaining_chat_sitemaps'] == -1) ? -1 : $dataw['user_remaining_chat_sitemaps'] + 1,
                            'current' => $dataw['user_remaining_chat_sitemaps'],
                            'user_package_history_comment' => 'Chat Sitemap: ' . $chat_file_id,
                            'user_package_history_added_by' => user_id()
                        ];
                        $this->userModel->addUserPackageHistory($dataz);
                        $this->userModel->editUserRemaining($dataw);
                    } else {
                        flash('user_message', 'sitemap', 'submitting_sitemap_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'sitemap', $data['input_error'], 'danger');
                }
            }
            redirect('links/sitemaps');
            die();
        }
        
        public function upload_sitemap()
        {
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                $data['input_error'] = 'you_do_not_have_an_active_subscription';
            }
            if ($subscription && (int)user(user_id())->user_remaining_chat_sitemaps == 0) {
                $data['input_error'] = 'maximum_number_of_chat_sitemaps_in_your_subscription_package_reached';
            }
            if (empty($data['input_error'])) {
                // Check if a file has been uploaded
                if ($_FILES['file']['error'] === UPLOAD_ERR_OK) {
                    // Check if the uploaded file is an XML file
                    if ($_FILES['file']['type'] === 'text/xml') {
                        // Move the uploaded file to a temporary location
                        $tempFilePath = $_FILES['file']['tmp_name'];
                        // Validate the XML sitemap
                        if (validate_xml_sitemap($tempFilePath)) {
                            // Process the valid XML sitemap
                            $target_dir = APPROOT . MEDIA_FOLDER;
                            $file = fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['file']['name']));
                            if (move_uploaded_file($_FILES['file']['tmp_name'], $target_dir . $file)) {
                                $data = [
                                    'chat_file' => $file,
                                    'chat_file_size' => filesize(APPROOT . MEDIA_FOLDER . $file),
                                    'chat_link' => $file,
                                    'is_sitemap' => 1
                                ];
                                if ($chat_file_id = $this->chatfileModel->addChatFile($data)) {
                                    foreach (sitemap_urls(URLROOT . '/app/uploads/media/' . $file) as $link) {
                                        $content = @file_get_contents($link->loc);
                                        if ($content !== false) {
                                            $file_name = time() . uid(5) . '.txt';
                                            $file = fopen(APPROOT . DIRECTORY_SEPARATOR . MEDIA_FOLDER . DIRECTORY_SEPARATOR . $file_name, "w") or die("Unable to open file!");
                                            fwrite($file, $content);
                                            fclose($file);
                                            $dataX = [
                                                'chat_file_id' => $chat_file_id,
                                                'chat_sitemap_link' => $link->loc,
                                                'chat_sitemap_file_name' => $file_name,
                                                'chat_sitemap_link_size' => filesize(APPROOT . MEDIA_FOLDER . $file_name)
                                            ];
                                            $this->chatfileModel->addChatSitemapLink($dataX);
                                        } else {
                                            $result = array(
                                                "chat_file_id" => false,
                                                "chat_file" => false,
                                                "status" => false,
                                                "message" => label('link_content_could_not_be_read')
                                            );
                                        }
                                    }
                                    addActivityLog('sitemap', $chat_file_id, 'sitemap_submitted');
                                    $result = array(
                                        "chat_file_id" => $chat_file_id,
                                        "chat_file" => URLROOT . '/app' . MEDIA_FOLDER . $file_name,
                                        "status" => true,
                                        "message" => label('sitemap_submitted')
                                    );
                                } else {
                                    $result = array(
                                        "chat_file_id" => false,
                                        "chat_file" => false,
                                        "status" => false,
                                        "message" => label('submitting_sitemap_failed')
                                    );
                                }
                            } else {
                                $result = array(
                                    "chat_file_id" => false,
                                    "chat_file" => false,
                                    "status" => false,
                                    "message" => label('failed_to_upload')
                                );
                            }
                        } else {
                            $result = array(
                                "chat_file_id" => false,
                                "chat_file" => false,
                                "status" => false,
                                "message" => label('xml_sitemap_is_not_valid')
                            );
                        }
                    } else {
                        $result = array(
                            "chat_file_id" => false,
                            "chat_file" => false,
                            "status" => false,
                            "message" => label('only_xml_files_are_allowed')
                        );
                    }
                } else {
                    $result = array(
                        "chat_file_id" => false,
                        "chat_file" => false,
                        "status" => false,
                        "message" => label('error_uploading_file')
                    );
                }
            } else {
                $result = array(
                    "chat_file_id" => false,
                    "chat_file" => false,
                    "status" => false,
                    "message" => label($data['input_error'])
                );
            }
            echo json_encode($result);
        }
        
        
        public function sitemap($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            
            $data = [
                'chat_link' => $chat_link
            ];
            $this->view('links/sitemap', $data);
        }
        
        public function upload_sitemap_for_chat($chat_file_id = NULL)
        {
            $chat_link = chat_file($chat_file_id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            $subscription = has_valid_subscription(user_id());
            
            if (!$subscription) {
                $data['input_error'] = 'you_do_not_have_an_active_subscription';
            }
            if ($subscription && (int)user(user_id())->user_remaining_chat_sitemaps == 0) {
                $data['input_error'] = 'maximum_number_of_chat_sitemaps_in_your_subscription_package_reached';
            }
            if (empty($_POST['chat_sitemap_link'])) {
                $data['input_error'] = 'select_links_to_upload_for_chat';
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                if (empty($data['input_error'])) {
                    foreach ($_POST['chat_sitemap_link'] as $chat_sitemap_link) {
                        $datax = [
                            'chat_sitemap_link_id' => $chat_sitemap_link,
                            'chat_sitemap_link_output_id' => now(),
                            'chat_sitemap_link_output_status' => 'processed'
                        ];
                        $this->chatfileModel->updateChatSitemapLinkOutput($datax);
                        
                        $files['{Website page starts here. Quote source link as : "' . chat_sitemap_link_by_id($chat_sitemap_link)->chat_sitemap_link . '"}'] = chat_sitemap_link_by_id($chat_sitemap_link)->chat_sitemap_file_name;
                    }
                    //unselected
                    foreach (sitemap_links($chat_file_id) as $link) {
                        if (!in_array($link->chat_sitemap_link_id, $_POST['chat_sitemap_link']) && !empty($link->chat_sitemap_link_output_id)) {
                            $this->chatfileModel->cancelChatSitemapLink($link->chat_sitemap_link_id);
                        }
                    }
                    
                    if (!empty($chat_link->chat_file_output_id)) {
                        unlinkUploads('media', $chat_link->chat_filename);
                        ai_delete_file($chat_link->chat_file_output_id);
                    }
                    
                    if ($merged_file = merge_files($files)) {
                        $response = ai_upload_assistant_file(URLROOT . '/app' . MEDIA_FOLDER . $merged_file);
                        if (!empty($response) && $response['status'] == 'error') {
                            $data['input_error'] = $response['message'];
                        }
                        
                        $datax = [
                            'chat_file_id' => $chat_file_id,
                            'chat_filename' => $merged_file,
                            'chat_file_output_id' => $response['file_id'],
                            'chat_file_output_filename' => $response['filename'],
                            'chat_file_output_size' => $response['bytes'],
                            'chat_file_output_status' => $response['file_status']
                        ];
                        $this->chatfileModel->updateChatFileOutput($datax);
                        
                    } else {
                        $data['input_error'] = 'error_submitting_sitemap';
                    }
                    if (empty($data['input_error'])) {
                        addActivityLog('sitemap', $chat_file_id, 'sitemap_submitted');
                        flash('user_message', 'sitemap', 'sitemap_submitted');
                    } else {
                        flash('user_message', 'sitemap', $data['input_error'], 'danger');
                    }
                } else {
                    flash('user_message', 'sitemap', $data['input_error'], 'danger');
                }
            }
            redirect('links/sitemap/' . $chat_file_id);
            die();
        }
        
        public function chat_sitemap($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            redirect('chats/sitemap/' . $id);
            die();
        }
        
        public function delete_sitemap($id = NULL)
        {
            $chat_link = chat_file($id);
            if (empty($chat_link) || $chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->chatfileModel->deleteChatFile($id)) {
                    foreach (sitemap_links($id) as $link) {
                        unlinkUploads('media', $link->chat_sitemap_file_name);
                        if (!empty($link->chat_sitemap_link_output_id)) {
                            ai_delete_file($link->chat_sitemap_link_output_id);
                        }
                        $this->chatfileModel->deleteChatSitemapLink($link->chat_sitemap_link_id);
                        addActivityLog('sitemap', $id, 'sitemap_deleted');
                    }
                    flash('user_message', 'sitemap', 'sitemap_deleted');
                } else {
                    flash('user_message', 'sitemap', 'sitemap_deleting_failed', 'danger');
                }
            }
            redirect('links/sitemaps');
            die();
        }
        
        public function cancel_sitemap_link($id = NULL)
        {
            $chat_sitemap_link = chat_sitemap_link_by_id($id);
            if (empty($chat_sitemap_link)) {
                redirect(404);
                die();
            }
            $chat_link = chat_file($chat_sitemap_link->chat_file_id);
            if ($chat_link->chat_file_added_by != user_id()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->chatfileModel->cancelChatSitemapLink($id)) {
                    if (!empty($chat_sitemap_link->chat_sitemap_link_output_id)) {
                        ai_delete_file($chat_sitemap_link->chat_sitemap_link_output_id);
                    }
                    addActivityLog('file', $id, 'sitemap_link_cancelled');
                    flash('user_message', 'file', 'sitemap_link_cancelled');
                } else {
                    flash('user_message', 'file', 'sitemap_link_cancelling_failed', 'danger');
                }
            }
            redirect('links/sitemap/' . $chat_sitemap_link->chat_file_id);
            die();
        }
        
        
    }