<?php
    
    class Newsletters extends Controller
    {
        public function __construct()
        {
            require APPROOT . '/views/inc/load_models.php';
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
        }
        
        public function index()
        {
            $newsletters = newsletters();
            
            $data = [
                'newsletters' => $newsletters
            ];
            $this->view('newsletters/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $data = [
                    'newsletter_title' => cleanString($_POST['newsletter_title']),
                    'newsletter_slug' => generate_seo_url($_POST['newsletter_title']),
                    'newsletter_description' => cleanString($_POST['newsletter_description'], 'ignore_tags'),
                    'newsletter_sent_at' => isset($_POST['send_to_subscribers']) ? now() : NULL,
                    'newsletter_added_by' => user_id()
                ];
                if (empty($data['newsletter_title'])) {
                    $data['input_error'][] = 'please_enter_newsletter_title';
                }
                if (empty($data['newsletter_description'])) {
                    $data['input_error'][] = 'please_enter_newsletter_description';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->newsletterModel->addNewsletter($data)) {
                        
                        if (!empty($data['newsletter_sent_at'])) {
                            foreach (subscribers(1) as $subscriber) {
                                sendCustomEmail($subscriber->subscriber_email, $data['newsletter_title'], $data['newsletter_description']);
                            }
                        }
                        addActivityLog('newsletter', $id, 'newsletter_added');
                        flash('user_message', 'newsletters', 'newsletter_added');
                        redirect('newsletters/show/' . $id);
                    } else {
                        flash('user_message', 'newsletters', 'adding_newsletter_failed', 'danger');
                        redirect('newsletters/add');
                    }
                    die();
                }
                $this->view('newsletters/add', $data);
            }
            $this->view('newsletters/add');
        }
        
        public function edit($id = NULL)
        {
            $newsletter = $this->newsletterModel->getNewsletterById($id);
            if (empty($newsletter)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'newsletter_id' => $id,
                    'newsletter_title' => cleanString($_POST['newsletter_title']),
                    'newsletter_description' => cleanString($_POST['newsletter_description'], 'ignore_tags'),
                    'newsletter' => $newsletter
                ];
                if (empty($data['newsletter_title'])) {
                    $data['input_error'][] = 'please_enter_newsletter_name';
                }
                if (empty($data['newsletter_description'])) {
                    $data['input_error'][] = 'please_enter_newsletter_description';
                }
                if (empty($data['input_error'])) {
                    if ($this->newsletterModel->updateNewsletter($data)) {
                        addActivityLog('newsletter', $id, 'newsletter_updated');
                        flash('user_message', 'newsletters', 'newsletter_updated');
                    } else {
                        flash('user_message', 'newsletters', 'newsletter_update_failed', 'danger');
                    }
                    redirect('newsletters/show/' . $id);
                    die();
                }
                $this->view('newsletters/edit', $data);
            }
            $data = [
                'newsletter' => $newsletter
            ];
            $this->view('newsletters/edit', $data);
        }
        
        public function send($id = NULL)
        {
            $newsletter = $this->newsletterModel->getNewsletterById($id);
            if (empty($newsletter)) {
                redirect(404);
            }
            $data = [
                'newsletter_id' => $id,
                'newsletter_sent_at' => now(),
                'newsletter' => $newsletter
            ];
            if ($this->newsletterModel->sendNewsletter($data)) {
                foreach (subscribers(1) as $subscriber) {
                    sendCustomEmail($subscriber->subscriber_email, $data['newsletter']->newsletter_title, $data['newsletter']->newsletter_description);
                }
                addActivityLog('newsletter', $id, 'newsletter_sent');
                flash('user_message', 'newsletters', 'newsletter_sent');
            } else {
                flash('user_message', 'newsletters', 'newsletter_sending_failed', 'danger');
            }
            redirect('newsletters/show/' . $id);
            die();
        }
        
        public function show($id = NULL)
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            $newsletter = $this->newsletterModel->getNewsletterById($id);
            if (empty($newsletter)) {
                redirect(404);
            }
            $data = [
                'newsletter' => $newsletter
            ];
            $this->view('newsletters/show', $data);
        }
        
        public function delete($id = NULL)
        {
            $newsletter = $this->newsletterModel->getNewsletterById($id);
            if (empty($newsletter)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->newsletterModel->deleteNewsletter($id)) {
                    addActivityLog('newsletter', $id, 'newsletter_deleted');
                    flash('user_message', 'newsletters', 'newsletter_deleted');
                    redirect('newsletters');
                } else {
                    flash('user_message', 'newsletters', 'newsletter_delete_failed', 'danger');
                    redirect('newsletters/show/' . $id);
                }
                die();
            } else {
                redirect('newsletters');
            }
        }
        
        
    }