<?php
    
    
    class Packages extends Controller
    {
    
        public function __construct()
        {
            if (!isLoggedIn() && isset($_POST['package_id']) && $_POST['package_group']) {
                $_SESSION['selected_package'] = array($_POST['package_id'], $_POST['package_group']);
                redirect('users/login');
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index($package_slug = NULL)
        {
            if (!getOption('enable_homepage')) {
                redirect('users/login');
                die();
            }
            if (!empty($package_slug)) {
                if (in_array($package_slug, reserved_pages())) {
                    if ($package_slug == 'faq') {
                        $data = [
                            'faqs' => faqs()
                        ];
                        $this->view('themes/' . getOption('active_theme') . '/faq', $data);
                    } elseif ($package_slug == 'use-cases') {
                        $data = [
                            'faqs' => faqs()
                        ];
                        $this->view('themes/' . getOption('active_theme') . '/use_cases', $data);
                    }
                } elseif ($package_slug == 'all') {
                    $data = [
                        'packages' => packages()
                    ];
                    if (isLoggedIn() && isSuperAdmin()) {
                        $this->view('packages/all', $data);
                    } elseif (isLoggedIn()) {
                        $this->view('packages/plans', $data);
                    }else{
						$this->view('themes/' . getOption('active_theme') . '/index');
					}
                } else {
                    $page = page_by_slug($package_slug);
                    if (empty($page)) {
                        redirect(404, true);
                    }
                    if ($page->page_status_id != 2) {
                        redirect(404, true);
                    }
                    $data = [
                        'page' => $page,
                        'seo' => true
                    ];
                    $this->view('themes/' . getOption('active_theme') . '/page', $data);
                }
            } else {
                $this->view('themes/' . getOption('active_theme') . '/index');
            }
        }
        
        public function add()
        {
        if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_name' => cleanString($_POST['package_name']),
                    'package_duration' => isset($_POST['package_duration']) ? cleanString($_POST['package_duration']) : NULL,
                    'package_duration_type' => isset($_POST['package_duration_type']) ? cleanString($_POST['package_duration_type']) : NULL,
                    'package_chat_assistants' => cleanString($_POST['package_chat_assistants']),
                    'package_trained_models' => cleanString($_POST['package_trained_models']),
                    'package_chat_files' => cleanString($_POST['package_chat_files']),
                    'package_chat_links' => cleanString($_POST['package_chat_links']),
                    'package_chat_sitemaps' => cleanString($_POST['package_chat_sitemaps']),
                    'package_words' => cleanString($_POST['package_words']),
                    'accept_custom_branding' => isset($_POST['accept_custom_branding']) ? cleanString($_POST['accept_custom_branding']) : 0,
                    'package_cost' => isset($_POST['package_cost']) ? cleanString($_POST['package_cost']) : NULL,
                    'package_monthly_cost' => isset($_POST['package_monthly_cost']) ? cleanString($_POST['package_monthly_cost']) : NULL,
                    'package_yearly_cost' => isset($_POST['package_yearly_cost']) ? cleanString($_POST['package_yearly_cost']) : NULL,
                    'package_lifetime_cost' => isset($_POST['package_lifetime_cost']) ? cleanString($_POST['package_lifetime_cost']) : NULL,
                    'is_custom' => isset($_POST['is_custom']) ? 1 : 0,
                    'is_free_trial' => isset($_POST['is_free_trial']) ? 1 : NULL,
                    'is_private' => isset($_POST['is_private']) ? 1 : NULL
                ];
                if (empty($data['package_name'])) {
                    $data['input_error'][] = 'please_enter_package_name';
                }
                if ($data['is_custom'] && empty($data['package_duration'])) {
                    $data['input_error'][] = 'please_enter_package_duration';
                }
                 if ($data['is_custom'] && empty($data['package_duration_type'])) {
                    $data['input_error'][] = 'please_enter_package_duration_type';
                }
                if (($data['is_custom'] && !is_numeric($data['package_cost'])) || (!$data['is_free_trial'] && ($data['package_monthly_cost']+$data['package_yearly_cost']+$data['package_lifetime_cost'])==0)) {
                    $data['input_error'][] = 'please_enter_package_cost';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->packageModel->addPackage($data)) {
                        if (isset($_POST['features'])) {
                            for ($i = 0; $i < count($_POST['features']); $i++) {
                                $dataf = [
                                    'package_id' => $id,
                                    'package_feature_id' => cleanString($_POST['features'][$i])
                                ];
                                $this->packageModel->addPackageItem($dataf);
                            }
                        }
                        addActivityLog('package', $id, 'package_added');
                        flash('user_message', 'packages', 'package_added');
                    } else {
                        flash('user_message', 'packages', 'adding_new_package_failed', 'danger');
                    }
                    redirect('packages/all');
                    die();
                }
                $this->view('packages/add', $data);
            }
            $this->view('packages/add');
        }
        
        public function custom()
        {
            $this->view('packages/custom');
        }
        
        public function edit($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_id' => $id,
                    'package_name' => cleanString($_POST['package_name']),
                    'package_duration' => isset($_POST['package_duration']) ? cleanString($_POST['package_duration']) : NULL,
                    'package_duration_type' => isset($_POST['package_duration_type']) ? cleanString($_POST['package_duration_type']) : NULL,
                    'package_chat_assistants' => cleanString($_POST['package_chat_assistants']),
                    'package_trained_models' => cleanString($_POST['package_trained_models']),
                    'package_chat_files' => cleanString($_POST['package_chat_files']),
                    'package_chat_links' => cleanString($_POST['package_chat_links']),
                    'package_chat_sitemaps' => cleanString($_POST['package_chat_sitemaps']),
                    'package_words' => cleanString($_POST['package_words']),
                    'accept_custom_branding' => isset($_POST['accept_custom_branding']) ? cleanString($_POST['accept_custom_branding']) : 0,
                    'package_cost' => isset($_POST['package_cost']) ? cleanString($_POST['package_cost']) : NULL,
                    'package_monthly_cost' => isset($_POST['package_monthly_cost']) ? cleanString($_POST['package_monthly_cost']) : NULL,
                    'package_yearly_cost' => isset($_POST['package_yearly_cost']) ? cleanString($_POST['package_yearly_cost']) : NULL,
                    'package_lifetime_cost' => isset($_POST['package_lifetime_cost']) ? cleanString($_POST['package_lifetime_cost']) : NULL,
                    'is_custom' => isset($_POST['is_custom']) ? 1 : 0,
                    'is_free_trial' => isset($_POST['is_free_trial']) ? 1 : NULL,
                    'is_private' => isset($_POST['is_private']) ? 1 : NULL,
                    'package' => $package
                ];
                if (empty($data['package_name'])) {
                    $data['input_error'][] = 'please_enter_package_name';
                }
                if ($data['is_custom'] && empty($data['package_duration'])) {
                    $data['input_error'][] = 'please_enter_package_duration';
                }
                 if ($data['is_custom'] && empty($data['package_duration_type'])) {
                    $data['input_error'][] = 'please_enter_package_duration_type';
                }
                
                if (($data['is_custom'] && !is_numeric($data['package_cost'])) || (!$data['is_free_trial'] && ($data['package_monthly_cost']+$data['package_yearly_cost']+$data['package_lifetime_cost'])==0)) {
                    $data['input_error'][] = 'please_enter_package_cost';
                }
                if (empty($data['input_error'])) {
                    if ($this->packageModel->updatePackage($data)) {
                        addActivityLog('package', $id, 'package_updated');
                        flash('user_message', 'packages', 'package_updated');
                    } else {
                        flash('user_message', 'packages', 'updating_package_failed', 'danger');
                    }
                    redirect('packages/show/' . $id);
                    die();
                }
                $this->view('packages/edit', $data);
            }
            $data = [
                'package' => $package
            ];
            $this->view('packages/edit', $data);
        }
        
        public function users($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_id' => $id,
                    'users' => $_POST['users'],
                    'package' => $package
                ];
                if (empty($data['users'])) {
                    $data['input_error'][] = 'please_select_private_package_users';
                }
                if (empty($data['input_error'])) {
                	
                	$common = array_intersect($data['users'], private_package_users($data['package_id']));
                	$new = array_diff($data['users'], private_package_users($data['package_id']));
                	$remove = array_diff(private_package_users($data['package_id']), $data['users']);
					
                	for($j = 0; $j < count($remove); $j++){
		                $dataR = [
		                    'package_id' => $id,
		                    'user_id' => $remove[$j]
		                ];
		                $this->packageModel->deletePrivatePackageUser($dataR);
					}
					
                	$success = FALSE;
                	for($i = 0; $i < count($new); $i++){
		                $dataN = [
		                    'package_id' => $id,
		                    'user_id' => $new[$i]
		                ];
		                $this->packageModel->addPrivatePackageUser($dataN);
						$success = TRUE;
					}
                    if ($success) {
                        addActivityLog('package', $id, 'package_updated');
                        flash('user_message', 'packages', 'package_updated');
                    } else {
                        flash('user_message', 'packages', 'updating_package_failed', 'danger');
                    }
                    redirect('packages/users/' . $id);
                    die();
                }
                $this->view('packages/users', $data);
            }
            $data = [
                'package' => $package
            ];
            $this->view('packages/users', $data);
        }
        
        public function make_package_featured($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'package_id' => $id,
                    'package_featured' => '1'
                ];
                if ($this->packageModel->updatePackageFeatured($data)) {
                    addActivityLog('plan', $id, 'package_made_featured');
                    flash('user_message', 'plans', 'package_made_featured');
                } else {
                    flash('user_message', 'plan', 'making_package_featured_failed', 'danger');
                }
            }
            redirect('packages/all');
            die();
        }
        
        public function make_package_not_featured($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'package_id' => $id,
                    'package_featured' => '0'
                ];
                if ($this->packageModel->updatePackageFeatured($data)) {
                    addActivityLog('plan', $id, 'package_made_not_featured');
                    flash('user_message', 'plans', 'package_made_not_featured');
                } else {
                    flash('user_message', 'plan', 'making_package_not_featured_failed', 'danger');
                }
            }
            redirect('packages/all');
            die();
        }
        
        public function show($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            $data = [
                'package' => $package
            ];
            $this->view('packages/show', $data);
        }
        
        public function delete($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if ($this->packageModel->deletePackage($id)) {
                    addActivityLog('package', $id, 'package_deleted');
                    flash('user_message', 'packages', 'package_deleted');
                } else {
                    flash('user_message', 'package', 'package_delete_failed', 'danger');
                }
            }
            redirect('packages/all');
            die();
        }
        
        public function features($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $data = [
                'package_features' => package_features()
            ];
            $this->view('packages/features', $data);
        }
        
        public function add_package_item()
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_id' => cleanString($_POST['package_id']),
                    'package_feature_id' => cleanString($_POST['package_feature_id'])
                ];
                if (empty($data['input_error'])) {
                    if ($id = $this->packageModel->addPackageItem($data)) {
                        addActivityLog('package', $id, 'package_item_added');
                        $status = 'success';
                        $message = 'package_item_added';
                    } else {
                        $status = 'error';
                        $message = 'adding_package_item_failed';
                    }
                }
            }
            
            $result = array(
                "status" => $status,
                "message" => $message
            );
            
            echo json_encode($result);
        }
        
        public function delete_package_item()
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_id' => cleanString($_POST['package_id']),
                    'package_feature_id' => cleanString($_POST['package_feature_id'])
                ];
                if (empty($data['input_error'])) {
                    if ($id = $this->packageModel->deletePackageItem($data)) {
                        addActivityLog('package', $id, 'package_item_deleted');
                        $status = 'success';
                        $message = 'package_item_deleted';
                    } else {
                        $status = 'error';
                        $message = 'deleting_package_item_failed';
                    }
                }
            }
            
            $result = array(
                "status" => $status,
                "message" => $message
            );
            
            echo json_encode($result);
        }
        
        public function add_feature()
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_feature_description' => cleanString($_POST['package_feature_description'])
                ];
                if (empty($data['package_feature_description'])) {
                    $data['input_error'][] = 'please_enter_package_feature_description';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->packageModel->addPackageFeature($data)) {
                        addActivityLog('package', $id, 'package_feature_added');
                        flash('user_message', 'packages', 'package_feature_added');
                    } else {
                        flash('user_message', 'packages', 'adding_new_package_feature_failed', 'danger');
                    }
                    redirect('packages/features');
                    die();
                }
                $this->view('packages/features', $data);
            }
            $this->view('packages/features');
        }
        
        public function edit_feature($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package_feature = package_feature($id);
            if (empty($package_feature)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'package_feature_id' => $id,
                    'package_feature_description' => cleanString($_POST['package_feature_description'])
                ];
                if (empty($data['package_feature_description'])) {
                    $data['input_error'][] = 'please_enter_package_feature_description';
                }
                if (empty($data['input_error'])) {
                    if ($this->packageModel->updatePackageFeature($data)) {
                        addActivityLog('package_feature', $id, 'package_feature_updated');
                        flash('user_message', 'packages', 'package_feature_updated');
                    } else {
                        flash('user_message', 'packages', 'updating_package_feature_failed', 'danger');
                    }
                    redirect('packages/features');
                    die();
                }
                $this->view('packages/edit_feature', $data);
            }
            $data = [
                'package_feature' => $package_feature
            ];
            $this->view('packages/edit_feature', $data);
        }
        
        public function delete_feature($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package_feature = package_feature($id);
            if (empty($package_feature)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if ($this->packageModel->deletePackageFeature($id)) {
                    addActivityLog('package', $id, 'package_feature_deleted');
                    flash('user_message', 'packages', 'package_feature_deleted');
                } else {
                    flash('user_message', 'package', 'package_feature_delete_failed', 'danger');
                }
            }
            redirect('packages/features');
            die();
        }
        
        public function choose($id = NULL)
        {
            $free_trial_subscription = free_trial_subscription(user_id(), payment_option_slug('free-trial')->payment_option_id);
            $trial_used = FALSE;
            if ($free_trial_subscription) {
                if ($free_trial_subscription->user_id == user_id() && $free_trial_subscription->package_id == $id && $free_trial_subscription->payment_option_id == payment_option_slug('free-trial')->payment_option_id) {
                    $trial_used = TRUE;
                }
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                if ($trial_used) {
                    flash('user_message', 'subscriptions', 'you_have_already_subscribed_to_free_trial_package', 'danger');
                    redirect('subscriptions');
                    die();
                }
                $_POST = sanitize_post();
                $data = [
                    'package_id' => $id,
                    'package_group' => $_POST['package_group']
                ];
                $this->view('packages/choose', $data);
            } elseif (isset($_SESSION['selected_package']) && is_array($_SESSION['selected_package'])) {
                $package = package($id);
                if (empty($package)) {
                    redirect(404);
                }
                $data = [
                    'package_id' => $_SESSION['selected_package'][0],
                    'package_group' => $_SESSION['selected_package'][1]
                ];
            } else {
                $data = [
                    'package_id' => $id,
                    'package_group' => 'months'
                ];
            }
            $this->view('packages/choose', $data);
        }
        
        public function choosen($id = NULL)
        {
            if ($id == 0) {
                unset($id);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if (isset($id)) {
                    $package = package($id);
                }
                
                if (isset($_POST['payment_option'])) {
                    $datax = [
                        'package_id' => $id ?? NULL,
                        'package_group' => isset($_POST['package_group']) ? cleanString($_POST['package_group']) : NULL,
                        'payment_option_id' => payment_option_slug(cleanString($_POST['payment_option']))->payment_option_id
                    ];
                    
                    $_SESSION['custom'] = user_id() . '|' . $datax['package_id'] . '|' . $datax['package_group'];
                    
                    if ($_POST['payment_option'] == "stripe") {
                        $stripe_live_enabled = $this->settingModel->getOptions('stripe_live_enabled');
                        $stripe_settings = $this->settingModel->getStripeSettings($stripe_live_enabled->option_value);
                        $datay = [
                            'stripe_live_enabled' => $stripe_live_enabled,
                            'stripe_settings' => $stripe_settings
                        ];
                        $this->view('gateways/stripe', $data = array_merge($datax, $datay));
                    } elseif (in_array($_POST['payment_option'], array("paypal_standard", "paystack", "instamojo", "hyperpay", "toyyibpay", "pesapal", "iyzico", "tap", "aamarpay","tripay"))) {
                        $getPaymentSettings = str_replace("_", "", "get" . ucfirst($_POST['payment_option']) . "Settings");
                        $live_enabled = $_POST['payment_option'] . "_live_enabled";
                        $out_live_enabled = $this->settingModel->getOptions($live_enabled);
                        $payment_settings = $this->settingModel->$getPaymentSettings($out_live_enabled->option_value);
                        $datay = [
                            $live_enabled => $out_live_enabled,
                            $_POST['payment_option'] . '_settings' => $payment_settings,
                        ];
                        $this->view('gateways/pmt', $data = array_merge($datax, $datay));
                    } elseif (in_array($_POST['payment_option'], array("sslcommerz", "paypal_pro", "twocheckout", "authorize_net", "paygate", "midtrans"))) {
                        $getPaymentSettings = str_replace("_", "", "get" . ucfirst($_POST['payment_option']) . "Settings");
                        $live_enabled = $_POST['payment_option'] . "_live_enabled";
                        $out_live_enabled = $this->settingModel->getOptions($live_enabled);
                        $payment_settings = $this->settingModel->$getPaymentSettings($out_live_enabled->option_value);
                        $datay = [
                            $live_enabled => $out_live_enabled,
                            $_POST['payment_option'] . '_settings' => $payment_settings,
                        ];
                        $this->view('gateways/pmt_form', $data = array_merge($datax, $datay));
                    } elseif ($_POST['payment_option'] == "paytr") {
                        $this->view('gateways/paytr', $datax);
                    } elseif ($_POST['payment_option'] == "flutterwave") {
                        $this->view('gateways/flutterwave', $datax);
                    } elseif ($_POST['payment_option'] == "esewa") {
                        $this->view('gateways/esewa', $datax);
                    } elseif ($_POST['payment_option']=="paddle") {
                        $getPaymentSettings = str_replace("_", "", "get" . ucfirst($_POST['payment_option']) . "Settings");
                        $live_enabled = $_POST['payment_option'] . "_live_enabled";
                        $out_live_enabled = $this->settingModel->getOptions($live_enabled);
                        $payment_settings = $this->settingModel->$getPaymentSettings($out_live_enabled->option_value);
                        
                        if (empty(user(user_id())->email) || empty(user(user_id())->country_id) || empty(user(user_id())->postal_code)) {
                            echo '<div class="alert alert-danger">' . label('either_email_or_country_or_postal_code_are_missing_in_profile') . ' <a href="' . URLROOT . '/users/profile">' . label('click_here_to_update') . '</a></div>';
                            die();
                        }
                        
                        $dataM = [
                            'product_id' => paddle_create_product(package($datax['package_id'])->package_name)['product_id'],
                            'description' => package($datax['package_id'])->package_name,
                            'name' => package($datax['package_id'])->package_name,
                            'amount' => getOption("default_currency") == 'USD' ? package($datax['package_id'])->package_cost * 100 : package($datax['package_id'])->package_cost,
                            'currency_code' => getOption("default_currency")
                        ];
                        
                        $datay = [
                            'price_id' => paddle_create_price($dataM)['status'] == 'success' ? paddle_create_price($dataM)['price_id'] : NULL,
                            $_POST['payment_option'] . '_settings' => $payment_settings,
                        ];
                        $this->view('gateways/' . $_POST['payment_option'], $data = array_merge($datax, $datay));
                    }else {
                        $datay = [
                            'payment_option_id' => payment_option_slug($_POST['payment_option'])->payment_option_id
                        ];
                        $this->view('gateways/cash_on_delivery', $data = array_merge($datax, $datay));
                    }
                }
            } else {
                redirect('subscriptions');
            }
        }
        
        public function subscribe($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'package' => $package
                ];
            }
            $this->view('packages/subscribe', $data);
        }
        
        
        public function record_payment($id = NULL)
        {
            if (!isLoggedIn() || !isSuperAdmin()) {
                redirect(404);
                die();
            }
            $package = package($id);
            if (empty($package)) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $data = [
                    'subscription_cost' => $_POST['amount_added'],
                    'subscription_uid' => uid(),
                    'transaction_id' => $_POST['transaction_id'],
                    'payment_option_id' => $_POST['payment_option_id'],
                    'user_id' => $_POST['user_id'],
                    'subscription_added_by' => user_id(),
                    'transaction_reference' => NULL,
                    'package_id' => $id,
                    'package_duration' => $package->package_duration,
                    'package_duration_type' => $package->package_duration_type,
                    'subscription_items' => json_encode($package)
                ];
                if (empty($data['payment_option_id'])) {
                    $data['input_error'] = 'please_enter_payment_mode';
                }
                if (!is_numeric($data['subscription_cost'])) {
                    $data['input_error'] = 'amount_paid_only_numbers_accepted';
                }
                if (!empty(subscription_payment_by_transaction_id($data['transaction_id']))) {
                    $data['input_error'] = 'unique_number_already_exists';
                }
                if ($data['subscription_cost'] != package($data['package_id'])->package_cost) {
                    $data['input_error'] = 'amount_paid_is_not_equal_to_package_cost';
                }
                if (empty($data['input_error'])) {
	                if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
	                    if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
	                        if (affiliate_accepted($subscription_id)) {
	                            $dataA = [
	                                'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
	                                'affiliate_rule_id' => getOption('affiliate_rule_id'),
	                                'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
	                                'subscription_id' => $subscription_id
	                            ];
	                            $this->affiliateModel->addAffiliate($dataA);
	                        }
	                        $dataw = [
	                            'user_id' => user_id(),
	                            'user_remaining_words' => add_user_remaining_balances(user_id(), $subscription_id, 'words'),
	                            'user_remaining_chat_assistants' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_assistants'),
	                            'user_remaining_trained_models' => add_user_remaining_balances(user_id(), $subscription_id, 'trained_models'),
	                            'user_remaining_chat_files' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_files'),
	                            'user_remaining_chat_links' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_links'),
	                            'user_remaining_chat_sitemaps' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_sitemaps')
	                        ];
	                        foreach (package_types() as $type) {
	                            $out_type = 'user_remaining_' . $type;
	                            $dataz = [
	                                'user_id' => user_id(),
	                                'type' => $type,
	                                'transaction_type' => 'in',
	                                'package_id' => $data['package_id'],
	                                'old' => (int)user(user_id())->$out_type,
	                                'current' => $dataw[$out_type],
	                                'user_package_history_comment' => json_encode(package($data['package_id'])),
	                                'user_package_history_added_by' => user_id()
	                            ];
	                            $this->userModel->addUserPackageHistory($dataz);
	                        }
	                        $this->userModel->editUserRemaining($dataw);
	                    }
	                    sendEmail('SubscriptionPaid', user(user_id())->email, 'subscriptions', $subscription_id, user_id());
	                    addActivityLog('subscription', $subscription_id, 'payment_added');
	                    flash('user_message', 'subscriptions', 'payment_added');
	                } else {
	                    flash('user_message', 'subscriptions', 'adding_payment_failed', 'danger');
	                }
	            }else{
					flash('user_message', 'subscriptions', $data['input_error'], 'danger');
				}
	        }
	        redirect('packages/show/'.$id);
            die();
	    }
    
        public function change_language()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $_SESSION['language'] = $_POST['language'];
            }
            redirect(isset($_SERVER['HTTP_REFERER']) ? referrer_without_urlroot($_SERVER['HTTP_REFERER']) : '');
        }
        
        public function change_theme($theme)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                
                $_SESSION['theme'] = $theme;
            }
            redirect(isset($_SERVER['HTTP_REFERER']) ? referrer_without_urlroot($_SERVER['HTTP_REFERER']) : '');
        }
        
        public function accept_cookies()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                setcookie("accept_cookies", 1, time() + (10 * 365 * 24 * 60 * 60), '/');
            }
            redirect(isset($_SERVER['HTTP_REFERER']) ? referrer_without_urlroot($_SERVER['HTTP_REFERER']) : '');
        }
        
        
    }