<?php
    
    
    class Pages extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'pages' => pages()
            ];
            $this->view('pages/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $page_background = !empty($_FILES['page_background']['name']) ? fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['page_background']['name'])) : NULL;
                
                $data = [
                    'page_title' => cleanString($_POST['page_title']),
                    'page_slug' => generate_seo_url($_POST['page_title']),
                    'page_background' => $page_background,
                    'page_short_description' => !empty($_POST['page_short_description']) && isset($_POST['privacy_policy_page']) ? cleanString($_POST['page_short_description'], 'ignore_tags') : NULL,
                    'page_description' => cleanString($_POST['page_description'], 'ignore_tags'),
                    'show_in_header' => isset($_POST['show_in_header']) ? cleanString($_POST['show_in_header']) : 0,
                    'show_in_footer' => isset($_POST['show_in_footer']) ? cleanString($_POST['show_in_footer']) : 0,
                    'page_status_id' => !empty($_POST['page_status_id']) ? cleanString($_POST['page_status_id']) : 1,
                    'page_added_by' => user_id()
                ];
                if (empty($data['page_title'])) {
                    $data['input_error'][] = 'please_enter_page_title';
                }
                if (page_exists($data['page_title'])) {
                    $data['input_error'][] = 'page_name_already_exists';
                }
                if (empty($data['page_description'])) {
                    $data['input_error'][] = 'please_enter_page_description';
                }
                if (!empty($page_background)) {
                    upload_image($data['page_background'], 'media', $_FILES['page_background']['tmp_name']);
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->pageModel->addPage($data)) {
                        $dataP = [
                            'privacy_policy_page' => isset($_POST['privacy_policy_page']) ? $id : "",
                            'terms_and_conditions_page' => isset($_POST['terms_and_conditions_page']) ? $id : "",
                            'contact_us_page' => isset($_POST['contact_us_page']) ? $id : "",
                            'show_contact_us_form' => isset($_POST['show_contact_us_form']) ? 1 : 0,
                            'show_contact_us_details' => isset($_POST['show_contact_us_details']) ? 1 : 0,
                            'contact_us_email' => cleanString($_POST['contact_us_email']),
                            'contact_us_phone' => cleanString($_POST['contact_us_phone']),
                            'contact_us_address' => cleanString($_POST['contact_us_address'])
                        ];
                        foreach ($dataP as $keys => $options) {
                            $data = [
                                'option_name' => $keys,
                                'option_value' => $options
                            ];
                            $this->settingModel->updateOptions($data);
                        }
                        addActivityLog('page', $id, 'page_added');
                        flash('user_message', 'pages', 'page_added');
                        redirect('pages/show/' . $id);
                    } else {
                        flash('user_message', 'pages', 'adding_page_failed', 'danger');
                        redirect('pages/add');
                    }
                    die();
                }
                $this->view('pages/add', $data);
            }
            $this->view('pages/add');
        }
        
        public function edit($id = NULL)
        {
            $page = $this->pageModel->getPageById($id);
            if (empty($page)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'page_id' => $id,
                    'page_title' => cleanString($_POST['page_title']),
                    'page_background' => $page->page_background,
                    'page_short_description' => !empty($_POST['page_short_description']) && isset($_POST['privacy_policy_page']) ? cleanString($_POST['page_short_description'], 'ignore_tags') : NULL,
                    'page_description' => cleanString($_POST['page_description'], 'ignore_tags'),
                    'show_in_header' => isset($_POST['show_in_header']) ? cleanString($_POST['show_in_header']) : 0,
                    'show_in_footer' => isset($_POST['show_in_footer']) ? cleanString($_POST['show_in_footer']) : 0,
                    'page_status_id' => !empty($_POST['page_status_id']) ? cleanString($_POST['page_status_id']) : 1,
                    'page' => $page
                ];
                if (empty($data['page_title'])) {
                    $data['input_error'][] = 'please_enter_page_name';
                }
                if (empty($data['page_description'])) {
                    $data['input_error'][] = 'please_enter_page_description';
                }
                if (empty($data['input_error'])) {
                    if ($this->pageModel->updatePage($data)) {
                        $dataP = [
                            'privacy_policy_page' => isset($_POST['privacy_policy_page']) ? $id : "",
                            'terms_and_conditions_page' => isset($_POST['terms_and_conditions_page']) ? $id : "",
                            'contact_us_page' => isset($_POST['contact_us_page']) ? $id : "",
                            'show_contact_us_form' => isset($_POST['show_contact_us_form']) ? 1 : 0,
                            'show_contact_us_details' => isset($_POST['show_contact_us_details']) ? 1 : 0,
                            'contact_us_email' => cleanString($_POST['contact_us_email']),
                            'contact_us_phone' => cleanString($_POST['contact_us_phone']),
                            'contact_us_address' => cleanString($_POST['contact_us_address'])
                        ];
                        foreach ($dataP as $keys => $options) {
                            $data = [
                                'option_name' => $keys,
                                'option_value' => $options
                            ];
                            $this->settingModel->updateOptions($data);
                        }
                        addActivityLog('page', $id, 'page_updated');
                        flash('user_message', 'pages', 'page_updated');
                    } else {
                        flash('user_message', 'pages', 'page_update_failed', 'danger');
                    }
                    redirect('pages/show/' . $id);
                    die();
                }
                $this->view('pages/edit', $data);
            }
            $data = [
                'page' => $page
            ];
            $this->view('pages/edit', $data);
        }
        
        public function update_page_background($id = NULL)
        {
            $page = $this->pageModel->getPageById($id);
            if (empty($page)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $page_background = fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['page_background']['name']));
                
                if (upload_image($page_background, 'media', $_FILES['page_background']['tmp_name'])) {
                    $data = [
                        'page_id' => $id,
                        'page_title' => $page->page_title,
                        'page_background' => $page_background,
                        'page_description' => $page->page_description,
                        'show_in_header' => $page->show_in_header,
                        'show_in_footer' => $page->show_in_footer,
                        'page_status_id' => $page->page_status_id,
                        'page' => $page
                    ];
                    $this->pageModel->updatePage($data);
                    unlinkUploads('media', $page->page_background);
                    addActivityLog('user', user_id(), 'page_background_updated');
                    flash('user_message', 'pages', 'page_background_updated');
                } else {
                    flash('user_message', 'pages', 'page_background_update_failed', 'danger');
                }
            }
            redirect('pages/show/' . $id);
            die();
        }
        
        
        public function delete_page_background($id = NULL)
        {
            $page = $this->pageModel->getPageById($id);
            if (empty($page)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->pageModel->deletePageBackground($id)) {
                    unlinkUploads('media', $page->page_background);
                    addActivityLog('page', $id, 'page_background_deleted');
                    flash('user_message', 'pages', 'page_background_deleted');
                } else {
                    flash('user_message', 'pages', 'page_background_delete_failed', 'danger');
                }
            }
            redirect('pages/show/' . $id);
            die();
        }
        
        public function show($id = NULL)
        {
            $page = $this->pageModel->getpageById($id);
            if (empty($page)) {
                redirect(404);
            }
            $data = [
                'page' => $page
            ];
            $this->view('pages/show', $data);
        }
        
        public function delete($id = NULL)
        {
            $page = $this->pageModel->getPageById($id);
            if (empty($page)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->pageModel->deletePage($id)) {
                    addActivityLog('page', $id, 'page_deleted');
                    flash('user_message', 'pages', 'page_deleted');
                    redirect('pages');
                } else {
                    flash('user_message', 'pages', 'page_delete_failed', 'danger');
                    redirect('pages/show/' . $id);
                }
                die();
            } else {
                redirect('pages');
            }
        }
        
        
    }