<?php
    
    
    class Payment_options extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $payment_options = $this->paymentModel->getPaymentOptions(company_id());
            
            $data = [
                'payment_options' => $payment_options
            ];
            
            $this->view('payment_options/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                // Sanitize POST array
                $_POST = sanitize_post();
                $data = [
                    'company_id' => company_id(),
                    'payment_option' => generate_seo_url(trim($_POST['payment_option_name'])),
                    'payment_option_name' => trim($_POST['payment_option_name']),
                    'payment_option_online' => 0,
                    'payment_option_added_by' => user_id(),
                    'payment_option_description' => trim($_POST['payment_option_description'])
                ];
                if (empty($data['payment_option_name'])) {
                    $data['input_error'][] = 'please_enter_payment_option_name';
                } else {
                    if ($this->paymentModel->findPaymentOptionByName($data['payment_option_name'])) {
                        $data['input_error'][] = 'payment_option_already_exists';
                    }
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->paymentModel->addPaymentOption($data)) {
                        addActivityLog('payment_options', $id, 'payment_option_added');
                        flash('user_message', 'payment_option_added');
                    } else {
                        flash('user_message', 'adding_payment_option_failed', 'danger');
                    }
                    redirect('payment_options/show/' . $id);
                    die();
                } else {
                    $this->view('payment_options/add', $data);
                }
            }
            $this->view('payment_options/add');
        }
        
        public function edit($id = NULL)
        {
            $payment_option = $this->paymentModel->getPaymentOptionById($id);
            if (empty($payment_option)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'payment_option_id' => $id,
                    'payment_option_name' => trim($_POST['payment_option_name']),
                    'payment_option_description' => trim($_POST['payment_option_description'])
                ];
                if (empty($data['payment_option_name'])) {
                    $data['input_error'][] = 'please_enter_payment_option';
                }
                if (empty($data['input_error'])) {
                    if ($this->paymentModel->updatePaymentOption($data)) {
                        addActivityLog('payment_option', $id, 'payment_option_updated');
                        flash('user_message', 'payment_options', 'payment_option_updated');
                    } else {
                        flash('user_message', 'payment_options', 'payment_option_update_failed', 'danger');
                    }
                    redirect('payment_options/show/' . $id);
                    die();
                } else {
                    $this->view('payment_options/edit', $data);
                }
            }
            $data = [
                'payment_option' => $payment_option
            ];
            $this->view('payment_options/edit', $data);
        }
        
        public function show($id = NULL)
        {
            $payment_option = $this->paymentModel->getPaymentOptionById($id);
            if (empty($payment_option)) {
                redirect(404);
            }
            $data = [
                'payment_option' => $payment_option
            ];
            $this->view('payment_options/show', $data);
        }
        
        public function delete($id = NULL)
        {
            $payment_option = $this->paymentModel->getPaymentOptionById($id);
            if (empty($payment_option)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if ($this->paymentModel->deletePaymentoption($id)) {
                    addActivityLog('payment_option', $id, 'payment_option_deleted');
                    flash('user_message', 'payment_options', 'payment_option_deleted');
                    redirect('payment_options');
                } else {
                    flash('user_message', 'payment_options', 'payment_option_delete_failed', 'danger');
                    redirect('payment_options/show/' . $id);
                }
                die();
            } else {
                redirect('payment_options');
            }
        }
        
    }