<?php
	require(APPROOT . '/third_party/authorizenet/vendor/autoload.php');
	use net\authorize\api\contract\v1 as AnetAPI;
	use net\authorize\api\controller as AnetController;
    
    class Pmt_authorize_net extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        
        public function index()
        {
            $authorize_net_live_enabled = $this->settingModel->getOptions('authorize_net_live_enabled');
            $authorize_net_settings = $this->settingModel->getAuthorizeNetSettings($authorize_net_live_enabled->option_value);
            
	        if (payment_option_settings('authorize_net')['settings']->authorize_net_status == 0) {
	            $authorize_net_environment = \net\authorize\api\constants\ANetEnvironment::SANDBOX;
	        } else {
	            $authorize_net_environment = \net\authorize\api\constants\ANetEnvironment::PRODUCTION;
	        }

            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $data = [
                    'firstname' => cleanString($_POST['firstname']),
                    'lastname' => cleanString($_POST['lastname']),
                    'email' => cleanString($_POST['email']),
                    'phone' => cleanString($_POST['phone']),
                    'card_number' => cleanString(str_replace(" ", "", $_POST['credit_card'])),
                    'cvv' => cleanString($_POST['cvv']),
                    'expiration_month' => cleanString($_POST['expiration_month']),
                    'expiration_year' => cleanString($_POST['expiration_year']),
                    'package_id' => $_POST['package_id'],
                    'package_group' => $_POST['package_group']
                ];
                
                define("AUTHORIZENET_LOG_FILE", "phplog");

			    /* Create a merchantAuthenticationType object with authentication details
			       retrieved from the constants file */
			    $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
			    $merchantAuthentication->setName($authorize_net_settings->authorize_net_login_id);
			    $merchantAuthentication->setTransactionKey($authorize_net_settings->authorize_net_transaction_key);
			    
			    // Set the transaction's refId
			    $refId = 'ref' . time();

			    // Create the payment data for a credit card
			    $creditCard = new AnetAPI\CreditCardType();
			    $creditCard->setCardNumber($data['card_number']);
			    $creditCard->setExpirationDate($data['expiration_year'].'-'.$data['expiration_month']);
			    $creditCard->setCardCode($data['cvv']);

			    // Add the payment data to a paymentType object
			    $paymentOne = new AnetAPI\PaymentType();
			    $paymentOne->setCreditCard($creditCard);

			    // Create order information
			    $order = new AnetAPI\OrderType();
			    $order->setInvoiceNumber(uid());
			    $order->setDescription(company(1)->company_name);

			    // Set the customer's Bill To address
			    $customerAddress = new AnetAPI\CustomerAddressType();
			    $customerAddress->setFirstName($data['firstname']);
			    $customerAddress->setLastName($data['lastname']);
			    $customerAddress->setCompany(company(1)->company_name);
			    $customerAddress->setAddress(user(user_id())->street);
			    $customerAddress->setCity(user(user_id())->city);
			    $customerAddress->setState(user(user_id())->state);
			    $customerAddress->setZip(user(user_id())->zip_code);
			    $customerAddress->setCountry(country(user(user_id())->country_id)->short_name);

			    // Set the customer's identifying information
			    $customerData = new AnetAPI\CustomerDataType();
			    $customerData->setType("individual");
			    $customerData->setId(uid());
			    $customerData->setEmail($data['email']);

			    // Add values for transaction settings
			    $duplicateWindowSetting = new AnetAPI\SettingType();
			    $duplicateWindowSetting->setSettingName("duplicateWindow");
			    $duplicateWindowSetting->setSettingValue("60");

			    // Create a TransactionRequestType object and add the previous objects to it
			    $transactionRequestType = new AnetAPI\TransactionRequestType();
			    $transactionRequestType->setTransactionType("authCaptureTransaction");
			    $transactionRequestType->setAmount(package_cost($data['package_id'], $data['package_group']));
			    $transactionRequestType->setOrder($order);
			    $transactionRequestType->setPayment($paymentOne);
			    $transactionRequestType->setBillTo($customerAddress);
			    $transactionRequestType->setCustomer($customerData);
			    $transactionRequestType->addToTransactionSettings($duplicateWindowSetting);

			    // Assemble the complete transaction request
			    $request = new AnetAPI\CreateTransactionRequest();
			    $request->setMerchantAuthentication($merchantAuthentication);
			    $request->setRefId($refId);
			    $request->setTransactionRequest($transactionRequestType);

			    // Create the controller and get the response
			    $controller = new AnetController\CreateTransactionController($request);
			    $response = $controller->executeWithApiResponse($authorize_net_environment);

			    if ($response != null) {
			        // Check to see if the API request was successfully received and acted upon
			        if ($response->getMessages()->getResultCode() == "Ok") {
			            // Since the API request was successful, look for a transaction response
			            // and parse it to display the results of authorizing the card
			            $tresponse = $response->getTransactionResponse();
			        
			            if ($tresponse != null && $tresponse->getMessages() != null) {
	                        $data = [
	                            'subscription_cost' => package_cost($data['package_id'], $data['package_group']),
	                            'subscription_uid' => uid(),
	                            'transaction_id' => $tresponse->getTransId(),
	                            'payment_option_id' => payment_option_slug('authorize_net')->payment_option_id,
	                            'subscription_added_by' => user_id(),
	                            'transaction_reference' => NULL,
	                            'package_id' => $data['package_id'],
			                    'package_duration' => package_duration($data['package_id'], $data['package_group'])[0],
			                    'package_duration_type' => package_duration($data['package_id'], $data['package_group'])[1],
	                            'subscription_items' => json_encode(package($data['package_id']))
	                        ];
	                        if (empty($data['input_error'])) {
	                            if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
	                                if (affiliate_accepted($subscription_id)) {
	                                    $dataA = [
	                                        'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
	                                        'affiliate_rule_id' => getOption('affiliate_rule_id'),
	                                        'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
	                                        'subscription_id' => $subscription_id
	                                    ];
	                                    $this->affiliateModel->addAffiliate($dataA);
	                                }
	                                if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
	                                    $dataw = [
	                                        'user_id' => user_id(),
	                                        'user_remaining_words' => add_user_remaining_balances(user_id(), $subscription_id, 'words'),
	                                        'user_remaining_chat_assistants' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_assistants'),
	                                        'user_remaining_trained_models' => add_user_remaining_balances(user_id(), $subscription_id, 'trained_models'),
	                                        'user_remaining_chat_files' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_files'),
	                                        'user_remaining_chat_links' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_links'),
	                                        'user_remaining_chat_sitemaps' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_sitemaps')
	                                    ];
	                                    foreach (package_types() as $type) {
	                                        $out_type = 'user_remaining_' . $type;
	                                        $dataz = [
	                                            'user_id' => user_id(),
	                                            'type' => $type,
	                                            'transaction_type' => 'in',
	                                            'package_id' => $data['package_id'],
	                                            'old' => (int)user(user_id())->$out_type,
	                                            'current' => $dataw[$out_type],
	                                            'user_package_history_comment' => json_encode(package($data['package_id'])),
	                                            'user_package_history_added_by' => user_id()
	                                        ];
	                                        $this->userModel->addUserPackageHistory($dataz);
	                                    }
	                                    $this->userModel->editUserRemaining($dataw);
	                                }
	                                sendEmail('SubscriptionPaid', user(user_id())->email, 'subscriptions', $subscription_id, user_id());
	                                notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
	                                addActivityLog('subscription', $subscription_id, 'authorize_net_payment_added');
	                                flash('user_message', 'subscriptions', 'authorize_net_payment_added');
	                            } else {
	                                flash('user_message', 'subscriptions', 'adding_authorize_net_payment_failed', 'danger');
	                            }
	                        } else {
	                            flash('user_message', 'subscriptions', 'authorize_net_payment_transaction_failed', 'danger');
	                        }
			            } else {
			                flash('user_message', 'subscriptions', 'authorize_net_payment_transaction_failed', 'danger');
			                if ($tresponse->getErrors() != null) {
			                	flash('user_message', 'subscriptions', $tresponse->getErrors()[0]->getErrorText(), 'danger');
			                }
			            }
			            // Or, print errors if the API request wasn't successful
			        } else {
			            flash('user_message', 'subscriptions', 'authorize_net_payment_transaction_failed', 'danger');
			            $tresponse = $response->getTransactionResponse();
			        
			            if ($tresponse != null && $tresponse->getErrors() != null) {
			                flash('user_message', 'subscriptions', $tresponse->getErrors()[0]->getErrorText(), 'danger');
			            } else {
			                flash('user_message', 'subscriptions', $response->getMessages()->getMessage()[0]->getText(), 'danger');
			            }
			        }
			    } else {
			        flash('user_message', 'subscriptions', 'no_response_returned', 'danger');
			    }
            }
            redirect('subscriptions');
            die();
        }
        
        
        
        
	}
