<?php
    
    
    class Pmt_instamojo extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $instamojo_live_enabled = $this->settingModel->getOptions('instamojo_live_enabled');
            $instamojo_settings = $this->settingModel->getInstamojoSettings($instamojo_live_enabled->option_value);
            
            require_once APPROOT . '/third_party/instamojo/vendor/autoload.php';
            $api = new Instamojo\Instamojo($instamojo_settings->instamojo_api_key, $instamojo_settings->instamojo_auth_token, $instamojo_settings->instamojo_url);
            
            try {
                $response = $api->paymentRequestCreate(array(
                    "purpose" => company(1)->company_name,
                    "amount" => package_cost($_POST['package_id'], $_POST['package_group']),
                    "buyer_name" => user(user_id())->firstname . ' ' . user(user_id())->lastname,
                    "send_email" => true,
                    "email" => user(user_id())->email,
                    "phone" => user(user_id())->calling_code . user(user_id())->phone,
                    "redirect_url" => URLROOT . '/pmt_instamojo/instamojo_success'
                ));
                
                header('Location: ' . $response['longurl']);
                exit();
            } catch (Exception $e) {
                print('Error: ' . $e->getMessage());
            }
        }
        
        public function instamojo_success()
        {
            $instamojo_live_enabled = $this->settingModel->getOptions('instamojo_live_enabled');
            $instamojo_settings = $this->settingModel->getInstamojoSettings($instamojo_live_enabled->option_value);

            require_once APPROOT . '/third_party/instamojo/vendor/autoload.php';
            $api = new Instamojo\Instamojo($instamojo_settings->instamojo_api_key, $instamojo_settings->instamojo_auth_token, $instamojo_settings->instamojo_url);
            
            $payid = $_GET["payment_request_id"];
            
            $response = $api->paymentRequestStatus($payid);
            $explode = explode("|", $_SESSION['custom']);
            if (!empty($response['payments'][0]['payment_id']) && $response['payments'][0]['status'] == 'Credit' && $response['payments'][0]['amount'] == package_cost($explode[1], $explode[2])) {
                $data = [
                    'subscription_cost' => $response['payments'][0]['amount'],
                    'subscription_uid' => uid(),
                    'transaction_id' => $response['payments'][0]['payment_id'],
                    'payment_option_id' => payment_option_slug('instamojo')->payment_option_id,
                    'subscription_added_by' => $explode[0],
                    'transaction_reference' => $payid,
                    'package_id' => $explode[1],
                    'package_duration' => package_duration($explode[1], $explode[2])[0],
                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
                    'subscription_items' => json_encode(package($explode[1]))
                ];
                if (empty($data['input_error'])) {
                    if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
                        if (affiliate_accepted($subscription_id)) {
                            $dataA = [
                                'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
                                'affiliate_rule_id' => getOption('affiliate_rule_id'),
                                'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                                'subscription_id' => $subscription_id
                            ];
                            $this->affiliateModel->addAffiliate($dataA);
                        }
                        if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                            $dataw = [
                                'user_id' => $explode[0],
                                'user_remaining_words' => add_user_remaining_balances($explode[0], $subscription_id, 'words'),
                                'user_remaining_chat_assistants' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_assistants'),
                                'user_remaining_trained_models' => add_user_remaining_balances($explode[0], $subscription_id, 'trained_models'),
                                'user_remaining_chat_files' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_files'),
                                'user_remaining_chat_links' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_links'),
                                'user_remaining_chat_sitemaps' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_sitemaps')
                            ];
                            foreach (package_types() as $type) {
                                $out_type = 'user_remaining_' . $type;
                                $dataz = [
                                    'user_id' => $explode[0],
                                    'type' => $type,
                                    'transaction_type' => 'in',
                                    'package_id' => $explode[1],
                                    'old' => (int)user($explode[0])->$out_type,
                                    'current' => $dataw[$out_type],
                                    'user_package_history_comment' => json_encode(package($explode[1])),
                                    'user_package_history_added_by' => $explode[0]
                                ];
                                $this->userModel->addUserPackageHistory($dataz);
                            }
                            $this->userModel->editUserRemaining($dataw);
                        }
                        sendEmail('SubscriptionPaid', user($explode[0])->email, 'subscriptions', $subscription_id, $explode[0]);
                        notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
                        addActivityLog('subscription', $subscription_id, 'instamojo_payment_added');
                        flash('user_message', 'subscriptions', 'instamojo_payment_added');
                    } else {
                        flash('user_message', 'subscriptions', 'adding_instamojo_payment_failed', 'danger');
                    }
                    redirect('subscriptions/show/' . $subscription_id);
                    die();
                } else {
                    redirect('subscriptions');
                }
            } else {
                flash('user_message', 'subscriptions', 'instamojo_payment_transaction_failed', 'danger');
                redirect('subscriptions');
            }
            die();
        }
        
        
    }
