<?php
    
    
    use Iyzipay\Model\Address;
    use Iyzipay\Model\BasketItem;
    use Iyzipay\Model\BasketItemType;
    use Iyzipay\Model\Buyer;
    use Iyzipay\Model\CheckoutForm;
    use Iyzipay\Model\CheckoutFormInitialize;
    use Iyzipay\Model\PaymentGroup;
    use Iyzipay\Options;
    use Iyzipay\Request\CreateCheckoutFormInitializeRequest;
    use Iyzipay\Request\RetrieveCheckoutFormRequest;
    
    class Pmt_iyzico extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $iyzico_live_enabled = $this->settingModel->getOptions('iyzico_live_enabled');
            $iyzico_settings = $this->settingModel->getIyzicoSettings($iyzico_live_enabled->option_value);
            
            $ConversationId = $_SESSION["ConversationId"] = uid();
            
            require_once APPROOT . '/third_party/iyzico/vendor/autoload.php';
            $options = new Options();
            $options->setApiKey($iyzico_settings->iyzico_api_key);
            $options->setSecretKey($iyzico_settings->iyzico_secret_key);
            $options->setBaseUrl($iyzico_settings->iyzico_url);

# create request class
            $request = new CreateCheckoutFormInitializeRequest();
            $request->setLocale(\Iyzipay\Model\Locale::EN);
            $request->setConversationId($ConversationId);
            $request->setPrice(package_cost($_POST['package_id'], $_POST['package_group']));
            $request->setPaidPrice(package_cost($_POST['package_id'], $_POST['package_group']));
            $request->setCurrency(getOption('default_currency'));
            $request->setBasketId(uid());
            $request->setPaymentGroup(PaymentGroup::PRODUCT);
            $request->setCallbackUrl(URLROOT . "/pmt_iyzico/iyzico_callback");
            $request->setEnabledInstallments(array(2, 3, 6, 9));
            
            $buyer = new Buyer();
            $buyer->setId(uid());
            $buyer->setName(user(user_id())->firstname);
            $buyer->setSurname(user(user_id())->lastname);
            $buyer->setGsmNumber(user(user_id())->phone);
            $buyer->setEmail(user(user_id())->email);
            $buyer->setIdentityNumber(uid());
            $buyer->setLastLoginDate(now());
            $buyer->setRegistrationDate(now());
            $buyer->setRegistrationAddress(user(user_id())->street);
            $buyer->setIp(ip_address());
            $buyer->setCity(user(user_id())->city);
            $buyer->setCountry(country(user(user_id())->country_id)->short_name);
            $buyer->setZipCode(user(user_id())->zip_code);
            $request->setBuyer($buyer);
            
            $shippingAddress = new Address();
            $shippingAddress->setContactName(user(user_id())->firstname . " " . user(user_id())->lastname);
            $shippingAddress->setCity(user(user_id())->city);
            $shippingAddress->setCountry(country(user(user_id())->country_id)->short_name);
            $shippingAddress->setAddress(user(user_id())->street);
            $shippingAddress->setZipCode(user(user_id())->zip_code);
            $request->setShippingAddress($shippingAddress);
            
            $billingAddress = new Address();
            $billingAddress->setContactName(user(user_id())->firstname . " " . user(user_id())->lastname);
            $billingAddress->setCity(user(user_id())->city);
            $billingAddress->setCountry(country(user(user_id())->country_id)->short_name);
            $billingAddress->setAddress(user(user_id())->street);
            $billingAddress->setZipCode(user(user_id())->zip_code);
            $request->setBillingAddress($billingAddress);
            
            $basketItems = array();
            $firstBasketItem = new BasketItem();
            $firstBasketItem->setId(uid());
            $firstBasketItem->setName(company('1')->company_name);
            $firstBasketItem->setCategory1(company('1')->company_name);
            $firstBasketItem->setCategory2(company('1')->company_name);
            $firstBasketItem->setItemType(BasketItemType::PHYSICAL);
            $firstBasketItem->setPrice(package_cost($_POST['package_id'], $_POST['package_group']));
            $basketItems[0] = $firstBasketItem;
            $request->setBasketItems($basketItems);

# make request
            $checkoutFormInitialize = CheckoutFormInitialize::create($request, $options);
            if ($checkoutFormInitialize->getStatus() == "success") {
                $_SESSION["iyzico_token"] = $checkoutFormInitialize->getToken();
                echo $checkoutFormInitialize->getCheckoutFormContent();
            }else{
				echo $checkoutFormInitialize->getErrorMessage();
			}
            die();
        }
        
        public function iyzico_callback()
        {
            $iyzico_live_enabled = $this->settingModel->getOptions('iyzico_live_enabled');
            $iyzico_settings = $this->settingModel->getIyzicoSettings($iyzico_live_enabled->option_value);
            require_once APPROOT . '/third_party/iyzico/vendor/autoload.php';
            
            $options = new Options();
            $options->setApiKey($iyzico_settings->iyzico_api_key);
            $options->setSecretKey($iyzico_settings->iyzico_secret_key);
            $options->setBaseUrl($iyzico_settings->iyzico_url);
# create request class
            $request = new RetrieveCheckoutFormRequest();
            $request->setLocale(\Iyzipay\Model\Locale::TR);
            $request->setConversationId($_SESSION["ConversationId"]);
            $request->setToken($_SESSION['iyzico_token']);

# make request
            $checkoutForm = CheckoutForm::retrieve($request, $options);
            $result = json_decode($checkoutForm->getRawResult());
# print result
            $explode = explode("|", $_SESSION['custom']);
            if ($result->paymentId && $result->price == package_cost($explode[1], $explode[2])) {
                $data = [
                    'subscription_cost' => $result->price,
                    'subscription_uid' => uid(),
                    'transaction_id' => $result->paymentId,
                    'payment_option_id' => payment_option_slug('iyzico')->payment_option_id,
                    'subscription_added_by' => $explode[0],
                    'transaction_reference' => $result->conversationId,
                    'package_id' => $explode[1],
                    'package_duration' => package_duration($explode[1], $explode[2])[0],
                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
                    'subscription_items' => json_encode(package($explode[1]))
                ];
                if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
                    if (affiliate_accepted($subscription_id)) {
                        $dataA = [
                            'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
                            'affiliate_rule_id' => getOption('affiliate_rule_id'),
                            'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                            'subscription_id' => $subscription_id
                        ];
                        $this->affiliateModel->addAffiliate($dataA);
                    }
                    if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                        $dataw = [
                            'user_id' => $explode[0],
                            'user_remaining_words' => add_user_remaining_balances($explode[0], $subscription_id, 'words'),
                            'user_remaining_chat_assistants' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_assistants'),
                            'user_remaining_trained_models' => add_user_remaining_balances($explode[0], $subscription_id, 'trained_models'),
                            'user_remaining_chat_files' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_files'),
                            'user_remaining_chat_links' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_links'),
                            'user_remaining_chat_sitemaps' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_sitemaps')
                        ];
                        foreach (package_types() as $type) {
                            $out_type = 'user_remaining_' . $type;
                            $dataz = [
                                'user_id' => $explode[0],
                                'type' => $type,
                                'transaction_type' => 'in',
                                'package_id' => $explode[1],
                                'old' => (int)user($explode[0])->$out_type,
                                'current' => $dataw[$out_type],
                                'user_package_history_comment' => json_encode(package($explode[1])),
                                'user_package_history_added_by' => $explode[0]
                            ];
                            $this->userModel->addUserPackageHistory($dataz);
                        }
                        $this->userModel->editUserRemaining($dataw);
                    }
                    sendEmail('SubscriptionPaid', user($explode[0])->email, 'subscriptions', $subscription_id, $explode[0]);
                    addActivityLog('subscription', $subscription_id, 'iyzico_payment_added');
                    flash('user_message', 'subscriptions', 'iyzico_payment_added');
                } else {
                    flash('user_message', 'subscriptions', 'adding_iyzico_payment_failed', 'danger');
                }
                redirect('subscriptions');
            } else {
                flash('user_message', 'subscriptions', json_encode($result), 'danger');
        	}
            redirect('subscriptions');
        	die();
        }
        
        public function iyzico_error()
        {
            flash('user_message', 'subscriptions', 'adding_iyzico_payment_failed', 'danger');
            redirect('subscriptions');
            die();
        }
        
    }
