<?php
    
    
    class Pmt_paypal_pro extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $paypal_live_enabled = $this->settingModel->getOptions('paypal_pro_live_enabled');
            $paypal_settings = $this->settingModel->getPaypalProSettings($paypal_live_enabled->option_value);
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $data = [
                    'firstname' => cleanString($_POST['firstname']),
                    'lastname' => cleanString($_POST['lastname']),
                    'email' => cleanString($_POST['email']),
                    'phone' => cleanString($_POST['phone']),
                    'card_number' => cleanString(str_replace(" ", "", $_POST['credit_card'])),
                    'cvv' => cleanString($_POST['cvv']),
                    'expiration_month' => cleanString($_POST['expiration_month']),
                    'expiration_year' => cleanString($_POST['expiration_year']),
                    'package_id' => $_POST['package_id'],
                    'package_group' => $_POST['package_group']
                ];
                require_once APPROOT . '/third_party/paypal/PaypalPro.php';
                $paypal = new PaypalPro;
                $paypalParams = array(
                    'paymentAction' => 'Sale',
                    'amount' => package_cost($data['package_id'], $data['package_group']),
                    'currencyCode' => getOption('default_currency'),
                    'creditCardType' => '',
                    'creditCardNumber' => $data['card_number'],
                    'expMonth' => $data['expiration_month'],
                    'expYear' => $data['expiration_year'],
                    'cvv' => $data['cvv'],
                    'firstName' => $data['firstname'],
                    'lastName' => $data['lastname'],
                    'city' => user(user_id())->city,
                    'zip' => user(user_id())->zip_code,
                    'countryCode' => country(user(user_id())->country_id)->iso2,
                );
                $configOptions = array(
                    'apiUsername' => $paypal_settings->paypal_pro_username,
                    'apiPassword' => $paypal_settings->paypal_pro_password,
                    'apiSignature' => $paypal_settings->paypal_pro_signature,
                    'apiEndpoint' => $paypal_settings->paypal_pro_endpoint,
                    'subject' => '',
                    'authToken' => '',
                    'authSignature' => '',
                    'authTimestamp' => '',
                    'useProxy' => FALSE,
                    'proxyHost' => '127.0.0.1',
                    'proxyPort' => 808,
                    'paypalURL' => $paypal_settings->paypal_pro_url,
                    'version' => '65.1',
                    'ackSuccess' => 'SUCCESS',
                    'ackSuccessWarning' => 'SUCCESSWITHWARNING',
                );
                $response = $paypal->paypalCall($paypalParams, $configOptions);
                $paymentStatus = strtoupper($response["ACK"]);
                if ($paymentStatus == "SUCCESS" && $response['AMT'] == package_cost($data['package_id'], $data['package_group'])) {
                    $data = [
                        'subscription_cost' => $response['AMT'],
                        'subscription_uid' => uid(),
                        'transaction_id' => $response['TRANSACTIONID'],
                        'payment_option_id' => payment_option_slug('paypal_pro')->payment_option_id,
                        'subscription_added_by' => user_id(),
                        'transaction_reference' => NULL,
                        'package_id' => $data['package_id'],
	                    'package_duration' => package_duration($data['package_id'], $data['package_group'])[0],
	                    'package_duration_type' => package_duration($data['package_id'], $data['package_group'])[1],
                        'subscription_items' => json_encode(package($data['package_id']))
                    ];
                    if (empty($data['input_error'])) {
                        if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
                            if (affiliate_accepted($subscription_id)) {
                                $dataA = [
                                    'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
                                    'affiliate_rule_id' => getOption('affiliate_rule_id'),
                                    'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                                    'subscription_id' => $subscription_id
                                ];
                                $this->affiliateModel->addAffiliate($dataA);
                            }
                            if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                                $dataw = [
                                    'user_id' => user_id(),
                                    'user_remaining_words' => add_user_remaining_balances(user_id(), $subscription_id, 'words'),
                                    'user_remaining_chat_assistants' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_assistants'),
                                    'user_remaining_trained_models' => add_user_remaining_balances(user_id(), $subscription_id, 'trained_models'),
                                    'user_remaining_chat_files' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_files'),
                                    'user_remaining_chat_links' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_links'),
                                    'user_remaining_chat_sitemaps' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_sitemaps')
                                ];
                                foreach (package_types() as $type) {
                                    $out_type = 'user_remaining_' . $type;
                                    $dataz = [
                                        'user_id' => user_id(),
                                        'type' => $type,
                                        'transaction_type' => 'in',
                                        'package_id' => $data['package_id'],
                                        'old' => (int)user(user_id())->$out_type,
                                        'current' => $dataw[$out_type],
                                        'user_package_history_comment' => json_encode(package($data['package_id'])),
                                        'user_package_history_added_by' => user_id()
                                    ];
                                    $this->userModel->addUserPackageHistory($dataz);
                                }
                                $this->userModel->editUserRemaining($dataw);
                            }
                            sendEmail('SubscriptionPaid', user(user_id())->email, 'subscriptions', $subscription_id, user_id());
                            notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
                            addActivityLog('subscription', $subscription_id, 'paypal_pro_payment_added');
                            flash('user_message', 'subscriptions', 'paypal_pro_payment_added');
                        } else {
                            flash('user_message', 'subscriptions', 'adding_paypal_pro_payment_failed', 'danger');
                        }
                        redirect('subscriptions/show/' . $subscription_id);
                        die();
                    } else {
                        redirect('subscriptions');
                    }
                } else {
                    flash('user_message', 'subscriptions', 'paypal_pro_payment_transaction_failed', 'danger');
                    redirect('subscriptions');
                }
                die();
            }
        }
        
        
    }
