<?php
    
    
    class Pmt_paystack extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $paystack_live_enabled = $this->settingModel->getOptions('paystack_live_enabled');
            $paystack_settings = $this->settingModel->getPaystackSettings($paystack_live_enabled->option_value);

            $curl = curl_init();
            
            $email = user(user_id())->email;
            $amount = package_cost($_POST['package_id'], $_POST['package_group']) * 100;  //the amount in kobo. This value is actually NGN 300

// url to go to after payment
            $callback_url = URLROOT . '/pmt_paystack/paystack_callback';
            
            curl_setopt_array($curl, array(
                CURLOPT_URL => $paystack_settings->paystack_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_CUSTOMREQUEST => "POST",
                CURLOPT_POSTFIELDS => json_encode([
                    'amount' => $amount,
                    'email' => $email,
                    'callback_url' => $callback_url
                ]),
                CURLOPT_HTTPHEADER => [
                    "authorization: Bearer " . $paystack_settings->paystack_secret_key, //replace this with your own secret key
                    "content-type: application/json",
                    "cache-control: no-cache"
                ],
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            if ($err) {
                // there was an error contacting the Paystack API
                die('Curl returned error: ' . $err);
            }
            
            $tranx = json_decode($response, true);
            
            if (!$tranx['status']) {
                // there was an error from the API
                print_r('API returned error: ' . $tranx['message']);
            }
            
            header('Location: ' . $tranx['data']['authorization_url']);
            die();
            
        }
        
        public function paystack_callback()
        {
            $paystack_live_enabled = $this->settingModel->getOptions('paystack_live_enabled');
            $paystack_settings = $this->settingModel->getPaystackSettings($paystack_live_enabled->option_value);

            $curl = curl_init();
            $reference = $_GET['reference'] ?? '';
            if (!$reference) {
                die('No reference supplied');
            }
            
            curl_setopt_array($curl, array(
                CURLOPT_URL => $paystack_settings->paystack_verification_url . rawurlencode($reference),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_HTTPHEADER => [
                    "accept: application/json",
                    "authorization: Bearer $paystack_settings->paystack_secret_key",
                    "cache-control: no-cache"
                ],
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            if ($err) {
                // there was an error contacting the Paystack API
                die('Curl returned error: ' . $err);
            }
            
            $tranx = json_decode($response);
            
            if (!$tranx->status) {
                // there was an error from the API
                die('API returned error: ' . $tranx->message);
            }
            
            $explode = explode("|", $_SESSION['custom']);
            if ('success' == $tranx->data->status && ($tranx->data->amount / 100) == package_cost($explode[1], $explode[2])) {
                $data = [
                    'subscription_cost' => $tranx->data->amount / 100,
                    'subscription_uid' => uid(),
                    'transaction_id' => $tranx->data->reference,
                    'payment_option_id' => payment_option_slug('paystack')->payment_option_id,
                    'subscription_added_by' => $explode[0],
                    'transaction_reference' => NULL,
                    'package_id' => $explode[1],
                    'package_duration' => package_duration($explode[1], $explode[2])[0],
                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
                    'subscription_items' => json_encode(package($explode[1]))
                ];
                if (empty($data['input_error'])) {
                    if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
                        if (affiliate_accepted($subscription_id)) {
                            $dataA = [
                                'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
                                'affiliate_rule_id' => getOption('affiliate_rule_id'),
                                'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                                'subscription_id' => $subscription_id
                            ];
                            $this->affiliateModel->addAffiliate($dataA);
                        }
                        if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                            $dataw = [
                                'user_id' => $explode[0],
                                'user_remaining_words' => add_user_remaining_balances($explode[0], $subscription_id, 'words'),
                                'user_remaining_chat_assistants' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_assistants'),
                                'user_remaining_trained_models' => add_user_remaining_balances($explode[0], $subscription_id, 'trained_models'),
                                'user_remaining_chat_files' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_files'),
                                'user_remaining_chat_links' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_links'),
                                'user_remaining_chat_sitemaps' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_sitemaps')
                            ];
                            foreach (package_types() as $type) {
                                $out_type = 'user_remaining_' . $type;
                                $dataz = [
                                    'user_id' => $explode[0],
                                    'type' => $type,
                                    'transaction_type' => 'in',
                                    'package_id' => $explode[1],
                                    'old' => (int)user($explode[0])->$out_type,
                                    'current' => $dataw[$out_type],
                                    'user_package_history_comment' => json_encode(package($explode[1])),
                                    'user_package_history_added_by' => $explode[0]
                                ];
                                $this->userModel->addUserPackageHistory($dataz);
                            }
                            $this->userModel->editUserRemaining($dataw);
                        }
                        sendEmail('SubscriptionPaid', user($explode[0])->email, 'subscriptions', $subscription_id, $explode[0]);
                        addActivityLog('subscription', $subscription_id, 'paystack_payment_added');
                        flash('user_message', 'subscriptions', 'paystack_payment_added');
                    } else {
                        flash('user_message', 'subscriptions', 'adding_paystack_payment_failed', 'danger');
                    }
                    redirect('subscriptions/show/' . $subscription_id);
                    die();
                } else {
                    redirect('subscriptions');
                }
            } else {
                flash('user_message', 'subscriptions', 'paystack_payment_transaction_failed', 'danger');
                redirect('subscriptions');
            }
            die();
        }
        
    }
