<?php
    
    
class Pmt_stripe extends Controller{
	public function __construct(){
		if(!isLoggedIn()){
			logout();
		}
		require APPROOT . '/views/inc/load_models.php';
	}
        
	public function stripe_payment_intent(){
		$stripe_live_enabled = $this->settingModel->getOptions('stripe_live_enabled');
		$stripe_settings = $this->settingModel->getStripeSettings($stripe_live_enabled->option_value);
            
		require_once APPROOT . '/third_party/stripe/vendor/autoload.php';
	        
		\Stripe\Stripe::setApiKey($stripe_settings->stripe_secret_key);
			
		header('Content-Type: application/json');
			
		$jsonStr = file_get_contents('php://input');
		$jsonObj = json_decode($jsonStr);
		
		$explode = explode("|", $_SESSION['custom']);
			
		$accepted_currencies_array = array_map('trim', explode(',', payment_option_slug('stripe')->payment_option_zero_decimal_currencies));
		if(in_array(getOption('default_currency'), $accepted_currencies_array)){
			$amount_in_cents = package_cost($explode[1], $explode[2]);
		} else{
			$amount_in_cents = package_cost($explode[1], $explode[2]) * 100;
		}

		try{ 
			// Create PaymentIntent with amount, currency and description
			$paymentIntent = \Stripe\PaymentIntent::create([ 
					'amount' => $amount_in_cents,
					'currency' => getOption('default_currency'), 
					'description' => company(1)->company_name, 
					'automatic_payment_methods' => ['enabled' => true]
				]);
			    
			$output = [ 
				'paymentIntentId' => $paymentIntent->id, 
				'clientSecret' => $paymentIntent->client_secret 
			];
			    
			echo json_encode($output); 
		} catch(Error $e){
			http_response_code(500); 
			echo json_encode(['error' => $e->getMessage()]); 
		}
	}
        
	public function stripe_customer(){
		$stripe_live_enabled = $this->settingModel->getOptions('stripe_live_enabled');
		$stripe_settings = $this->settingModel->getStripeSettings($stripe_live_enabled->option_value);
            
		require_once APPROOT . '/third_party/stripe/vendor/autoload.php';
	        
		\Stripe\Stripe::setApiKey($stripe_settings->stripe_secret_key);
			
		header('Content-Type: application/json');
			
		$jsonStr = file_get_contents('php://input');
		$jsonObj = json_decode($jsonStr);
			
		$payment_intent_id = !empty($jsonObj->payment_intent_id)?$jsonObj->payment_intent_id:''; 
		$fullname = !empty($jsonObj->fullname)?$jsonObj->fullname:''; 
		$email = !empty($jsonObj->email)?$jsonObj->email:''; 
			    
		// Add new customer fullname and email to stripe 
		try{   
			$customer = \Stripe\Customer::create(array(  
					'name' => $fullname,  
					'email' => $email 
				));  
		}catch(Exception $e){   
			$error = $e->getMessage();
		} 
			    
		if(empty($error) && !empty($customer)){
			try{
				// Attach Customer Data with PaymentIntent using customer ID
				\Stripe\PaymentIntent::update($payment_intent_id, [
						'customer' => $customer->id 
					]);
			} catch(Exception $e){  
				$error = $e->getMessage();
			}
			$output = [
				'customer_id' => $customer->id 
			];
			echo json_encode($output); 

		}else{ 
			http_response_code(500);
			echo json_encode(['error' => $error]); 
		}
	}
        
	public function stripe_payment_success(){
		$stripe_live_enabled = $this->settingModel->getOptions('stripe_live_enabled');
		$stripe_settings = $this->settingModel->getStripeSettings($stripe_live_enabled->option_value);
            
		require_once APPROOT . '/third_party/stripe/vendor/autoload.php';
	        
		\Stripe\Stripe::setApiKey($stripe_settings->stripe_secret_key);
			
		header('Content-Type: application/json');
			
		$jsonStr = file_get_contents('php://input');
		$jsonObj = json_decode($jsonStr);
			
		$payment = !empty($jsonObj->payment_intent)?$jsonObj->payment_intent:''; 
		$customer_id = !empty($jsonObj->customer_id)?$jsonObj->customer_id:''; 
		
		// Retrieve customer information from stripe
		try{
			$customerData = \Stripe\Customer::retrieve($customer_id);  
		}catch(Exception $e){ 
			$error = $e->getMessage(); 
		}
		if(empty($error)){
			// If transaction was successful
			if(!empty($payment) && $payment->status == 'succeeded'){
				$output = [ 
					'transaction_id' => $transaction_id
				];
				echo json_encode($output); 
			}else{ 
				http_response_code(500); 
				echo json_encode(['error' => 'Transaction has been failed!']); 
			} 
		}else{ 
			http_response_code(500);
			echo json_encode(['error' => $error]); 
		}
	}
        
	public function stripe_payment_status(){
		$stripe_live_enabled = $this->settingModel->getOptions('stripe_live_enabled');
		$stripe_settings = $this->settingModel->getStripeSettings($stripe_live_enabled->option_value);
            
		require_once APPROOT . '/third_party/stripe/vendor/autoload.php';
	        
		\Stripe\Stripe::setApiKey($stripe_settings->stripe_secret_key);

		// Get the parameters from the URL
		$customer_id = $_GET['customer_id'] ?? null;
		$payment_intent_id = $_GET['payment_intent'] ?? null;
		$client_secret = $_GET['payment_intent_client_secret'] ?? null;
		$redirect_status = $_GET['redirect_status'] ?? null;

		if($customer_id && $payment_intent_id && $client_secret && $redirect_status){
			try{
				// Retrieve the PaymentIntent to confirm its status
				$payment_intent = \Stripe\PaymentIntent::retrieve($payment_intent_id);
        
				// Check if the payment was successful
				if($payment_intent->status == 'succeeded'){
					$transaction_id = $payment_intent->id;
					$accepted_currencies_array = array_map('trim', explode(',', payment_option_slug('stripe')->payment_option_zero_decimal_currencies));
					if(in_array(getOption('default_currency'), $accepted_currencies_array)){
						$amount = $payment_intent->amount;
					} else{
						$amount = $payment_intent->amount / 100;
					}
					$currency = $payment_intent->currency;
			        
					$explode = explode("|", $_SESSION['custom']);
					$data = [
						'subscription_cost' => $amount,
						'subscription_uid' => uid(),
						'transaction_id' => $transaction_id,
						'payment_option_id' => payment_option_slug('stripe')->payment_option_id,
						'subscription_added_by' => user_id(),
						'transaction_reference' => NULL,
						'package_id' => $explode[1],
	                    'package_duration' => package_duration($explode[1], $explode[2])[0],
	                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
						'subscription_items' => json_encode(package($explode[1]))
					];
					if(!empty($data['transaction_id'])){
						if($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)){
							if(affiliate_accepted($subscription_id)){
								$dataA = [
									'user_id' => user(user_id())->referred_by,
									'affiliate_rule_id' => getOption('affiliate_rule_id'),
									'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
									'subscription_id' => $subscription_id
								];
								$this->affiliateModel->addAffiliate($dataA);
							}
							if(getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1){
								$dataw = [
									'user_id' => user_id(),
									'user_remaining_words' => add_user_remaining_balances(user_id(), $subscription_id, 'words'),
									'user_remaining_chat_assistants' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_assistants'),
									'user_remaining_trained_models' => add_user_remaining_balances(user_id(), $subscription_id, 'trained_models'),
									'user_remaining_chat_files' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_files'),
									'user_remaining_chat_links' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_links'),
									'user_remaining_chat_sitemaps' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_sitemaps')
								];
								foreach(package_types() as $type){
									$out_type = 'user_remaining_' . $type;
									$dataz = [
										'user_id' => user_id(),
										'type' => $type,
										'transaction_type' => 'in',
										'package_id' => $explode[1],
										'old' => (int)user(user_id())->$out_type,
										'current' => $dataw[$out_type],
										'user_package_history_comment' => json_encode(package($explode[1])),
										'user_package_history_added_by' => user_id()
									];
									$this->userModel->addUserPackageHistory($dataz);
								}
								$this->userModel->editUserRemaining($dataw);
							}
							sendEmail('SubscriptionPaid', user(user_id())->email, 'subscriptions', $subscription_id, user_id());
							addActivityLog('subscription', $subscription_id, 'stripe_payment_added');
							notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
							flash('user_message', 'subscriptions', 'stripe_payment_added');
							redirect('subscriptions/show/' . $subscription_id);
						} else{
							flash('user_message', 'subscriptions', 'adding_stripe_payment_failed', 'danger');
							redirect('subscriptions');
						}
					} else{
						flash('user_message', 'subscriptions', 'stripe_payment_transaction_failed', 'danger');
						redirect('subscriptions');
					}
        	
				} else{
					//echo "Payment was not successful. Status: " . $payment_intent->status;
					flash('user_message', 'subscriptions', 'adding_stripe_payment_failed', 'danger');
					redirect('subscriptions');
				}
			} catch(\Stripe\Exception\ApiErrorException $e){
					flash('user_message', 'subscriptions', 'adding_stripe_payment_failed'.' : '.$e->getMessage(), 'danger');
					redirect('subscriptions');
			}
		} else{
			//echo 'Missing required parameters.';
			flash('user_message', 'subscriptions', 'adding_stripe_payment_failed', 'danger');
			redirect('subscriptions');
		}
		die();


	}
}
