<?php
    
    
    class Pmt_tap extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
			$tap_live_enabled = $this->settingModel->getOptions('tap_live_enabled');
			$tap_settings     = $this->settingModel->getTapSettings($tap_live_enabled->option_value);
			
			$uid = uid();
			
			$explode = explode("|", $_SESSION['custom']);
			
			$post= [
				'amount' => package_cost($explode[1], $explode[2]),
				'currency' => getOption('default_currency'),
				'customer_initiated' => true,
				'threeDSecure' => true,
				'save_card' => false,
				'description' => package($explode[1])->package_name,
				'metadata' => [
					'udf1' => 'Metadata 1'
				],
				'reference' => [
					'transaction' => 'txn_'.$uid,
					'order' => 'ord_'.$uid
				],
				'receipt' => [
					'email' => true,
					'sms' => true
				],
				'customer' => [
					'first_name' => user(user_id())->firstname,
					'middle_name' => NULL,
					'last_name' => user(user_id())->lastname,
					'email' => user(user_id())->email,
					'phone' => [
						'country_code' => user(user_id())->calling_code,
						'number' => user(user_id())->phone
					]
				],
				'merchant' => [
					'id' => NULL
				],
				'source' => [
					'id' => 'src_all'
				],
				'post' => [
					'url' => URLROOT . '/listeners/tap_listener'
				],
				'redirect' => [
					'url' => URLROOT . '/pmt_tap/tap_success'
				]
			];
			
			$curl = curl_init();

			curl_setopt_array($curl, [
					CURLOPT_URL => $tap_settings->tap_url."/v2/charges/",
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => "",
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 30,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST => "POST",
					CURLOPT_POSTFIELDS => json_encode($post),
					CURLOPT_HTTPHEADER => [
						"Authorization: Bearer ".$tap_settings->tap_secret_key,
						"accept: application/json",
						"content-type: application/json"
					],
				]);

			$response = curl_exec($curl);
			$err      = curl_error($curl);

			curl_close($curl);

			if($err || empty($response))
			{
				flash('user_message', 'subscriptions', "cURL Error #:" . $err, 'danger');
				redirect('subscriptions');
			}
			elseif(!empty($response) && !empty(json_decode($response)->id))
			{
				$_SESSION['transaction_id'] = json_decode($response)->id;
				header("Location: ".json_decode($response)->transaction->url);
				die();
			}else{
				flash('user_message', 'subscriptions', json_decode($response)->errors[0]->description, 'danger');
				redirect('subscriptions');
			}
        }
        
        public function tap_success()
        {
			$tap_live_enabled = $this->settingModel->getOptions('tap_live_enabled');
			$tap_settings     = $this->settingModel->getTapSettings($tap_live_enabled->option_value);
			
			$curl = curl_init();

			curl_setopt_array($curl, [
					CURLOPT_URL => $tap_settings->tap_url."/v2/charges/".$_SESSION['transaction_id'],
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => "",
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 30,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST => "GET",
					CURLOPT_HTTPHEADER => [
						"Authorization: Bearer ".$tap_settings->tap_secret_key,
						"accept: application/json"
					],
				]);

			$result   = curl_exec($curl);

			$response = json_decode($result);

			$explode = explode("|", $_SESSION['custom']);
			if(!empty($response) && $response->status == 'CAPTURED' && $response->amount == package_cost($explode[1], $explode[2]))
			{
				$data    = [
					'subscription_cost' => $response->amount,
					'subscription_uid' => $response->reference->transaction,
					'transaction_id' => $response->id,
					'payment_option_id' => payment_option_slug('tap')->payment_option_id,
					'subscription_added_by' => $explode[0],
					'transaction_reference' => NULL,
					'package_id' => $explode[1],
                    'package_duration' => package_duration($explode[1], $explode[2])[0],
                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
					'subscription_items' => json_encode(package($explode[1]))
				];
				if(empty($data['input_error']))
				{
					if($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data))
					{
						if(affiliate_accepted($subscription_id))
						{
							$dataA = [
								'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
								'affiliate_rule_id' => getOption('affiliate_rule_id'),
								'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
								'subscription_id' => $subscription_id
							];
							$this->affiliateModel->addAffiliate($dataA);
						}
						if(getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1)
						{
							$dataw = [
								'user_id' => $explode[0],
								'user_remaining_words' => add_user_remaining_balances($explode[0], $subscription_id, 'words'),
								'user_remaining_chat_assistants' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_assistants'),
								'user_remaining_trained_models' => add_user_remaining_balances($explode[0], $subscription_id, 'trained_models'),
								'user_remaining_chat_files' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_files'),
								'user_remaining_chat_links' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_links'),
								'user_remaining_chat_sitemaps' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_sitemaps')
							];
							foreach(package_types() as $type)
							{
								$out_type = 'user_remaining_' . $type;
								$dataz    = [
									'user_id' => $explode[0],
									'type' => $type,
									'transaction_type' => 'in',
									'package_id' => $explode[1],
									'old' => (int)user($explode[0])->$out_type,
									'current' => $dataw[$out_type],
									'user_package_history_comment' => json_encode(package($explode[1])),
									'user_package_history_added_by' => $explode[0]
								];
								$this->userModel->addUserPackageHistory($dataz);
							}
							$this->userModel->editUserRemaining($dataw);
						}
						sendEmail('SubscriptionPaid', user($explode[0])->email, 'subscriptions', $subscription_id, $explode[0]);
						notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
						addActivityLog('subscription', $subscription_id, 'tap_payment_added');
						flash('user_message', 'subscriptions', 'tap_payment_added');
						redirect('subscriptions/show/' . $subscription_id);
					}
					else
					{
						flash('user_message', 'subscriptions', 'adding_tap_payment_failed', 'danger');
						redirect('subscriptions');
					}
				}
				else
				{
					flash('user_message', 'subscriptions', 'adding_tap_payment_failed', 'danger');
					redirect('subscriptions');
				}
			}
			else
			{
				flash('user_message', 'subscriptions', 'adding_tap_payment_failed', 'danger');
				redirect('subscriptions');
			}

			if(curl_errno($curl))
			{
				flash('user_message', 'subscriptions', curl_error($curl), 'danger');
				redirect('subscriptions');
			}

			curl_close($curl);
			die();

			}
        
    }
