<?php
    
    
    class Pmt_tripay extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
			$tripay_live_enabled = $this->settingModel->getOptions('tripay_live_enabled');
			$tripay_settings     = $this->settingModel->getTripaySettings($tripay_live_enabled->option_value);
			
			$explode = explode("|", $_SESSION['custom']);
			
			$apiKey       = $tripay_settings->tripay_api_key;
			$privateKey   = $tripay_settings->tripay_private_key;
			$merchantCode = $tripay_settings->tripay_merchant_code;
			$merchantRef  = uid();
			$amount       = package_cost($explode[1], $explode[2]);

			$data = [
			    'method'         => $tripay_settings->tripay_payment_method,
			    'merchant_ref'   => $merchantRef,
			    'amount'         => $amount,
			    'customer_name'  => user(user_id())->firstname.' '.user(user_id())->lastname,
			    'customer_email' => user(user_id())->email,
			    'customer_phone' => user(user_id())->calling_code.user(user_id())->phone,
			    'order_items'    => [
			        [
			            'sku'         => '',
			            'name'        => package($explode[1])->package_name,
			            'price'       => $amount,
			            'quantity'    => 1,
			            'product_url' => '',
			            'image_url'   => '',
			        ]
			    ],
			    'return_url'   => URLROOT . '/pmt_tripay/tripay_success',
			    'expired_time' => (time() + (24 * 60 * 60)), // 24 jam
			    'signature'    => hash_hmac('sha256', $merchantCode.$merchantRef.$amount, $privateKey)
			];

			$curl = curl_init();

			curl_setopt_array($curl, [
			    CURLOPT_FRESH_CONNECT  => true,
			    CURLOPT_URL            => $tripay_settings->tripay_url.'/create',
			    CURLOPT_RETURNTRANSFER => true,
			    CURLOPT_HEADER         => false,
			    CURLOPT_HTTPHEADER     => ['Authorization: Bearer '.$apiKey],
			    CURLOPT_FAILONERROR    => false,
			    CURLOPT_POST           => true,
			    CURLOPT_POSTFIELDS     => http_build_query($data),
			    CURLOPT_IPRESOLVE      => CURL_IPRESOLVE_V4
			]);

			$response = curl_exec($curl);
			$error = curl_error($curl);

			curl_close($curl);


			if($error || empty($response))
			{
				flash('user_message', 'subscriptions', "cURL Error #:" . $error, 'danger');
				redirect('subscriptions');
			}
			elseif(!empty($response) && empty(json_decode($response)->message))
			{
				$_SESSION['reference'] = json_decode($response)->data->reference;
				header("Location: ".json_decode($response)->data->checkout_url);
				die();
			}else{
				flash('user_message', 'subscriptions', json_decode($response)->message, 'danger');
				redirect('subscriptions');
			}
        }
        
        public function tripay_success()
        {
			$tripay_live_enabled = $this->settingModel->getOptions('tripay_live_enabled');
			$tripay_settings     = $this->settingModel->getTripaySettings($tripay_live_enabled->option_value);
			
			$payload = [
			    'reference' => $_SESSION['reference'],
			];

			$curl = curl_init();
			curl_setopt_array($curl, [
			    CURLOPT_FRESH_CONNECT  => true,
			    CURLOPT_URL            => $tripay_settings->tripay_url.'/detail?'.http_build_query($payload),
			    CURLOPT_RETURNTRANSFER => true,
			    CURLOPT_HEADER         => false,
			    CURLOPT_HTTPHEADER     => ['Authorization: Bearer '.$tripay_settings->tripay_api_key],
			    CURLOPT_FAILONERROR    => false,
			    CURLOPT_IPRESOLVE      => CURL_IPRESOLVE_V4
			]);
			$response = curl_exec($curl);
			$error = curl_error($curl);
			curl_close($curl);
			
			if($error || empty($response))
			{
				flash('user_message', 'subscriptions', "cURL Error #:" . $error, 'danger');
				redirect('subscriptions');
			}

			$explode = explode("|", $_SESSION['custom']);
			
			if(!empty($response) && $response->data->status == 'PAID' && $response->data->amount == package_cost($explode[1], $explode[2]))
			{
				$data    = [
					'subscription_cost' => $response->data->amount,
					'subscription_uid' => $response->data->merchant_ref,
					'transaction_id' => $response->data->reference,
					'payment_option_id' => payment_option_slug('tripay')->payment_option_id,
					'subscription_added_by' => $explode[0],
					'transaction_reference' => NULL,
					'package_id' => $explode[1],
                    'package_duration' => package_duration($explode[1], $explode[2])[0],
                    'package_duration_type' => package_duration($explode[1], $explode[2])[1],
					'subscription_items' => json_encode(package($explode[1]))
				];
				if(empty($data['input_error']))
				{
					if($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data))
					{
						if(affiliate_accepted($subscription_id))
						{
							$dataA = [
								'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
								'affiliate_rule_id' => getOption('affiliate_rule_id'),
								'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
								'subscription_id' => $subscription_id
							];
							$this->affiliateModel->addAffiliate($dataA);
						}
						if(getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1)
						{
							$dataw = [
								'user_id' => $explode[0],
								'user_remaining_words' => add_user_remaining_balances($explode[0], $subscription_id, 'words'),
								'user_remaining_chat_assistants' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_assistants'),
								'user_remaining_trained_models' => add_user_remaining_balances($explode[0], $subscription_id, 'trained_models'),
								'user_remaining_chat_files' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_files'),
								'user_remaining_chat_links' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_links'),
								'user_remaining_chat_sitemaps' => add_user_remaining_balances($explode[0], $subscription_id, 'chat_sitemaps')
							];
							foreach(package_types() as $type)
							{
								$out_type = 'user_remaining_' . $type;
								$dataz    = [
									'user_id' => $explode[0],
									'type' => $type,
									'transaction_type' => 'in',
									'package_id' => $explode[1],
									'old' => (int)user($explode[0])->$out_type,
									'current' => $dataw[$out_type],
									'user_package_history_comment' => json_encode(package($explode[1])),
									'user_package_history_added_by' => $explode[0]
								];
								$this->userModel->addUserPackageHistory($dataz);
							}
							$this->userModel->editUserRemaining($dataw);
						}
						sendEmail('SubscriptionPaid', user($explode[0])->email, 'subscriptions', $subscription_id, $explode[0]);
						notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
						addActivityLog('subscription', $subscription_id, 'tripay_payment_added');
						flash('user_message', 'subscriptions', 'tripay_payment_added');
						redirect('subscriptions/show/' . $subscription_id);
					}
					else
					{
						flash('user_message', 'subscriptions', 'adding_tripay_payment_failed', 'danger');
						redirect('subscriptions');
					}
				}
				else
				{
					flash('user_message', 'subscriptions', 'adding_tripay_payment_failed', 'danger');
					redirect('subscriptions');
				}
			}
			else
			{
				flash('user_message', 'subscriptions', json_decode($response)->message, 'danger');
				redirect('subscriptions');
			}

			if(curl_errno($curl))
			{
				flash('user_message', 'subscriptions', curl_error($curl), 'danger');
				redirect('subscriptions');
			}
			}
        
    }
