<?php

use Tco\TwocheckoutFacade;
    
class Pmt_twocheckout extends Controller{
	public function __construct(){
		if(!isLoggedIn()){
			logout();
		}
		require APPROOT . '/views/inc/load_models.php';
	}
        
	public function index(){
		$twocheckout_live_enabled = $this->settingModel->getOptions('twocheckout_live_enabled');
		$twocheckout_settings = $this->settingModel->getTwocheckoutSettings($twocheckout_live_enabled->option_value);
            
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $data = [
                'firstname' => cleanString($_POST['firstname']),
                'lastname' => cleanString($_POST['lastname']),
                'email' => cleanString($_POST['email']),
                'phone' => cleanString($_POST['phone']),
                'card_number' => cleanString(str_replace(" ", "", $_POST['credit_card'])),
                'cvv' => cleanString($_POST['cvv']),
                'expiration_month' => cleanString($_POST['expiration_month']),
                'expiration_year' => cleanString($_POST['expiration_year']),
                'package_id' => $_POST['package_id'],
                'package_group' => $_POST['package_group'],
            ];
            
            require_once (APPROOT . '/third_party/2checkout/autoloader.php');				
		    
			$config    = array(
	            'sellerId'      => $twocheckout_settings->twocheckout_merchant_code,
	            'secretKey'     => $twocheckout_settings->twocheckout_secret_key,
	            'jwtExpireTime' => 30,
	            'curlVerifySsl' => 0
			);

			$tco = new TwocheckoutFacade($config);
			
			$dynamicOrderParams = array(
	            'Country'           => country(user(user_id())->country_id)->iso2,
	            'Currency'          => getOption('default_currency'),
	            'CustomerIP'        => ip_address(),
	            'ExternalReference' => uid(),
	            'Language'          => 'en',
	            'Source'            => URLROOT,
	            'BillingDetails'    =>
	                array(
	                    'Address1'    => user(user_id())->street,
	                    'City'        => user(user_id())->city,
	                    'State'       => user(user_id())->state,
	                    'CountryCode' => country(user(user_id())->country_id)->iso2,
	                    'Email'       => $data['email'],
	                    'FirstName'   => $data['firstname'],
	                    'LastName'    => $data['lastname'],
	                    'Zip'         => user(user_id())->zip_code,
	                ),
	            'Items'             =>
	                array(
	                    0 =>
	                        array(
	                            'Name'         => package($data['package_id'])->package_name,
	                            'Description'  => package($data['package_id'])->package_name,
	                            'Quantity'     => 1, //units
	                            'IsDynamic'    => true,
	                            'Tangible'     => false,
	                            'PurchaseType' => 'PRODUCT',
	                            'Price'        =>
	                                array(
	                                    'Amount' => package_cost($data['package_id'], $data['package_group']), //value
	                                    'Type'   => 'CUSTOM',
	                                ),
	                        ),
	                ),
	            'PaymentDetails'    =>
	                array(
	                    'Type'          => 'TEST', //'TEST' or 'EES_TOKEN_PAYMENT'
	                    'Currency'      => getOption('default_currency'),
	                    'CustomerIP'    => ip_address(),
	                    'PaymentMethod' =>
	                array(
	                    'CCID'               => $data['cvv'],
	                    'CardNumber'         => $data['card_number'],
	                    'CardNumberTime'     => NULL,
	                    'CardType'           => detect_card_type($data['card_number']),
	                    'ExpirationMonth'    => $data['expiration_month'],
	                    'ExpirationYear'     => $data['expiration_year'],
	                    'HolderName'         => $data['firstname'] . ' ' . $data['lastname'],
	                    'HolderNameTime'     => NULL,
	                    'RecurringEnabled'   => true,
	                    'Vendor3DSReturnURL' => URLROOT . '/pmt_twocheckout/twocheckout_return',
	                    'Vendor3DSCancelURL' => URLROOT . '/pmt_twocheckout/twocheckout_cancel',
	                ),
	            ),
	        );
	                
	        $order = $tco->order();
	        
	        $orderData = $order->place($dynamicOrderParams);
	        
	        $response = $tco->order()->getOrder(array('RefNo' => $orderData['RefNo']));
	        	        
            if (in_array($response['status'], array('AUTHRECEIVED','COMPLETE')) && $response['Items'][0]['Price']['GrossPrice'] == package_cost($data['package_id'], $data['package_group'])) {
                $data = [
                    'subscription_cost' => $response['Items'][0]['Price']['GrossPrice'],
                    'subscription_uid' => uid(),
                    'transaction_id' => $response['RefNo'],
                    'payment_option_id' => payment_option_slug('twocheckout')->payment_option_id,
                    'subscription_added_by' => user_id(),
                    'transaction_reference' => $response['ExternalReference'],
                    'package_id' => $data['package_id'],
                    'package_duration' => package_duration($data['package_id'], $data['package_group'])[0],
                    'package_duration_type' => package_duration($data['package_id'], $data['package_group'])[1],
                    'subscription_items' => json_encode(package($data['package_id']))
                ];
                if ($subscription_id = $this->subscriptionModel->addSubscriptionPayment($data)) {
                    if (affiliate_accepted($subscription_id)) {
                        $dataA = [
                            'user_id' => user(subscription($subscription_id)->user_id)->referred_by,
                            'affiliate_rule_id' => getOption('affiliate_rule_id'),
                            'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                            'subscription_id' => $subscription_id
                        ];
                        $this->affiliateModel->addAffiliate($dataA);
                    }
                    if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                        $dataw = [
                            'user_id' => user_id(),
                            'user_remaining_words' => add_user_remaining_balances(user_id(), $subscription_id, 'words'),
                            'user_remaining_chat_assistants' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_assistants'),
                            'user_remaining_trained_models' => add_user_remaining_balances(user_id(), $subscription_id, 'trained_models'),
                            'user_remaining_chat_files' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_files'),
                            'user_remaining_chat_links' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_links'),
                            'user_remaining_chat_sitemaps' => add_user_remaining_balances(user_id(), $subscription_id, 'chat_sitemaps')
                        ];
                        foreach (package_types() as $type) {
                            $out_type = 'user_remaining_' . $type;
                            $dataz = [
                                'user_id' => user_id(),
                                'type' => $type,
                                'transaction_type' => 'in',
                                'package_id' => $data['package_id'],
                                'old' => (int)user(user_id())->$out_type,
                                'current' => $dataw[$out_type],
                                'user_package_history_comment' => json_encode(package($data['package_id'])),
                                'user_package_history_added_by' => user_id()
                            ];
                            $this->userModel->addUserPackageHistory($dataz);
                        }
                        $this->userModel->editUserRemaining($dataw);
                    }
                    sendEmail('SubscriptionPaid', user(user_id())->email, 'subscriptions', $subscription_id, user_id());
                    notificationToUser('subscription_paid', 1, 'subscriptions/show/' . $subscription_id);
                    addActivityLog('subscription', $subscription_id, 'twocheckout_payment_added');
                    flash('user_message', 'subscriptions', 'twocheckout_payment_added');
                } else {
                    flash('user_message', 'subscriptions', 'adding_twocheckout_payment_failed', 'danger');
                }
            } else {
                flash('user_message', 'subscriptions', 'twocheckout_payment_transaction_failed', 'danger');
            }
            redirect('subscriptions');
            die();
        }
	}
        
	public function twocheckout_return(){
		
	}
        
	public function twocheckout_cancel(){
		flash('user_message', 'subscriptions', 'adding_twocheckout_payment_failed', 'danger');
		redirect('subscriptions');
		die();
	}
}
