<?php
    
    
    class Posts extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $posts = $this->postModel->getPosts();
            $data = [
                'posts' => $posts
            ];
            $this->view('posts/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $featured_image = fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['featured_image']['name']));
                upload_image($featured_image, 'media', $_FILES['featured_image']['tmp_name']);
                
                $data = [
                    'post_title' => cleanString($_POST['post_title']),
                    'post_category_id' => cleanString($_POST['post_category_id']),
                    'post_featured_image' => $featured_image,
                    'post_description' => cleanString($_POST['post_description'], 'ignore_tags'),
                    'post_status_id' => !empty($_POST['post_status_id']) ? cleanString($_POST['post_status_id']) : 0
                ];
                if (empty($data['post_title'])) {
                    $data['input_error'][] = 'please_enter_post_title';
                }
                if (empty($data['post_description'])) {
                    $data['input_error'][] = 'please_enter_post_description';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->postModel->addPost($data)) {
                        addActivityLog('post', $id, 'post_added');
                        flash('user_message', 'post', 'post_added');
                        redirect('posts/show/' . $id);
                    } else {
                        flash('user_message', 'post', 'adding_post_failed', 'danger');
                        redirect('posts/add');
                    }
                    die();
                }
                $this->view('posts/add', $data);
            }
            $this->view('posts/add');
        }
        
        public function edit($id = NULL)
        {
            $post = $this->postModel->getPostById($id);
            if (empty($post)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'post_id' => $id,
                    'post_title' => cleanString($_POST['post_title']),
                    'post_category_id' => cleanString($_POST['post_category_id']),
                    'post_description' => cleanString($_POST['post_description'], 'ignore_tags'),
                    'post_status_id' => !empty($_POST['post_status_id']) ? cleanString($_POST['post_status_id']) : 0,
                    'post' => $post
                ];
                if (empty($data['post_title'])) {
                    $data['input_error'][] = 'please_enter_post_title';
                }
                if (empty($data['post_description'])) {
                    $data['input_error'][] = 'please_enter_post_description';
                }
                if (empty($data['input_error'])) {
                    if ($this->postModel->updatePost($data)) {
                        addActivityLog('post', $id, 'post_updated');
                        flash('user_message', 'post', 'post_updated');
                    } else {
                        flash('user_message', 'post', 'post_update_failed', 'danger');
                    }
                    redirect('posts/show/' . $id);
                    die();
                }
                $this->view('posts/edit', $data);
            }
            $data = [
                'post' => $post
            ];
            $this->view('posts/edit', $data);
        }
        
        public function update_featured_image($id = NULL)
        {
            $post = $this->postModel->getPostById($id);
            if (empty($post)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $featured_image = fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['featured_image']['name']));
                upload_image($featured_image, 'media', $_FILES['featured_image']['tmp_name']);
                
                $data = [
                    'post_id' => $id,
                    'post_featured_image' => $featured_image
                ];
                if (empty($data['post_featured_image'])) {
                    $data['input_error'] = 'please_select_featured_image';
                }
                if (empty($data['input_error'])) {
                    if ($this->postModel->updateFeaturedImage($data)) {
                        unlinkUploads('media', $post->post_featured_image);
                        addActivityLog('post', $id, 'featured_image_updated');
                        flash('user_message', 'post', 'featured_image_updated');
                    } else {
                        flash('user_message', 'post', 'updating_featured_image_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'post', $data['input_error'], 'danger');
                }
            }
            redirect('posts/show/' . $id);
            die();
        }
        
        public function show($id = NULL)
        {
            $post = $this->postModel->getPostById($id);
            if (empty($post)) {
                redirect(404);
            }
            $data = [
                'post' => $post
            ];
            $this->view('posts/show', $data);
        }
        
        public function delete($id = NULL)
        {
            $post = $this->postModel->getPostById($id);
            if (empty($post)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->postModel->deletePost($id)) {
                    addActivityLog('post', $id, 'post_deleted');
                    flash('user_message', 'post', 'post_deleted');
                    redirect('posts');
                } else {
                    flash('user_message', 'post', 'post_delete_failed', 'danger');
                    redirect('posts/show/' . $id);
                }
                die();
            } else {
                redirect('posts');
            }
        }
        
        
        public function categories()
        {
            $categories = $this->postModel->getPostCategories();
            $data = [
                'categories' => $categories
            ];
            $this->view('posts/categories', $data);
        }
        
        public function category($id = NULL)
        {
            $category = $this->postModel->getPostCategoryById($id);
            if (empty($category)) {
                redirect(404);
            }
            $data = [
                'category' => $category
            ];
            $this->view('posts/category', $data);
        }
        
        public function add_category()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'post_category_name' => cleanString($_POST['post_category_name']),
                    'post_category_status_id' => !empty($_POST['post_category_status_id']) ? cleanString($_POST['post_category_status_id']) : 0
                ];
                if (empty($data['post_category_name'])) {
                    $data['input_error'][] = 'please_enter_post_category_name';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->postModel->addPostCategory($data)) {
                        addActivityLog('post', $id, 'post_category_added');
                        flash('user_message', 'post', 'post_category_added');
                        redirect('posts/category/' . $id);
                    } else {
                        flash('user_message', 'post', 'adding_post_category_failed', 'danger');
                        redirect('posts/add_category');
                    }
                    die();
                }
                $this->view('posts/add_category', $data);
            }
            $this->view('posts/add_category');
        }
        
        public function edit_category($id = NULL)
        {
            $category = $this->postModel->getPostCategoryById($id);
            if (empty($category)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'post_category_id' => $id,
                    'post_category_name' => cleanString($_POST['post_category_name']),
                    'post_category_status_id' => !empty($_POST['post_category_status_id']) ? cleanString($_POST['post_category_status_id']) : 0,
                    'category' => $category
                ];
                if (empty($data['post_category_name'])) {
                    $data['input_error'][] = 'please_enter_post_category_name';
                }
                if (empty($data['input_error'])) {
                    if ($this->postModel->updatePostCategory($data)) {
                        addActivityLog('post', $id, 'post_category_updated');
                        flash('user_message', 'post', 'post_category_updated');
                    } else {
                        flash('user_message', 'post', 'post_caategory_update_failed', 'danger');
                    }
                    redirect('posts/category/' . $id);
                    die();
                }
                $this->view('posts/edit_category', $data);
            }
            $data = [
                'category' => $category
            ];
            $this->view('posts/edit_category', $data);
        }
        
        public function delete_category($id = NULL)
        {
            $category = $this->postModel->getPostCategoryById($id);
            if (empty($category)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->postModel->deletePostCategory($id)) {
                    addActivityLog('post', $id, 'post_deleted');
                    flash('user_message', 'post', 'post_deleted');
                    redirect('posts/categories');
                } else {
                    flash('user_message', 'post', 'post_delete_failed', 'danger');
                    redirect('posts/category/' . $id);
                }
                die();
            } else {
                redirect('posts');
            }
        }
        
        
    }
