<?php
    
    
    class Servers extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            $data = [
                'ai_files' => (ai_files()['status'] == 'success') ? ai_files()['data'] : array()
            ];
            $this->view('servers/index', $data);
        }
        
        public function delete($file_id = NULL)
        {
            $file = ai_retrieve_finetune_file($file_id);
            if (empty($file)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if (ai_delete_file($file_id)['status'] == 'success') {
                    addActivityLog('file', $file_id, 'file_deleted');
                    flash('user_message', 'file', 'file_deleted');
                } elseif (ai_delete_file($file_id)['status'] == 'error') {
                    flash('user_message', 'file', ai_delete_file($file_id)['message'], 'danger');
                } else {
                    flash('user_message', 'file', 'file_delete_failed', 'danger');
                }
            }
            redirect('servers');
            die();
        }

        
        
        public function models()
        {
            $models = openai_models();
            
            $data = [
                'models' => $models
            ];
            $this->view('servers/models', $data);
        }
        
        public function add_model()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                $data = [
                    'openai_model' => cleanString($_POST['openai_model']),
                    'openai_model_type' => cleanString($_POST['openai_model_type']),
                    'openai_model_usage_type' => isset($_POST['openai_model_usage_type']) ? json_encode($_POST['openai_model_usage_type']) : NULL,
                    'openai_model_description' => cleanString($_POST['openai_model_description'], 'ignore_tags'),
                    'openai_model_tokens' => cleanString($_POST['openai_model_tokens']),
                    'openai_model_status_id' => 1
                ];
                if (empty($data['openai_model'])) {
                    $data['input_error'][] = 'please_enter_model';
                }
                if (empty($data['openai_model_usage_type'])) {
                    $data['input_error'][] = 'please_enter_model_usage_type';
                }
                if(openai_model_by_model($data['openai_model']))
                {
                    $data['input_error'][] = 'model_already_exists';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->openaiModel->addModel($data)) {
                        addActivityLog('model', $id, 'model_added');
                        flash('user_message', 'models', 'model_added');
                        redirect('servers/show_model/' . $id);
                    } else {
                        flash('user_message', 'models', 'adding_model_failed', 'danger');
                        redirect('servers/add_model');
                    }
                    die();
                }
                $this->view('servers/add_model', $data);
            }
            $this->view('servers/add_model');
        }
        
        public function edit_model($id = NULL)
        {
            $model = openai_model_by_id($id);
            if (empty($model)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'openai_model_id' => $id,
                    'openai_model' => cleanString($_POST['openai_model']),
                    'openai_model_type' => cleanString($_POST['openai_model_type']),
                    'openai_model_usage_type' => isset($_POST['openai_model_usage_type']) ? json_encode($_POST['openai_model_usage_type']) : NULL,
                    'openai_model_description' => cleanString($_POST['openai_model_description'], 'ignore_tags'),
                    'openai_model_tokens' => cleanString($_POST['openai_model_tokens']),
                    'model' => $model
                ];
                
                if (empty($data['openai_model'])) {
                    $data['input_error'][] = 'please_enter_model';
                }
                if (empty($data['openai_model_usage_type'])) {
                    $data['input_error'][] = 'please_enter_model_usage_type';
                }
                if (empty($data['input_error'])) {
                    if ($this->openaiModel->updateModel($data)) {
                        addActivityLog('model', $id, 'model_updated');
                        flash('user_message', 'models', 'model_updated');
                    } else {
                        flash('user_message', 'models', 'model_update_failed', 'danger');
                    }
                    redirect('servers/show_model/' . $id);
                    die();
                }
                $this->view('servers/edit_model', $data);
            }
            $data = [
                'model' => $model
            ];
            $this->view('servers/edit_model', $data);
        }
        
        public function show_model($id = NULL)
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            $model = openai_model_by_id($id);
            if (empty($model)) {
                redirect(404);
            }
            $data = [
                'model' => $model
            ];
            $this->view('servers/show_model', $data);
        }
        
        public function delete_model($id = NULL)
        {
            $model = openai_model_by_id($id);
            if (empty($model)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->openaiModel->deleteOpenAiModel($id)) {
                    addActivityLog('model', $id, 'model_deleted');
                    flash('user_message', 'models', 'model_deleted');
                    redirect('servers/models');
                } else {
                    flash('user_message', 'models', 'model_delete_failed', 'danger');
                    redirect('servers/show_model/' . $id);
                }
                die();
            } else {
                redirect('servers/models');
            }
        }
        
        
    }
