<?php
    
    
    class Settings extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if (!isSuperAdmin()) {
                redirect(404);
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            redirect('dashboards');
        }
        
        public function payment_settings()
        {
            $this->view('settings/payment_settings');
        }
        
        public function email_settings()
        {
            $smtp_live_enabled = $this->settingModel->getOptions('smtp_live_enabled');
            $smtp_settings = $this->settingModel->getSMTPSettings($smtp_live_enabled->option_value);
            $sendgrid_live_enabled = $this->settingModel->getOptions('sendgrid_live_enabled');
            $sendgrid_settings = $this->settingModel->getSendgridSettings($sendgrid_live_enabled->option_value);
            $data = [
                'smtp_live_enabled' => $smtp_live_enabled,
                'smtp_settings' => $smtp_settings,
                'sendgrid_live_enabled' => $sendgrid_live_enabled,
                'sendgrid_settings' => $sendgrid_settings,
            ];
            $this->view('settings/email_settings', $data);
        }
        
        public function update_paypal_standard_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paypal_standard_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paypal_standard_status']),
                    'paypal_standard_email' => cleanString($_POST['paypal_standard_email'])
                ];
                $data_label = [
                    'option_name' => 'paypal_standard_label',
                    'option_value' => cleanString($_POST['paypal_standard_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paypal_standard')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatePaypalStandardSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paypal_standard_settings_updated');
                    flash('user_message', 'settings', 'paypal_standard_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paypal_standard_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_paypal_pro_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paypal_pro_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paypal_pro_status']),
                    'paypal_pro_username' => cleanString($_POST['paypal_pro_username']),
                    'paypal_pro_password' => cleanString($_POST['paypal_pro_password']),
                    'paypal_pro_signature' => cleanString($_POST['paypal_pro_signature'])
                ];
                $data_label = [
                    'option_name' => 'paypal_pro_label',
                    'option_value' => cleanString($_POST['paypal_pro_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paypal_pro')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatePaypalProSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paypal_pro_settings_updated');
                    flash('user_message', 'settings', 'paypal_pro_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paypal_pro_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_stripe_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'stripe_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['stripe_status']),
                    'stripe_secret_key' => cleanString($_POST['stripe_secret_key']),
                    'stripe_publishable_key' => cleanString($_POST['stripe_publishable_key'])
                ];
                $data_label = [
                    'option_name' => 'stripe_label',
                    'option_value' => cleanString($_POST['stripe_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('stripe')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateStripeSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'stripe_settings_updated');
                    flash('user_message', 'settings', 'stripe_settings_updated');
                } else {
                    flash('user_message', 'settings', 'stripe_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_authorize_net_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'authorize_net_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['authorize_net_status']),
                    'authorize_net_login_id' => cleanString($_POST['authorize_net_login_id']),
                    'authorize_net_transaction_key' => cleanString($_POST['authorize_net_transaction_key'])
                ];
                $data_label = [
                    'option_name' => 'authorize_net_label',
                    'option_value' => cleanString($_POST['authorize_net_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('authorize_net')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateAuthorizeNetSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'authorize_net_settings_updated');
                    flash('user_message', 'settings', 'authorize_net_settings_updated');
                } else {
                    flash('user_message', 'settings', 'authorize_net_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_pesapal_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'pesapal_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['pesapal_status']),
                    'pesapal_consumer_key' => cleanString($_POST['pesapal_consumer_key']),
                    'pesapal_consumer_secret' => cleanString($_POST['pesapal_consumer_secret']),
                    'pesapal_notification_id' => cleanString($_POST['pesapal_notification_id'])
                ];
                $data_label = [
                    'option_name' => 'pesapal_label',
                    'option_value' => cleanString($_POST['pesapal_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('pesapal')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatePesapalSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'pesapal_settings_updated');
                    flash('user_message', 'settings', 'pesapal_settings_updated');
                } else {
                    flash('user_message', 'settings', 'pesapal_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_flutterwave_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'flutterwave_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['flutterwave_status']),
                    'flutterwave_public_key' => cleanString($_POST['flutterwave_public_key']),
                    'flutterwave_secret_key' => cleanString($_POST['flutterwave_secret_key']),
                    'flutterwave_encryption_key' => cleanString($_POST['flutterwave_encryption_key'])
                ];
                $data_label = [
                    'option_name' => 'flutterwave_label',
                    'option_value' => cleanString($_POST['flutterwave_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('flutterwave')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateFlutterwaveSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'flutterwave_settings_updated');
                    flash('user_message', 'settings', 'flutterwave_settings_updated');
                } else {
                    flash('user_message', 'settings', 'flutterwave_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_sslcommerz_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'sslcommerz_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['sslcommerz_status']),
                    'sslcommerz_store_id' => cleanString($_POST['sslcommerz_store_id']),
                    'sslcommerz_password' => cleanString($_POST['sslcommerz_password'])
                ];
                $data_label = [
                    'option_name' => 'sslcommerz_label',
                    'option_value' => cleanString($_POST['sslcommerz_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('sslcommerz')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateSslcommerzSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'sslcommerz_settings_updated');
                    flash('user_message', 'settings', 'sslcommerz_settings_updated');
                } else {
                    flash('user_message', 'settings', 'sslcommerz_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_esewa_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'esewa_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['esewa_status']),
                    'esewa_merchant_code' => cleanString($_POST['esewa_merchant_code'])
                ];
                $data_label = [
                    'option_name' => 'esewa_label',
                    'option_value' => cleanString($_POST['esewa_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('esewa')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateEsewaSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'esewa_settings_updated');
                    flash('user_message', 'settings', 'esewa_settings_updated');
                } else {
                    flash('user_message', 'settings', 'esewa_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_instamojo_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'instamojo_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['instamojo_status']),
                    'instamojo_api_key' => cleanString($_POST['instamojo_api_key']),
                    'instamojo_auth_token' => cleanString($_POST['instamojo_auth_token'])
                ];
                $data_label = [
                    'option_name' => 'instamojo_label',
                    'option_value' => cleanString($_POST['instamojo_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('instamojo')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateInstamojoSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'instamojo_settings_updated');
                    flash('user_message', 'settings', 'instamojo_settings_updated');
                } else {
                    flash('user_message', 'settings', 'instamojo_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_paystack_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paystack_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paystack_status']),
                    'paystack_secret_key' => cleanString($_POST['paystack_secret_key']),
                    'paystack_public_key' => cleanString($_POST['paystack_public_key'])
                ];
                $data_label = [
                    'option_name' => 'paystack_label',
                    'option_value' => cleanString($_POST['paystack_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paystack')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatepaystackSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paystack_settings_updated');
                    flash('user_message', 'settings', 'paystack_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paystack_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_aamarpay_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'aamarpay_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['aamarpay_status']),
                    'aamarpay_store_id' => cleanString($_POST['aamarpay_store_id']),
                    'aamarpay_signature_key' => cleanString($_POST['aamarpay_signature_key'])
                ];
                $data_label = [
                    'option_name' => 'aamarpay_label',
                    'option_value' => cleanString($_POST['aamarpay_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('aamarpay')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateAamarpaySettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'aamarpay_settings_updated');
                    flash('user_message', 'settings', 'aamarpay_settings_updated');
                } else {
                    flash('user_message', 'settings', 'aamarpay_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_hyperpay_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'hyperpay_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['hyperpay_status']),
                    'hyperpay_access_token' => cleanString($_POST['hyperpay_access_token']),
                    'hyperpay_entity_id' => cleanString($_POST['hyperpay_entity_id']),
                    'hyperpay_brands' => cleanString($_POST['hyperpay_brands'])
                ];
                $data_label = [
                    'option_name' => 'hyperpay_label',
                    'option_value' => cleanString($_POST['hyperpay_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('hyperpay')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatehyperpaySettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'hyperpay_settings_updated');
                    flash('user_message', 'settings', 'hyperpay_settings_updated');
                } else {
                    flash('user_message', 'settings', 'hyperpay_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_toyyibpay_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'toyyibpay_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['toyyibpay_status']),
                    'toyyibpay_secret_key' => cleanString($_POST['toyyibpay_secret_key']),
                    'toyyibpay_category' => cleanString($_POST['toyyibpay_category'])
                ];
                $data_label = [
                    'option_name' => 'toyyibpay_label',
                    'option_value' => cleanString($_POST['toyyibpay_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('toyyibpay')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateToyyibpaySettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'toyyibpay_settings_updated');
                    flash('user_message', 'settings', 'toyyibpay_settings_updated');
                } else {
                    flash('user_message', 'settings', 'toyyibpay_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_paygate_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paygate_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paygate_status']),
                    'paygate_paygate_id' => cleanString($_POST['paygate_paygate_id']),
                    'paygate_encryption_key' => cleanString($_POST['paygate_encryption_key'])
                ];
                $data_label = [
                    'option_name' => 'paygate_label',
                    'option_value' => cleanString($_POST['paygate_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paygate')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatePaygateSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paygate_settings_updated');
                    flash('user_message', 'settings', 'paygate_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paygate_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_paddle_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paddle_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paddle_status']),
                    'paddle_api_key' => cleanString($_POST['paddle_api_key']),
                    'paddle_client_side_token' => cleanString($_POST['paddle_client_side_token'])
                ];
                $data_label = [
                    'option_name' => 'paddle_label',
                    'option_value' => cleanString($_POST['paddle_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paddle')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatepaddleSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paddle_settings_updated');
                    flash('user_message', 'settings', 'paddle_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paddle_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_paytr_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'paytr_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['paytr_status']),
                    'paytr_merchant_id' => cleanString($_POST['paytr_merchant_id']),
                    'paytr_merchant_key' => cleanString($_POST['paytr_merchant_key']),
                    'paytr_merchant_salt' => cleanString($_POST['paytr_merchant_salt'])
                ];
                $data_label = [
                    'option_name' => 'paytr_label',
                    'option_value' => cleanString($_POST['paytr_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('paytr')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatePayTRSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'paytr_settings_updated');
                    flash('user_message', 'settings', 'paytr_settings_updated');
                } else {
                    flash('user_message', 'settings', 'paytr_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_iyzico_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'iyzico_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['iyzico_status']),
                    'iyzico_api_key' => cleanString($_POST['iyzico_api_key']),
                    'iyzico_secret_key' => cleanString($_POST['iyzico_secret_key'])
                ];
                $data_label = [
                    'option_name' => 'iyzico_label',
                    'option_value' => cleanString($_POST['iyzico_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('iyzico')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateIyzicoSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'iyzico_settings_updated');
                    flash('user_message', 'settings', 'iyzico_settings_updated');
                } else {
                    flash('user_message', 'settings', 'iyzico_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_tap_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'tap_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['tap_status']),
                    'tap_secret_key' => cleanString($_POST['tap_secret_key']),
                    'tap_publishable_key' => cleanString($_POST['tap_publishable_key']),
                    'tap_encryption_key' => cleanString($_POST['tap_encryption_key'])
                ];
                $data_label = [
                    'option_name' => 'tap_label',
                    'option_value' => cleanString($_POST['tap_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('tap')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatetapSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'tap_settings_updated');
                    flash('user_message', 'settings', 'tap_settings_updated');
                } else {
                    flash('user_message', 'settings', 'tap_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_two_checkout_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'twocheckout_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['twocheckout_status']),
                    'twocheckout_publishable_key' => cleanString($_POST['twocheckout_publishable_key']),
                    'twocheckout_private_key' => cleanString($_POST['twocheckout_private_key']),
                    'twocheckout_secret_key' => cleanString($_POST['twocheckout_secret_key']),
                    'twocheckout_merchant_code' => cleanString($_POST['twocheckout_merchant_code'])
                ];
                $data_label = [
                    'option_name' => 'twocheckout_label',
                    'option_value' => cleanString($_POST['twocheckout_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('twocheckout')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateTwoCheckoutSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), '2checkout_settings_updated');
                    flash('user_message', 'settings', '2checkout_settings_updated');
                } else {
                    flash('user_message', 'settings', '2checkout_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_midtrans_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'midtrans_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['midtrans_status']),
                    'midtrans_client_id' => cleanString($_POST['midtrans_client_id']),
                    'midtrans_server_key' => cleanString($_POST['midtrans_server_key'])
                ];
                $data_label = [
                    'option_name' => 'midtrans_label',
                    'option_value' => cleanString($_POST['midtrans_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('midtrans')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateMidtransSettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'midtrans_settings_updated');
                    flash('user_message', 'settings', 'midtrans_settings_updated');
                } else {
                    flash('user_message', 'settings', 'midtrans_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_tripay_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'tripay_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['tripay_status']),
                    'tripay_private_key' => cleanString($_POST['tripay_private_key']),
                    'tripay_api_key' => cleanString($_POST['tripay_api_key']),
                    'tripay_merchant_code' => cleanString($_POST['tripay_merchant_code']),
                    'tripay_payment_method' => cleanString($_POST['tripay_payment_method'])
                ];
                $data_label = [
                    'option_name' => 'tripay_label',
                    'option_value' => cleanString($_POST['tripay_label']),
                ];
                $data_currencies = [
                    'payment_option_currencies' => cleanString($_POST['payment_option_currencies']),
                    'payment_option_zero_decimal_currencies' => isset($_POST['payment_option_zero_decimal_currencies']) ? cleanString($_POST['payment_option_zero_decimal_currencies']) : NULL,
                    'payment_option_id' => payment_option_slug('tripay')->payment_option_id
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateTripaySettings($data) && $this->settingModel->updateOptionsAutoload($data) && $this->settingModel->updateOptions($data_label) && $this->paymentModel->updatePaymentOptionCurrencies($data_currencies)) {
                    addActivityLog('setting', user_id(), 'tripay_settings_updated');
                    flash('user_message', 'settings', 'tripay_settings_updated');
                } else {
                    flash('user_message', 'settings', 'tripay_settings_update_failed', 'danger');
                }
            }
            redirect('settings/payment_settings');
            die();
        }
        
        public function update_smtp_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'smtp_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['option_autoload']),
                    'smtp_host' => cleanString($_POST['smtp_host']),
                    'smtp_encryption' => cleanString($_POST['smtp_encryption']),
                    'smtp_username' => cleanString($_POST['smtp_username']),
                    'smtp_password' => cleanString($_POST['smtp_password']),
                    'smtp_port' => cleanString($_POST['smtp_port'])
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updateSMTPSettings($data) && $this->settingModel->updateOptionsAutoload($data)) {
                    addActivityLog('setting', user_id(), 'smtp_settings_updated');
                    flash('user_message', 'settings', 'smtp_settings_updated');
                } else {
                    flash('user_message', 'settings', 'smtp_settings_update_failed', 'danger');
                }
            }
            redirect('settings/email_settings');
            die();
        }
        
        public function update_mailchimp_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'mailchimp_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['option_autoload']),
                    'mailchimp_api_key' => cleanString($_POST['mailchimp_api_key']),
                    'mailchimp_list_id' => cleanString($_POST['mailchimp_list_id']),
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatemailchimpSettings($data) && $this->settingModel->updateOptionsAutoload($data)) {
                    addActivityLog('setting', user_id(), 'mailchimp_settings_updated');
                    flash('user_message', 'settings', 'mailchimp_settings_updated');
                } else {
                    flash('user_message', 'settings', 'mailchimp_settings_update_failed', 'danger');
                }
            }
            redirect('settings/email_settings');
            die();
        }
        
        public function update_sendgrid_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'sendgrid_live_enabled',
                    'option_autoload' => cleanString($_POST['option_autoload']),
                    'option_value' => cleanString($_POST['option_autoload']),
                    'sendgrid_api_key' => cleanString($_POST['sendgrid_api_key']),
                ];
                if ($this->settingModel->updateOptions($data) && $this->settingModel->updatesendgridSettings($data) && $this->settingModel->updateOptionsAutoload($data)) {
                    addActivityLog('setting', user_id(), 'sendgrid_settings_updated');
                    flash('user_message', 'settings', 'sendgrid_settings_updated');
                } else {
                    flash('user_message', 'settings', 'sendgrid_settings_update_failed', 'danger');
                }
            }
            redirect('settings/email_settings');
            die();
        }
        
        public function email_templates()
        {
            $users_templates = $this->settingModel->getEmailTemplatesByItem('users');
            $subscriptions_templates = $this->settingModel->getEmailTemplatesByItem('subscriptions');
            $affiliates_templates = $this->settingModel->getEmailTemplatesByItem('affiliates');
            $payouts_templates = $this->settingModel->getEmailTemplatesByItem('payouts');
            $data = [
                'users_templates' => $users_templates,
                'subscriptions_templates' => $subscriptions_templates,
                'affiliates_templates' => $affiliates_templates,
                'payouts_templates' => $payouts_templates
            ];
            $this->view('settings/email_templates', $data);
        }
        
        public function use_email()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'use_email',
                    'option_value' => cleanString($_POST['use_email'])
                ];
                if (!validate_email($data['use_email']) && $this->settingModel->updateOptions($data)) {
                    addActivityLog('user', user_id(), 'use_email_settings_updated');
                    flash('user_message', 'settings', 'use_email_settings_updated');
                } else {
                    flash('user_message', 'settings', 'use_email_settings_update_failed', 'danger');
                }
            }
            redirect('settings/email_templates');
            die();
        }
        
        public function email_template($id = NULL)
        {
            $email_template = $this->settingModel->getEmailTemplateById($id);
            if (empty($email_template)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'email_template_id' => $id,
                    'email_subject' => cleanString($_POST['email_subject']),
                    'email_message' => cleanString($_POST['email_message'], 'ignore_tags')
                ];
                if (empty($data['email_subject'])) {
                    $data['input_error'][] = 'please_enter_template_subject';
                }
                if (empty($data['email_message'])) {
                    $data['input_error'][] = 'please_enter_template_message';
                }
                if (empty($data['input_error'])) {
                    if ($this->settingModel->updateEmailTemplate($data)) {
                        addActivityLog('user', user_id(), 'email_template_updated');
                        flash('user_message', 'settings', 'email_template_updated');
                    } else {
                        flash('user_message', 'settings', 'email_template_update_failed', 'danger');
                    }
                    redirect('settings/email_template/' . $id);
                    die();
                } else {
                    $this->view('settings/email_template', $data);
                }
            }
            $data = [
                'email_template' => $email_template
            ];
            $this->view('settings/email_template', $data);
        }
        
        public function enable_template($id = NULL)
        {
            $email_template = $this->settingModel->getEmailTemplateById($id);
            if (empty($email_template)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if ($this->settingModel->enableTemplate($id)) {
                    addActivityLog('user', user_id(), 'email_template_enabled');
                    flash('user_message', 'settings', 'email_template_enabled');
                } else {
                    flash('user_message', 'settings', 'email_template_enabling_failed', 'danger');
                }
                redirect('settings/email_templates');
                die();
            } else {
                redirect('settings/email_templates');
            }
        }
        
        public function disable_template($id = NULL)
        {
            $email_template = $this->settingModel->getEmailTemplateById($id);
            if (empty($email_template)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if ($this->settingModel->disableTemplate($id)) {
                    addActivityLog('user', user_id(), 'email_template_disabled');
                    flash('user_message', 'settings', 'email_template_disabled');
                } else {
                    flash('user_message', 'settings', 'email_template_disabling_failed', 'danger');
                }
                redirect('settings/email_templates');
                die();
            } else {
                redirect('settings/email_templates');
            }
        }
        
        public function update_email_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'email_gateway',
                    'option_value' => cleanString($_POST['email_gateway'])
                ];
                if ($this->settingModel->updateOptions($data)) {
                    addActivityLog('user', user_id(), 'email_settings_updated');
                    flash('user_message', 'settings', 'email_settings_updated');
                } else {
                    flash('user_message', 'settings', 'email_settings_update_failed', 'danger');
                }
                redirect('settings/email_settings');
                die();
            }
            $this->view('settings/email_settings');
        }
        
        public function subscription_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
	            foreach ($_POST as $keys => $options) {
	                $data = [
	                    'option_name' => $keys,
	                    'option_value' => $options
	                ];
	                $insert = $this->settingModel->updateOptions($data);
	            }
	            if ($insert) {
	                addActivityLog('user', user_id(), 'subscription_settings_updated');
	                flash('user_message', 'settings', 'subscription_settings_updated');
	            } else {
	                flash('user_message', 'settings', 'subscription_settings_update_failed', 'danger');
	            }
	            redirect('settings/subscription_settings');
	            die();
            }
            $this->view('settings/subscription_settings');
        }
        
        public function datetime_formats()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'default_date_format' => cleanString($_POST['default_date_format']),
                    'default_datetime_format' => cleanString($_POST['default_datetime_format']),
                ];
                $stringD = str_split(onlyLetters($data['default_date_format']));
                foreach ($stringD as $character) {
                    if (!in_array($character, validDateFormatCharacters())) {
                        $data['default_datetime_format_err'] = 'invalid_date_format_character';
                    }
                }
                $stringDT = str_split(onlyLetters($data['default_datetime_format']));
                foreach ($stringDT as $character) {
                    if (!in_array($character, array_merge(validDateTimeFormatCharacters(), validDateFormatCharacters()))) {
                        $data['default_datetime_format_err'] = 'invalid_date_time_format_character';
                    }
                }
                if (empty($data['default_datetime_format_err'])) {
                    foreach ($data as $keys => $options) {
                        $data = [
                            'option_name' => $keys,
                            'option_value' => $options
                        ];
                        $insert = $this->settingModel->updateOptions($data);
                    }
                    if ($insert) {
                        addActivityLog('users', user_id(), 'date_time_formats_settings_updated');
                        flash('user_message', 'settings', 'date_time_formats_settings_updated');
                    } else {
                        flash('user_message', 'settings', 'date_time_settings_update_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'settings', $data['default_datetime_format_err'], 'danger');
                }
                redirect('settings/datetime_formats');
                die();
            }
            $this->view('settings/datetime_formats');
        }
        
        public function default_currency()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'default_currency' => cleanString($_POST['default_currency']),
                    'currency_symbol' => cleanString($_POST['currency_symbol']),
                    'use_currency_symbol' => isset($_POST['use_currency_symbol']) ? 1 : 0,
                    'currency_symbol_placement' => cleanString($_POST['currency_symbol_placement']),
                    'number_of_currency_decimals' => cleanString($_POST['number_of_currency_decimals'])
                ];
                foreach ($data as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'default_currency_settings_updated');
                    flash('user_message', 'settings', 'default_currency_settings_updated');
                } else {
                    flash('user_message', 'settings', 'default_currency_settings_update_failed', 'danger');
                }
                redirect('settings/default_currency');
                die();
            }
            $this->view('settings/default_currency');
        }
        
        public function default_currency_addon($iso = NULL)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $currency = $this->settingModel->getSupportedCurrencyByIso($iso);
                $data = [
                    'currency' => $currency
                ];
                $this->view('settings/defaultCurrencyAddon', $data);
            }
        }
        
        public function google_recaptcha_settings()
        {
            $google_recaptcha_settings = $this->settingModel->getGoogleRecaptchaSettings();
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'google_recaptcha_site_key' => cleanString($_POST['google_recaptcha_site_key']),
                    'google_recaptcha_secret_key' => cleanString($_POST['google_recaptcha_secret_key']),
                    'google_recaptcha_settings' => $google_recaptcha_settings
                ];
                if ($this->settingModel->updateGoogleRecaptchaSettings($data)) {
                    $data = [
                        'option_name' => 'enable_google_recaptcha',
                        'option_value' => isset($_POST['enable_google_recaptcha']) ? cleanString($_POST['enable_google_recaptcha']) : 0
                    ];
                    $this->settingModel->updateOptions($data);
                    addActivityLog('user', user_id(), 'google_recaptcha_settings_updated');
                    flash('user_message', 'settings', 'google_recaptcha_settings_updated');
                } else {
                    flash('user_message', 'settings', 'google_recaptcha_settings_update_failed', 'danger');
                }
                redirect('settings/google_recaptcha_settings');
                die();
            }
            $data = [
                'google_recaptcha_settings' => $google_recaptcha_settings
            ];
            $this->view('settings/google_recaptcha_settings', $data);
        }
        
        public function homepage_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'enable_homepage' => isset($_POST['enable_homepage']) ? 1 : 0,
                    'homepage_gradient_colour' => cleanString($_POST['homepage_gradient_colour']),
                ];
                foreach ($data as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_settings');
                die();
            }
            $this->view('settings/homepage_settings');
        }
        
        public function update_section_order()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                
                $i = 1;
                foreach (explode(',', $_GET['position']) as $k => $v) {
                    $data = [
                        'homepage_section_order' => $i,
                        'homepage_section_id' => $v
                    ];
                    $this->homepageModel->updateHomepageSectionOrder($data);
                    $i++;
                }
            }
        }
        
        public function homepage_section_status()
        {
            $status = 'error';
            $message = 'method_not_allowed';
            
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'homepage_section_id' => cleanString($_POST['homepage_section_id']),
                    'homepage_section_status_id' => cleanString($_POST['homepage_section_status_id'])
                ];
                if ($this->homepageModel->updateHomepageSectionStatus($data)) {
                    addActivityLog('homepage', $data['homepage_section_id'], 'homepage_section_status_updated');
                    $status = 'success';
                    $message = 'homepage_section_status_updated';
                } else {
                    $message = 'homepage_section_status_update_failed';
                }
            }
            $result = array(
                "status" => $status,
                "message" => $message
            );
            
            echo json_encode($result);
        }
        
        public function homepage_section_show_in_header()
        {
            $status = 'error';
            $message = 'method_not_allowed';
            
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'homepage_section_id' => cleanString($_POST['homepage_section_id']),
                    'homepage_section_show_in_header' => cleanString($_POST['homepage_section_show_in_header'])
                ];
                if ($this->homepageModel->updateHomepageSectionShowInHeader($data)) {
                    addActivityLog('homepage', $data['homepage_section_id'], 'homepage_show_in_header_settings_updated');
                    $status = 'success';
                    $message = 'homepage_show_in_header_settings_updated';
                } else {
                    $message = 'homepage_show_in_header_settings_failed';
                }
            }
            $result = array(
                "status" => $status,
                "message" => $message
            );
            
            echo json_encode($result);
        }
        
        public function homepage_intro()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_intro_background_type'] == 'static') {
                    $homepage_intro_background = $_POST['homepage_intro_background_static'];
                } elseif ($_POST['homepage_intro_background_type'] == 'gradient') {
                    $homepage_intro_background = $_POST['homepage_intro_background_gradient'];
                } elseif ($_POST['homepage_intro_background_type'] == 'image' && $_FILES['homepage_intro_background_image']['error'] == 0) {
                    $homepage_intro_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_intro_background_image"]["name"]);
                    any_file($homepage_intro_background, 'media', $_FILES['homepage_intro_background_image']['tmp_name']);
                } elseif ($_POST['homepage_intro_background_type'] == 'image' && $_FILES['homepage_intro_background_image']['error'] > 0) {
                    $homepage_intro_background = homepage_intro()->homepage_intro_background;
                }
                if (homepage_intro()->homepage_intro_background_type == 'image' && $_FILES['homepage_intro_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_intro()->homepage_intro_background);
                }
                $data = [
                    'homepage_intro_title' => cleanString($_POST['homepage_intro_title'], 'ignore_tags'),
                    'homepage_intro_description' => cleanString($_POST['homepage_intro_description'], 'ignore_tags'),
                    'homepage_intro_sub_description' => cleanString($_POST['homepage_intro_sub_description'], 'ignore_tags'),
                    'homepage_intro_image_text' => cleanString($_POST['homepage_intro_image_text'], 'ignore_tags'),
                    'homepage_intro_background_type' => cleanString($_POST['homepage_intro_background_type']),
                    'homepage_intro_background' => $homepage_intro_background,
                    'homepage_intro_id' => 1
                ];
                if ($this->homepageModel->updateHomepageIntro($data)) {
                    if (!empty($_FILES['homepage_intro_image']['name'])) {
                        $image_name = fileExists(APPROOT . MEDIA_FOLDER, $_FILES['homepage_intro_image']['name']);
                        upload_image($image_name, 'media', $_FILES['homepage_intro_image']['tmp_name']);
                        $dataX = [
                            'homepage_intro_image' => $image_name,
                            'homepage_intro_id' => 1
                        ];
                        $this->homepageModel->updateHomepageIntroImage($dataX);
                    }
                    for ($i = 0; $i < count($_POST['homepage_intro_item']); $i++) {
                        $data = [
                            'homepage_intro_item_id' => cleanString($_POST['homepage_intro_item_id'][$i]),
                            'homepage_intro_item' => cleanString($_POST['homepage_intro_item'][$i]),
                            'homepage_intro_item_value' => cleanString($_POST['homepage_intro_item_value'][$i])
                        ];
                        if (empty($data['homepage_intro_item_id']) && !empty($data['homepage_intro_item'])) {
                            $this->homepageModel->addHomepageIntroItem($data);
                        } elseif (!empty($data['homepage_intro_item_id']) && empty($data['homepage_intro_item'])) {
                            $this->homepageModel->deleteHomepageIntroItem($data);
                        } else {
                            $this->homepageModel->updateHomepageIntroItem($data);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_intro');
                die();
            }
            $this->view('settings/homepage_intro');
        }
        
        public function delete_homepage_intro_item($id = NULL)
        {
            $homepage_intro_item = homepage_intro_item($id);
            if (empty($homepage_intro_item)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_intro_item_id' => $id
                ];
                if ($this->homepageModel->deleteHomepageIntroItem($data)) {
                    addActivityLog('user', user_id(), 'homepage_intro_item_deleted');
                    flash('user_message', 'settings', 'homepage_intro_item_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_intro_item_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_intro');
            die();
        }
        
        public function homepage_features()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_features_background_type'] == 'static') {
                    $homepage_features_background = $_POST['homepage_features_background_static'];
                } elseif ($_POST['homepage_features_background_type'] == 'gradient') {
                    $homepage_features_background = $_POST['homepage_features_background_gradient'];
                } elseif ($_POST['homepage_features_background_type'] == 'image' && $_FILES['homepage_features_background_image']['error'] == 0) {
                    $homepage_features_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_features_background_image"]["name"]);
                    any_file($homepage_features_background, 'media', $_FILES['homepage_features_background_image']['tmp_name']);
                } elseif ($_POST['homepage_applications_background_type'] == 'image' && $_FILES['homepage_features_background_image']['error'] > 0) {
                    $homepage_features_background = homepage_features()->homepage_features_background;
                }
                if (homepage_features()->homepage_features_background_type == 'image' && $_FILES['homepage_features_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_features()->homepage_features_background);
                }
                $data = [
                    'homepage_features_heading' => cleanString($_POST['homepage_features_heading']),
                    'homepage_features_title' => cleanString($_POST['homepage_features_title']),
                    'homepage_features_background_type' => cleanString($_POST['homepage_features_background_type']),
                    'homepage_features_background' => $homepage_features_background,
                    'homepage_features_id' => 1
                ];
                if ($this->homepageModel->updateHomepageFeature($data)) {
                    if (!empty($_FILES['homepage_features_image']['name'])) {
                        $image_name = fileExists(APPROOT . MEDIA_FOLDER, $_FILES['homepage_features_image']['name']);
                        upload_image($image_name, 'media', $_FILES['homepage_features_image']['tmp_name']);
                        $dataX = [
                            'homepage_features_image' => $image_name,
                            'homepage_features_id' => 1
                        ];
                        $this->homepageModel->updateHomepageFeaturesImage($dataX);
                    }
                    for ($i = 0; $i < count($_POST['homepage_features_item']); $i++) {
                        $data = [
                            'homepage_features_item_id' => cleanString($_POST['homepage_features_item_id'][$i]),
                            'homepage_features_item' => cleanString($_POST['homepage_features_item'][$i]),
                            'homepage_features_item_description' => cleanString($_POST['homepage_features_item_description'][$i])
                        ];
                        if (empty($data['homepage_features_item_id']) && !empty($data['homepage_features_item'])) {
                            $this->homepageModel->addHomepageFeaturesItem($data);
                        } elseif (!empty($data['homepage_features_item_id']) && empty($data['homepage_features_item'])) {
                            $this->homepageModel->deleteHomepageFeaturesItem($data);
                        } else {
                            $this->homepageModel->updateHomepageFeaturesItem($data);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_features');
                die();
            }
            $this->view('settings/homepage_features');
        }
        
        public function delete_homepage_features_item($id = NULL)
        {
            $homepage_audio_item = homepage_features_item($id);
            if (empty($homepage_audio_item)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_features_item_id' => $id
                ];
                if ($this->homepageModel->deleteHomepageFeaturesItem($data)) {
                    addActivityLog('user', user_id(), 'homepage_features_item_deleted');
                    flash('user_message', 'settings', 'homepage_features_item_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_features_item_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_features');
            die();
        }
        
        public function homepage_applications()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_applications_background_type'] == 'static') {
                    $homepage_applications_background = $_POST['homepage_applications_background_static'];
                } elseif ($_POST['homepage_applications_background_type'] == 'gradient') {
                    $homepage_applications_background = $_POST['homepage_applications_background_gradient'];
                } elseif ($_POST['homepage_applications_background_type'] == 'image' && $_FILES['homepage_applications_background_image']['error'] == 0) {
                    $homepage_applications_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_applications_background_image"]["name"]);
                    any_file($homepage_applications_background, 'media', $_FILES['homepage_applications_background_image']['tmp_name']);
                } elseif ($_POST['homepage_applications_background_type'] == 'image' && $_FILES['homepage_applications_background_image']['error'] > 0) {
                    $homepage_applications_background = homepage_applications()->homepage_applications_background;
                }
                if (homepage_applications()->homepage_applications_background_type == 'image' && $_FILES['homepage_applications_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_applications()->homepage_applications_background);
                }
                $data = [
                    'homepage_applications_heading' => cleanString($_POST['homepage_applications_heading']),
                    'homepage_applications_title' => cleanString($_POST['homepage_applications_title']),
                    'homepage_applications_background_type' => cleanString($_POST['homepage_applications_background_type']),
                    'homepage_applications_background' => $homepage_applications_background,
                    'homepage_applications_id' => 1
                ];
                if ($this->homepageModel->updateHomepageApplications($data)) {
                    for ($i = 0; $i < count($_POST['homepage_applications_item']); $i++) {
                        $data = [
                            'homepage_applications_item_id' => cleanString($_POST['homepage_applications_item_id'][$i]),
                            'homepage_applications_item' => cleanString($_POST['homepage_applications_item'][$i]),
                            'homepage_applications_item_icon' => cleanString($_POST['homepage_applications_item_icon'][$i], 'ignore_tags')
                        ];
                        if (empty($data['homepage_applications_item_id']) && !empty($data['homepage_applications_item'])) {
                            $this->homepageModel->addHomepageapplicationsItem($data);
                        } elseif (!empty($data['homepage_applications_item_id']) && empty($data['homepage_applications_item'])) {
                            $this->homepageModel->deleteHomepageapplicationsItem($data);
                        } else {
                            $this->homepageModel->updateHomepageapplicationsItem($data);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_applications');
                die();
            }
            $this->view('settings/homepage_applications');
        }
        
        public function delete_homepage_applications_item($id = NULL)
        {
            $homepage_audio_item = homepage_applications_item($id);
            if (empty($homepage_audio_item)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_applications_item_id' => $id
                ];
                if ($this->homepageModel->deleteHomepageapplicationsItem($data)) {
                    addActivityLog('user', user_id(), 'homepage_application_item_deleted');
                    flash('user_message', 'settings', 'homepage_application_item_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_application_item_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_applications');
            die();
        }
        
        public function homepage_howitworks()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_howitworks_background_type'] == 'static') {
                    $homepage_howitworks_background = $_POST['homepage_howitworks_background_static'];
                } elseif ($_POST['homepage_howitworks_background_type'] == 'gradient') {
                    $homepage_howitworks_background = $_POST['homepage_howitworks_background_gradient'];
                } elseif ($_POST['homepage_howitworks_background_type'] == 'image' && $_FILES['homepage_howitworks_background_gradient']['error'] == 0) {
                    $homepage_howitworks_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_howitworks_background_gradient"]["name"]);
                    any_file($homepage_howitworks_background, 'media', $_FILES['homepage_howitworks_background_gradient']['tmp_name']);
                } elseif ($_POST['homepage_howitworks_background_type'] == 'image' && $_FILES['homepage_howitworks_background_gradient']['error'] > 0) {
                    $homepage_howitworks_background = homepage_howitworks()->homepage_howitworks_background;
                }
                if (homepage_howitworks()->homepage_howitworks_background_type == 'image' && $_FILES['homepage_howitworks_background_gradient']['error'] == 0) {
                    unlinkUploads('media', homepage_howitworks()->homepage_howitworks_background);
                }
                $data = [
                    'homepage_howitworks_heading' => cleanString($_POST['homepage_howitworks_heading']),
                    'homepage_howitworks_title' => cleanString($_POST['homepage_howitworks_title']),
                    'homepage_howitworks_description' => cleanString($_POST['homepage_howitworks_description']),
                    'homepage_howitworks_background_type' => cleanString($_POST['homepage_howitworks_background_type']),
                    'homepage_howitworks_background' => $homepage_howitworks_background,
                    'homepage_howitworks_id' => 1
                ];
                if ($this->homepageModel->updateHomepageHowitworks($data)) {
                    for ($i = 0; $i <= count($_POST['homepage_howitworks_step_title']); $i++) {
                        $dataX = [
                            'homepage_howitworks_step_icon' => cleanString($_POST['homepage_howitworks_step_icon'][$i], 'ignore_tags'),
                            'homepage_howitworks_step_title' => cleanString($_POST['homepage_howitworks_step_title'][$i]),
                            'homepage_howitworks_step_description' => cleanString($_POST['homepage_howitworks_step_description'][$i], 'ignore_tags'),
                            'homepage_howitworks_step_id' => cleanString($_POST['homepage_howitworks_step_id'][$i])
                        ];
                        if (empty($dataX['homepage_howitworks_step_id']) && !empty($dataX['homepage_howitworks_step_title'])) {
                            $this->homepageModel->addHomepageHowitworksStep($dataX);
                        } elseif (!empty($dataX['homepage_howitworks_step_id']) && empty($dataX['homepage_howitworks_step_title'])) {
                            $this->homepageModel->deleteHomepageHowitworksStep($dataX);
                        } else {
                            $this->homepageModel->updateHomepageHowitworksStep($dataX);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_howitworks');
                die();
            }
            $this->view('settings/homepage_howitworks');
        }
        
        public function delete_homepage_howitworks_step($id = NULL)
        {
            $homepage_howitworks_step = homepage_howitworks_step($id);
            if (empty($homepage_howitworks_step)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_howitworks_step_id' => $id
                ];
                if ($this->homepageModel->deleteHomepageHowitworksStep($data)) {
                    addActivityLog('user', user_id(), 'homepage_howitworks_step_deleted');
                    flash('user_message', 'settings', 'homepage_howitworks_step_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_howitworks_step_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_howitworks');
            die();
        }
        
        public function homepage_pricing()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_pricing_background_type'] == 'static') {
                    $homepage_pricing_background = $_POST['homepage_pricing_background_static'];
                } elseif ($_POST['homepage_pricing_background_type'] == 'gradient') {
                    $homepage_pricing_background = $_POST['homepage_pricing_background_gradient'];
                } elseif ($_POST['homepage_pricing_background_type'] == 'image' && $_FILES['homepage_pricing_background_image']['error'] == 0) {
                    $homepage_pricing_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_pricing_background_image"]["name"]);
                    any_file($homepage_pricing_background, 'media', $_FILES['homepage_pricing_background_image']['tmp_name']);
                } elseif ($_POST['homepage_pricing_background_type'] == 'image' && $_FILES['homepage_pricing_background_image']['error'] > 0) {
                    $homepage_pricing_background = homepage_pricing()->homepage_pricing_background;
                }
                if (homepage_pricing()->homepage_pricing_background_type == 'image' && $_FILES['homepage_pricing_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_pricing()->homepage_pricing_background);
                }
                $data = [
                    'homepage_pricing_heading' => cleanString($_POST['homepage_pricing_heading']),
                    'homepage_pricing_title' => cleanString($_POST['homepage_pricing_title']),
                    'homepage_pricing_background_type' => cleanString($_POST['homepage_pricing_background_type']),
                    'homepage_pricing_background' => $homepage_pricing_background,
                    'homepage_pricing_id' => 1
                ];
                if ($this->homepageModel->updateHomepagePricing($data)) {
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_pricing');
                die();
            }
            $this->view('settings/homepage_pricing');
        }
        
        public function homepage_trust()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_trust_background_type'] == 'static') {
                    $homepage_trust_background = $_POST['homepage_trust_background_static'];
                } elseif ($_POST['homepage_trust_background_type'] == 'gradient') {
                    $homepage_trust_background = $_POST['homepage_trust_background_gradient'];
                } elseif ($_POST['homepage_trust_background_type'] == 'image' && $_FILES['homepage_trust_background_image']['error'] == 0) {
                    $homepage_trust_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_trust_background_image"]["name"]);
                    any_file($homepage_trust_background, 'media', $_FILES['homepage_trust_background_image']['tmp_name']);
                } elseif ($_POST['homepage_trust_background_type'] == 'image' && $_FILES['homepage_trust_background_image']['error'] > 0) {
                    $homepage_trust_background = homepage_trust()->homepage_trust_background;
                }
                if (homepage_trust()->homepage_trust_background_type == 'image' && $_FILES['homepage_trust_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_trust()->homepage_trust_background);
                }
                $data = [
                    'homepage_trust_heading' => cleanString($_POST['homepage_trust_heading']),
                    'homepage_trust_title' => cleanString($_POST['homepage_trust_title']),
                    'homepage_trust_description' => cleanString($_POST['homepage_trust_description'], 'ignore_tags'),
                    'homepage_trust_background_type' => cleanString($_POST['homepage_trust_background_type']),
                    'homepage_trust_background' => $homepage_trust_background,
                    'homepage_trust_id' => 1
                ];
                if ($this->homepageModel->updateHomepageTrust($data)) {
                    if (!empty($_FILES['homepage_trust_image']['name'])) {
                        $image_name = fileExists(APPROOT . MEDIA_FOLDER, $_FILES['homepage_trust_image']['name']);
                        upload_image($image_name, 'media', $_FILES['homepage_trust_image']['tmp_name']);
                        $dataX = [
                            'homepage_trust_image' => $image_name,
                            'homepage_trust_id' => 1
                        ];
                        $this->homepageModel->updateHomepageTrustImage($dataX);
                    }
                    for ($i = 0; $i < count($_POST['homepage_trust_item']); $i++) {
                        $data = [
                            'homepage_trust_item_id' => cleanString($_POST['homepage_trust_item_id'][$i]),
                            'homepage_trust_item' => cleanString($_POST['homepage_trust_item'][$i])
                        ];
                        if (empty($data['homepage_trust_item_id']) && !empty($data['homepage_trust_item'])) {
                            $this->homepageModel->addHomepagetrustItem($data);
                        } elseif (!empty($data['homepage_trust_item_id']) && empty($data['homepage_trust_item'])) {
                            $this->homepageModel->deleteHomepagetrustItem($data);
                        } else {
                            $this->homepageModel->updateHomepagetrustItem($data);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_trust');
                die();
            }
            $this->view('settings/homepage_trust');
        }
        
        public function delete_homepage_trust_item($id = NULL)
        {
            $homepage_audio_item = homepage_trust_item($id);
            if (empty($homepage_audio_item)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_trust_item_id' => $id
                ];
                if ($this->homepageModel->deleteHomepagetrustItem($data)) {
                    addActivityLog('user', user_id(), 'homepage_audio_item_deleted');
                    flash('user_message', 'settings', 'homepage_audio_item_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_audio_item_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_trust');
            die();
        }
        
        public function homepage_faqs()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_faqs_background_type'] == 'static') {
                    $homepage_faqs_background = $_POST['homepage_faqs_background_static'];
                } elseif ($_POST['homepage_faqs_background_type'] == 'gradient') {
                    $homepage_faqs_background = $_POST['homepage_faqs_background_gradient'];
                } elseif ($_POST['homepage_faqs_background_type'] == 'image' && $_FILES['homepage_faqs_background_image']['error'] == 0) {
                    $homepage_faqs_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_faqs_background_image"]["name"]);
                    any_file($homepage_faqs_background, 'media', $_FILES['homepage_faqs_background_image']['tmp_name']);
                } elseif ($_POST['homepage_faqs_background_type'] == 'image' && $_FILES['homepage_faqs_background_image']['error'] > 0) {
                    $homepage_faqs_background = homepage_faqs()->homepage_faqs_background;
                }
                if (homepage_faqs()->homepage_faqs_background_type == 'image' && $_FILES['homepage_faqs_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_faqs()->homepage_faqs_background);
                }
                $data = [
                    'homepage_faqs_heading' => cleanString($_POST['homepage_faqs_heading']),
                    'homepage_faqs_title' => cleanString($_POST['homepage_faqs_title']),
                    'homepage_faqs_background_type' => cleanString($_POST['homepage_faqs_background_type']),
                    'homepage_faqs_background' => $homepage_faqs_background,
                    'homepage_faqs_id' => 1
                ];
                if ($this->homepageModel->updateHomepageFaqs($data)) {
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_faqs');
                die();
            }
            $this->view('settings/homepage_faqs');
        }
        
        public function homepage_footer_block()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_footer_block_background_type'] == 'static') {
                    $homepage_footer_block_background = $_POST['homepage_footer_block_background_static'];
                } elseif ($_POST['homepage_footer_block_background_type'] == 'gradient') {
                    $homepage_footer_block_background = $_POST['homepage_footer_block_background_gradient'];
                } elseif ($_POST['homepage_footer_block_background_type'] == 'image' && $_FILES['homepage_footer_block_background_image']['error'] == 0) {
                    $homepage_footer_block_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_footer_block_background_image"]["name"]);
                    any_file($homepage_footer_block_background, 'media', $_FILES['homepage_footer_block_background_image']['tmp_name']);
                } elseif ($_POST['homepage_footer_block_background_type'] == 'image' && $_FILES['homepage_footer_block_background_image']['error'] > 0) {
                    $homepage_footer_block_background = homepage_footer_block()->homepage_footer_block_background;
                }
                if (homepage_footer_block()->homepage_footer_block_background_type == 'image' && $_FILES['homepage_footer_block_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_footer_block()->homepage_footer_block_background);
                }
                $data = [
                    'homepage_footer_block_heading' => cleanString($_POST['homepage_footer_block_heading']),
                    'homepage_footer_block_title' => cleanString($_POST['homepage_footer_block_title']),
                    'homepage_footer_block_link' => cleanString($_POST['homepage_footer_block_link']),
                    'homepage_footer_block_link_description' => cleanString($_POST['homepage_footer_block_link_description']),
                    'homepage_footer_block_background_type' => cleanString($_POST['homepage_footer_block_background_type']),
                    'homepage_footer_block_background' => $homepage_footer_block_background,
                    'homepage_footer_block_id' => 1
                ];
                
                if ($this->homepageModel->updateHomepageFooterBlock($data)) {
                    for ($i = 0; $i < count($_POST['homepage_footer_block_feature_description']); $i++) {
                        $dataX = [
                            'homepage_footer_block_feature_description' => cleanString($_POST['homepage_footer_block_feature_description'][$i]),
                            'homepage_footer_block_feature_id' => cleanString($_POST['homepage_footer_block_feature_id'][$i])
                        ];
                        if (empty($dataX['homepage_footer_block_feature_id']) && !empty($dataX['homepage_footer_block_feature_description'])) {
                            $this->homepageModel->addHomepageFooterBlockFeature($dataX);
                        } elseif (!empty($dataX['homepage_footer_block_feature_id']) && empty($dataX['homepage_footer_block_feature_description'])) {
                            $this->homepageModel->deleteHomepageFooterBlockFeature($dataX);
                        } else {
                            $this->homepageModel->updateHomepageFooterBlockFeature($dataX);
                        }
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_footer_block');
                die();
            }
            $this->view('settings/homepage_footer_block');
        }
        
        public function delete_homepage_footer_block_feature($id = NULL)
        {
            $homepage_footer_block_feature = homepage_footer_block_feature($id);
            if (empty($homepage_footer_block_feature)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $data = [
                    'homepage_footer_block_feature_id' => $id
                ];
                if ($this->homepageModel->deleteHomepageFooterBlockFeature($data)) {
                    addActivityLog('user', user_id(), 'homepage_footer_block_feature_deleted');
                    flash('user_message', 'settings', 'homepage_footer_block_feature_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_footer_block_feature_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_footer_block');
            die();
        }
        
        public function homepage_blog()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                if ($_POST['homepage_blog_background_type'] == 'static') {
                    $homepage_blog_background = $_POST['homepage_blog_background_static'];
                } elseif ($_POST['homepage_blog_background_type'] == 'gradient') {
                    $homepage_blog_background = $_POST['homepage_blog_background_gradient'];
                } elseif ($_POST['homepage_blog_background_type'] == 'image' && $_FILES['homepage_blog_background_image']['error'] == 0) {
                    $homepage_blog_background = fileExists(APPROOT . MEDIA_FOLDER, $_FILES["homepage_blog_background_image"]["name"]);
                    any_file($homepage_blog_background, 'media', $_FILES['homepage_blog_background_image']['tmp_name']);
                } elseif ($_POST['homepage_blog_background_type'] == 'image' && $_FILES['homepage_blog_background_image']['error'] > 0) {
                    $homepage_blog_background = homepage_blog()->homepage_blog_background;
                }
                if (homepage_blog()->homepage_blog_background_type == 'image' && $_FILES['homepage_blog_background_image']['error'] == 0) {
                    unlinkUploads('media', homepage_blog()->homepage_blog_background);
                }
                $data = [
                    'homepage_blog_heading' => cleanString($_POST['homepage_blog_heading']),
                    'homepage_blog_title' => cleanString($_POST['homepage_blog_title']),
                    'homepage_blog_description' => cleanString($_POST['homepage_blog_description']),
                    'homepage_blog_background_type' => cleanString($_POST['homepage_blog_background_type']),
                    'homepage_blog_background' => $homepage_blog_background,
                    'homepage_blog_id' => 1
                ];
                if ($this->homepageModel->updateHomepageBlog($data)) {
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_blog');
                die();
            }
            $this->view('settings/homepage_blog');
        }
        
        public function affiliate_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'enable_affiliate' => isset($_POST['enable_affiliate']) ? cleanString($_POST['enable_affiliate']) : 0,
                    'affiliate_rule_id' => cleanString($_POST['affiliate_rule_id']),
                    'affiliate_commission_rate' => cleanString($_POST['affiliate_commission_rate']),
                    'affiliate_minimum_payout' => cleanString($_POST['affiliate_minimum_payout']),
                    'affiliate_action_phrase' => cleanString($_POST['affiliate_action_phrase'], 'ignore_tags'),
                    'affiliate_guidelines' => cleanString($_POST['affiliate_guidelines'], 'ignore_tags')
                ];
                foreach ($data as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'affiliate_settings_updated');
                    flash('user_message', 'settings', 'affiliate_settings_updated');
                } else {
                    flash('user_message', 'settings', 'affiliate_settings_update_failed', 'danger');
                }
                redirect('settings/affiliate_settings');
                die();
            }
            $this->view('settings/affiliate_settings');
        }
        
        public function update_homepage_image()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $data = [
                    'company_banner' => !empty($_FILES['file']['name']) ? trim($_FILES['file']['name']) : company(1)->company_banner,
                    'company_id' => 1
                ];
                if ($this->companyModel->updateCompanyHomepageImage($data)) {
                    if (!empty($_FILES['file']['name'])) {
                        $image_name = fileExists(APPROOT . MEDIA_FOLDER, $data['company_banner']);
                        upload_image($image_name, 'media', $_FILES['file']['tmp_name']);
                    }
                    addActivityLog('user', user_id(), 'homepage_settings_updated');
                    flash('user_message', 'settings', 'homepage_settings_updated');
                } else {
                    flash('user_message', 'settings', 'homepage_settings_update_failed', 'danger');
                }
                redirect('settings/homepage_settings');
                die();
            }
            $this->view('settings/homepage_settings');
        }
        
        public function company()
        {
            $this->view('settings/company');
        }
        
        public function update_company_logo()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'image' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['image']['name'])),
                    'current_logo' => company(1)->company_logo,
                    'company_id' => 1
                ];
                if (upload_image($data['image'], 'media', $_FILES['image']['tmp_name'])) {
                    $this->companyModel->updateCompanyLogo($data);
                    unlinkUploads('media', $data['current_logo']);
                    addActivityLog('user', user_id(), 'company_logo_updated');
                    flash('user_message', 'settings', 'company logo_updated');
                } else {
                    flash('user_message', 'settings', 'update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function update_company_profile()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'company_name' => cleanString($_POST['company_name']),
                    'company_street' => cleanString($_POST['company_street']),
                    'company_city' => cleanString($_POST['company_city']),
                    'company_state' => cleanString($_POST['company_state']),
                    'company_zip' => cleanString($_POST['company_zip']),
                    'company_website' => cleanString($_POST['company_website']),
                    'company_email' => cleanString($_POST['company_email']),
                    'company_phone' => cleanString($_POST['company_phone']),
                    'company_country_id' => cleanString($_POST['company_country_id']),
                    'company_tax' => cleanString($_POST['company_tax']),
                    'company_title' => NULL,
                    'company_subtitle' => NULL,
                    'company_website_title' => cleanString($_POST['company_website_title']),
                    'company_website_description' => cleanString($_POST['company_website_description']),
                    'company_website_keywords' => cleanString($_POST['company_website_keywords']),
                    'company_id' => 1
                ];
                if ($this->companyModel->updateCompany($data)) {
                    addActivityLog('user', user_id(), 'company_profile_updated');
                    flash('user_message', 'settings', 'company_profile_updated');
                } else {
                    flash('user_message', 'settings', 'company_profile_update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function update_company_website_logo()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'image' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['image']['name'])),
                    'company_id' => 1,
                    'current_logo' => company(1)->company_website_logo
                ];
                if (upload_image($data['image'], 'media', $_FILES['image']['tmp_name'])) {
                    $this->companyModel->updateCompanyWebsiteLogo($data);
                    unlinkUploads('media', $data['current_logo']);
                    addActivityLog('user', user_id(), 'company_logo_updated');
                    flash('user_message', 'settings', 'company logo_updated');
                } else {
                    flash('user_message', 'settings', 'update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function update_company_website_favicon()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'image' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['image']['name'])),
                    'company_id' => 1,
                    'current_favicon' => company(1)->company_website_favicon
                ];
                if (upload_image($data['image'], 'media', $_FILES['image']['tmp_name'])) {
                    $this->companyModel->updateCompanyWebsiteFavicon($data);
                    unlinkUploads('media', $data['current_favicon']);
                    addActivityLog('user', user_id(), 'favicon_updated');
                    flash('user_message', 'settings', 'favicon_updated');
                } else {
                    flash('user_message', 'settings', 'favicon_update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function update_system_logo()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'option_name' => 'default_system_logo',
                    'option_value' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['image']['name'])),
                    'current_system' => getOption('default_system_logo')
                ];
                if (upload_image($data['option_value'], 'media', $_FILES['image']['tmp_name'])) {
                    $this->settingModel->updateOptions($data);
                    unlinkUploads('media', $data['current_system']);
                    addActivityLog('user', user_id(), 'system_logo_updated');
                    flash('user_message', 'settings', 'system_logo_updated');
                } else {
                    flash('user_message', 'settings', 'system_logo_update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function delete_system_logo()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                $data = [
                    'option_name' => 'default_system_logo',
                    'option_value' => 'default_logo.png'
                ];
                if ($this->settingModel->updateOptions($data)) {
                    if ($this->settingModel->getOptions('default_system_logo')->option_value != 'default_logo.png') {
                        unlinkUploads('media', $this->settingModel->getOptions('default_system_logo')->option_value);
                    }
                    addActivityLog('user', user_id(), 'system_logo_deleted');
                    flash('user_message', 'settings', 'system_logo_deleted');
                } else {
                    flash('user_message', 'settings', 'system_logo_delete_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function delete_system_favicon()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                $data = [
                    'option_name' => 'default_system_favicon',
                    'option_value' => 'default_favicon.png'
                ];
                if ($this->settingModel->updateOptions($data)) {
                    if ($this->settingModel->getOptions('default_system_favicon')->option_value != 'default_favicon.png') {
                        unlinkUploads('media', $this->settingModel->getOptions('default_system_favicon')->option_value);
                    }
                    addActivityLog('user', user_id(), 'favicon_deleted');
                    flash('user_message', 'settings', 'favicon_deleted');
                } else {
                    flash('user_message', 'settings', 'favicon_delete_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function delete_company_logo()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->settingModel->deleteCompanyLogo()) {
                    if (company(company_id())->company_logo != 'default_company.png') {
                        unlinkUploads('media', company(company_id())->company_logo);
                    }
                    addActivityLog('user', user_id(), 'company_logo_deleted');
                    flash('user_message', 'settings', 'company_logo_deleted');
                } else {
                    flash('user_message', 'settings', 'company_logo_delete_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function update_social_profiles()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                for ($i = 0; $i <= count(array_filter($_POST['social_profile_link'])); $i++) {
                    $data = [
                        'social_profile_id' => !empty($_POST['social_profile_id'][$i]) ? cleanString($_POST['social_profile_id'][$i]) : '',
                        'social_profile_name' => cleanString($_POST['social_profile_name'][$i]),
                        'social_profile_link' => cleanString($_POST['social_profile_link'][$i]),
                        'social_profile_icon' => cleanString($_POST['social_profile_icon'][$i], 'ignore_tags'),
                        'social_profile_added_by' => user_id(),
                        'social_profile_status_id' => 2,
                    ];
                    if (empty($data['social_profile_id']) && !empty($data['social_profile_link'])) {
                        $insert = $this->settingModel->addSocialProfile($data);
                    } elseif (!empty($data['social_profile_id']) && !empty($data['social_profile_link'])) {
                        $insert = $this->settingModel->updateSocialProfile($data);
                    }
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'social_profiles_updated');
                    flash('user_message', 'settings', 'social_profiles_updated');
                } else {
                    flash('user_message', 'settings', 'social_profiles_update_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function delete_social_profile($social_profile_id)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->settingModel->deleteSocialProfile($social_profile_id)) {
                    addActivityLog('user', user_id(), 'social_profile_deleted');
                    flash('user_message', 'settings', 'social_profile_deleted');
                } else {
                    flash('user_message', 'settings', 'social_profile_delete_failed', 'danger');
                }
            }
            redirect('settings/company');
            die();
        }
        
        public function delete_homepage_feature($id)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->homepageModel->deleteHomepageFeature($id)) {
                    addActivityLog('user', user_id(), 'homepage_feature_deleted');
                    flash('user_message', 'settings', 'homepage_feature_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_feature_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_settings');
            die();
        }
        
        public function delete_homepage_testimonial($id)
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if ($this->homepageModel->deleteHomepageTestimonial($id)) {
                    addActivityLog('user', user_id(), 'homepage_testimonial_deleted');
                    flash('user_message', 'settings', 'homepage_testimonial_deleted');
                } else {
                    flash('user_message', 'settings', 'homepage_testimonial_delete_failed', 'danger');
                }
            }
            redirect('settings/homepage_settings');
            die();
        }
        
        public function ai_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                foreach ($_POST as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'ai_settings_updated');
                    flash('user_message', 'settings', 'ai_settings_updated');
                } else {
                    flash('user_message', 'settings', 'ai_settings_update_failed', 'danger');
                }
                redirect('settings/ai_settings');
                die();
            }
            $this->view('settings/ai_settings');
        }
        
        public function general_settings()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                if (!empty($_FILES['not_found_public_image']['name'])) {
                    
                    unlinkUploads('media', getOption('not_found_public_image'));
                    
                    $not_found_public_image = fileExists(APPROOT . MEDIA_FOLDER, $_FILES['not_found_public_image']['name']);
                    upload_image($not_found_public_image, 'media', $_FILES['not_found_public_image']['tmp_name']);
                    $data = [
                        'option_name' => 'not_found_public_image',
                        'option_value' => $not_found_public_image
                    ];
                    $this->settingModel->updateOptions($data);
                }
                
                foreach ($_POST as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    if (!empty($this->settingModel->getOptions($keys))) {
                        $insert = $this->settingModel->updateOptions($data);
                    }
                }
                
                for ($i = 0; $i < count($_POST['item_id']); $i++) {
                    $datax = [
                        'item_rgb_colour' => rgb2hex2rgb($_POST['item_hex_colour'][$i])
                    ];
                    $data = [
                        'item_id' => cleanString($_POST['item_id'][$i]),
                        'item_hex_colour' => cleanString($_POST['item_hex_colour'][$i]),
                        'item_rgb_colour' => $datax['item_rgb_colour']['r'] . ',' . $datax['item_rgb_colour']['g'] . ',' . $datax['item_rgb_colour']['b']
                    ];
                    $update = $this->settingModel->updateItemSettings($data);
                }
                
                if (!isset($_POST['create_accounts'])) {
                    $data = [
                        'option_name' => 'create_accounts',
                        'option_value' => 0
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                
                if (!isset($_POST['email_custom_errors'])) {
                    $data = [
                        'option_name' => 'email_custom_errors',
                        'option_value' => 0
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                
                if (!isset($_POST['htmlentities_decode_enabled'])) {
                    $data = [
                        'option_name' => 'htmlentities_decode_enabled',
                        'option_value' => 0
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                
                if (!isset($_POST['enable_google_recaptcha'])) {
                    $data = [
                        'option_name' => 'enable_google_recaptcha',
                        'option_value' => 0
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                //google recaptcha
                $data = [
                    'google_recaptcha_site_key' => cleanString($_POST['google_recaptcha_site_key']),
                    'google_recaptcha_secret_key' => cleanString($_POST['google_recaptcha_secret_key'])
                ];
                $insert = $this->settingModel->updateGoogleRecaptchaSettings($data);
                
                file_put_contents(APPROOT . MEDIA_FOLDER . 'manifest.json', $_POST['pwa_settings']);
                
                if ($insert) {
                    addActivityLog('user', user_id(), 'general_settings_updated');
                    flash('user_message', 'settings', 'general_settings_updated');
                } else {
                    flash('user_message', 'settings', 'general_settings_update_failed', 'danger');
                }
                redirect('settings/general_settings');
                die();
            }
            $this->view('settings/general_settings');
        }
        
        
    }
