<?php
    
    
    class Subscriptions extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index($type = NULL)
        {
            $data = [
                'subscriptions' => subscriptions()
            ];
            $this->view('subscriptions/index', $data);
        }
        
        public function show($id = NULL)
        {
            $subscription = subscription($id);
            if (!isSuperAdmin() && $subscription->user_id != user_id()) {
                redirect(404);
                die();
            }
            if (empty($subscription)) {
                redirect(404);
                die();
            }
            $data = [
                'subscription' => $subscription
            ];
            $this->view('subscriptions/show', $data);
        }
        
        public function order_code_search(){
        	
			$searchTerm = $_GET['term'];

			$result = $this->subscriptionModel->getSubscriptionPaymentByOrderCode($searchTerm);

			$data = [];
			if (!empty($result)) {
			    foreach($result as $subscription){
			        $data[] = [
			            'subscription_id' => $subscription->subscription_id,
			            'order_code' => $subscription->order_code
			        ];
			    }
			}

			echo json_encode($data);
		}
        
        public function approve($id = NULL)
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $subscription = subscription($id);
            if (empty($subscription)) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if (in_array($subscription->subscription_status_id, array('2', '3'))) {
                    flash('user_message', 'subscriptions', 'subscription_already_processed', 'danger');
                } else {
                    $data = [
                        'subscription_id' => $id,
                        'subscription_status_id' => '2',
                        'subscription_date_activated' => now(),
                        'subscription' => $subscription
                    ];
                    if ($this->subscriptionModel->updateSubscriptionStatus($data)) {
                        $this->subscriptionModel->updateSubscriptionDateActivated($data);
                        if (affiliate_accepted($id)) {
                            $dataA = [
                                'user_id' => user($subscription->user_id)->referred_by,
                                'affiliate_rule_id' => getOption('affiliate_rule_id'),
                                'affiliate_commission_rate' => getOption('affiliate_commission_rate'),
                                'subscription_id' => $id
                            ];
                            $this->affiliateModel->addAffiliate($dataA);
                        }
                        $dataw = [
                            'user_id' => $subscription->user_id,
                            'user_remaining_words' => add_user_remaining_balances($subscription->user_id, $id, 'words'),
                            'user_remaining_chat_assistants' => add_user_remaining_balances($subscription->user_id, $id, 'chat_assistants'),
                            'user_remaining_trained_models' => add_user_remaining_balances($subscription->user_id, $id, 'trained_models'),
                            'user_remaining_chat_files' => add_user_remaining_balances($subscription->user_id, $id, 'chat_files'),
                            'user_remaining_chat_links' => add_user_remaining_balances($subscription->user_id, $id, 'chat_links'),
                            'user_remaining_chat_sitemaps' => add_user_remaining_balances($subscription->user_id, $id, 'chat_sitemaps')
                        ];
                        foreach (package_types() as $type) {
                            $out_type = 'user_remaining_' . $type;
                            $dataz = [
                                'user_id' => $subscription->user_id,
                                'type' => $type,
                                'transaction_type' => 'in',
                                'package_id' => $subscription->package_id,
                                'old' => (int)user($subscription->user_id)->$out_type,
                                'current' => $dataw[$out_type],
                                'user_package_history_comment' => json_encode(package($subscription->package_id)),
                                'user_package_history_added_by' => $subscription->user_id
                            ];
                            $this->userModel->addUserPackageHistory($dataz);
                        }
                        $this->userModel->editUserRemaining($dataw);
                        sendEmail('SubscriptionApproved', user($subscription->user_id)->email, 'subscriptions', $id);
                        addActivityLog('subscription', $id, 'subscription_approved');
                        flash('user_message', 'subscriptions', 'subscription_approved');
                    } else {
                        flash('user_message', 'subscriptions', 'approving_subscription_failed', 'danger');
                    }
                }
                redirect('subscriptions/show/' . $id);
                die();
            } else {
                redirect('subscriptions');
            }
        }
        
        public function cancel($id = NULL)
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $subscription = subscription($id);
            if (empty($subscription)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if (in_array($subscription->subscription_status_id, array('2', '3'))) {
                    flash('user_message', 'subscriptions', 'subscription_already_processed', 'danger');
                } else {
                    $data = [
                        'subscription_id' => $id,
                        'subscription_status_id' => '3',
                        'subscription' => $subscription
                    ];
                    if ($this->subscriptionModel->updateSubscriptionStatus($data)) {
                        sendEmail('SubscriptionCancelled', user($data['subscription']->subscription_added_by)->email, 'subscriptions', $id);
                        addActivityLog('subscription', $id, 'subscription_cancelled');
                        flash('user_message', 'subscriptions', 'subscription_cancelled');
                    } else {
                        flash('user_message', 'subscriptions', 'cancelling_subscription_failed', 'danger');
                    }
                }
                redirect('subscriptions/show/' . $id);
                die();
            } else {
                redirect('subscriptions');
            }
        }
	}