<?php
    
    
    class Tickets extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            if (isSuperAdmin()) {
                $tickets = tickets();
            } else {
                $tickets = tickets(user_id());
            }
            $data = [
                'tickets' => $tickets
            ];
            $this->view('tickets/index', $data);
        }
        
        public function add()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'ticket_priority_id' => cleanString($_POST['ticket_priority_id']),
                    'ticket_subject' => cleanString($_POST['ticket_subject']),
                    'ticket_description' => cleanString($_POST['ticket_description'], 'ignore_tags')
                ];
                if (empty($data['ticket_subject'])) {
                    $data['input_error'][] = 'please_enter_ticket_subject';
                }
                if (empty($data['ticket_description'])) {
                    $data['input_error'][] = 'please_enter_ticket_description';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->ticketModel->addTicket($data)) {
                        notificationToUser('ticket_added', 1, 'tickets/show/' . $id);
                        addActivityLog('ticket', $id, 'ticket_added');
                        flash('user_message', 'ticket', 'ticket_added');
                        redirect('tickets/show/' . $id);
                    } else {
                        flash('user_message', 'ticket', 'adding_ticket_failed', 'danger');
                        redirect('tickets/add');
                    }
                    die();
                }
                $this->view('tickets/add', $data);
            }
            $this->view('tickets/add');
        }
        
        public function show($id = NULL)
        {
            $ticket = ticket($id);
            if (empty($ticket)) {
                redirect(404);
            }
            if (!isSuperAdmin() && $ticket->ticket_added_by != user_id()) {
                redirect(404);
            }
            $data = [
                'ticket' => $ticket
            ];
            $this->view('tickets/show', $data);
        }
        
        public function mark_as($id = NULL)
        {
            $ticket = ticket($id);
            if (empty($ticket)) {
                redirect(404);
            }
            if (!isSuperAdmin() && $ticket->ticket_added_by != user_id()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $ticket_status_id = $_GET['status'];
                $data = [
                    'ticket_status_id' => $ticket_status_id,
                    'ticket_id' => $id
                ];
                $_GET = sanitize_get();
                if ($this->ticketModel->updateTicketStatus($data)) {
                    addActivityLog('ticket', $id, 'ticket_status_updated');
                    flash('user_message', 'tickets', 'ticket_status_updated');
                } else {
                    flash('user_message', 'tickets', 'updating_ticket_status_failed', 'danger');
                }
                redirect('tickets/show/' . $id);
                die();
            } else {
                redirect('tickets');
            }
        }
        
        public function add_ticket_reply($ticket_id)
        {
            $ticket = ticket($ticket_id);
            if (empty($ticket)) {
                redirect(404);
            }
            if (!isSuperAdmin() && $ticket->ticket_added_by != user_id()) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                if (user_id() == $ticket->ticket_added_by) {
                    $reply_to = 1;
                } else {
                    $reply_to = $ticket->ticket_added_by;
                }
                $_POST = sanitize_post();
                $data = [
                    'ticket_id' => $ticket_id,
                    'ticket_reply_added_by' => user_id(),
                    'reply_to' => $reply_to,
                    'ticket_reply' => cleanString($_POST['ticket_reply']),
                    'ticket_status_id' => isset($_POST['close_ticket']) ? 2 : 1,
                    'ticket' => $ticket
                ];
                if (empty($data['ticket_reply'])) {
                    $data['input_error'][] = 'please_enter_description';
                }
                if (empty($data['input_error'])) {
                    if ($this->ticketModel->addTicketReply($data)) {
                        if ($data['ticket_status_id']) {
                            $this->ticketModel->updateTicketStatus($data);
                        }
                        notificationToUser('ticket_reply_added', $data['reply_to'], 'tickets/show/' . $ticket_id);
                        addActivityLog('ticket', $ticket_id, 'ticket_reply_added');
                        flash('user_message', 'tickets', 'ticket_reply_added');
                    } else {
                        flash('user_message', 'tickets', 'ticket_reply_adding_failed', 'danger');
                    }
                    redirect('tickets/show/' . $ticket_id);
                    die();
                }
                $this->view('tickets/show', $data);
            }
            $this->view('tickets/show');
        }
        
        
    }
