<?php
    
    
    class Users extends Controller
    {
        public function __construct()
        {
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            if (!isLoggedIn()) {
                logout();
            } elseif (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $data = [
                'users' => users()
            ];
            $this->view('users/index', $data);
        }
        
        public function login()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'username' => cleanString($_POST['username']),
                    'password' => cleanString($_POST['password'])
                ];
                if (getOption('enable_google_recaptcha')) {
                    $googleReCaptcha = google_recaptcha(google_recaptcha_settings()->google_recaptcha_secret_key);
                    if (empty($googleReCaptcha)) {
                        $data['input_error'] = 'recaptcha_box_must_be_checked';
                    }
                    if (!empty($googleReCaptcha)) {
                        if (!$googleReCaptcha->success) {
                            $data['input_error'] = 'recaptcha_verification_failed';
                        }
                    }
                }
                if (empty($data['username'])) {
                    $data['input_error'] = 'please_enter_username_or_email';
                }
                if (empty($data['password'])) {
                    $data['input_error'] = 'please_enter_password';
                }
                if (empty($data['input_error'])) {
                    if ($this->userModel->findUserByEmail($data['username']) || $this->userModel->findUserByUsername($data['username'])) {
                        $loggedInUser = $this->userModel->login($data['username'], $data['password']);
                        if ($loggedInUser) {
                            if ($loggedInUser->user_status_id == 1) {
                                $this->createUserSession($loggedInUser);
                            } elseif ($loggedInUser->user_status_id == 3) {
                                flash('user_message', 'users', 'user_is_deactivated', 'danger');
                            } elseif ($loggedInUser->user_status_id == 4) {
                                flash('user_message', 'users', 'account_is_deleted', 'danger');
                            } elseif (company(user($loggedInUser->user_id)->company_id)->company_status_id != 1) {
                                flash('user_message', 'users', 'login_currently_disabled_contact_company_administrator', 'danger');
                            }
                        } else {
                            flash('user_message', 'users', 'password_incorrect', 'danger');
                        }
                    } else {
                        flash('user_message', 'users', 'login_details_incorrect', 'danger');
                    }
                    redirect('users/login');
                    die();
                }
                flash('user_message', 'users', $data['input_error'], 'danger');
                $this->view('themes/' . getOption('active_theme') . '/login', $data);
            }
            $this->view('themes/' . getOption('active_theme') . '/login');
        }
        
        public function register()
        {
            if (!getOption('create_accounts')) {
                flash('user_message', 'users', 'creating_account_is_currently_disabled', 'danger');
                redirect('users/login');
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                
                if (isset($_POST['from_home'])) {
                    $_SESSION['email'] = cleanString($_POST['email']);
                    redirect('users/register');
                    die();
                }
                if (isset($_SESSION['email'])) {
                    unset($_SESSION['email']);
                }
                
                $data = [
                    'role_id' => '2',
                    'referred_by' => !empty(get_affiliate_cookie()) ? user_by_uid(get_affiliate_cookie())->user_id : NULL,
                    'company' => NULL,
                    'username' => cleanString($_POST['username']),
                    'firstname' => cleanString($_POST['firstname']),
                    'lastname' => cleanString($_POST['lastname']),
                    'email' => cleanString($_POST['email']),
                    'calling_code' => NULL,
                    'phone' => NULL,
                    'plain_password' => cleanString($_POST['password']),
                    'password' => password_hash(cleanString($_POST['password']), PASSWORD_DEFAULT),
                    'country_id' => NULL,
                    'city' => NULL,
                    'state' => NULL,
                    'postal_code' => NULL,
                    'zip_code' => NULL,
                    'street' => NULL,
                    'description' => NULL,
                    'terms_and_conditions' => isset($_POST['terms_and_conditions']) ? cleanString($_POST['terms_and_conditions']) : NULL
                ];
                if (getOption('enable_google_recaptcha')) {
                    $googleReCaptcha = google_recaptcha(google_recaptcha_settings()->google_recaptcha_secret_key);
                    if (empty($googleReCaptcha)) {
                        $data['input_error'] = 'recaptcha_box_must_be_checked';
                    }
                    if (!empty($googleReCaptcha)) {
                        if (!$googleReCaptcha->success) {
                            $data['input_error'] = 'recaptcha_verification_failed';
                        }
                    }
                }
                if (empty($data['email'])) {
                    $data['input_error'] = 'please_enter_email';
                }
                if (email_exists($data['email'])) {
                    $data['input_error'] = 'email_is_already_taken';
                }
                if (empty($data['username'])) {
                    $data['input_error'] = 'please_enter_username';
                }
                if (username_exists($data['username'])) {
                    $data['input_error'] = 'username_is_already_taken';
                }
                if (empty($data['plain_password'])) {
                    $data['input_error'] = 'please_enter_password';
                }
                if (strlen($data['plain_password']) < 6) {
                    $data['input_error'] = 'password_must_be_at_least_6_characters';
                }
                if (is_null($data['terms_and_conditions']) && page(getOption('terms_and_conditions_page'))) {
                    $data['input_error'] = 'you_must_accept_terms_and_conditions_to_register';
                }
                if (empty($data['input_error'])) {
                    if ($id = $this->userModel->addUser($data)) {
                        sendEmail('CreateAccount', $data['email'], 'users', $id);
                        addActivityLog('user', $id, 'account_created');
                        flash('user_message', 'users', 'account_created');
                    } else {
                        flash('user_message', 'users', 'creating_account_failed', 'danger');
                    }
                    redirect('users/register');
                    die();
                }
                flash('user_message', 'users', $data['input_error'], 'danger');
                $this->view('themes/' . getOption('active_theme') . '/register', $data);
            }
            $this->view('themes/' . getOption('active_theme') . '/register');
        }
        
        public function change_password()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'old_password' => cleanString($_POST['old_password']),
                    'new_password' => cleanString($_POST['new_password']),
                    'confirm_password' => cleanString($_POST['confirm_password'])
                ];
                if (empty($data['old_password'])) {
                    $data['input_error'][] = 'please_enter_old_password';
                } elseif (!password_verify($data['old_password'], user(user_id())->password)) {
                    $data['input_error'][] = 'password_does_not_match_old_password';
                }
                if (empty($data['new_password'])) {
                    $data['input_error'][] = 'please_enter_password';
                }
                if (strlen($data['new_password']) < 6) {
                    $data['input_error'][] = 'password_must_be_at_least_6_characters';
                }
                if (empty($data['confirm_password'])) {
                    $data['input_error'][] = 'please_enter_confirm_password';
                }
                if ($data['new_password'] != $data['confirm_password']) {
                    $data['input_error'][] = 'confirm_password_does_not_match';
                }
                if (empty($data['input_error'])) {
                    $data['new_password'] = password_hash($data['new_password'], PASSWORD_DEFAULT);
                    if ($this->userModel->changePassword($data)) {
                        sendEmail('PasswordChanged', user(user_id())->email, 'users', user_id());
                        addActivityLog('user', user_id(), 'user_password_changed');
                        flash('user_message', 'users', 'user_password_changed');
                    } else {
                        flash('user_message', 'users', 'user_password_change_failed', 'danger');
                    }
                    redirect('users/change_password');
                    die();
                } else {
                    $this->view('users/change_password', $data);
                }
            }
            $data = [];
            $this->view('users/change_password', $data);
        }
        
        public function forgot_password()
        {
            if (isLoggedIn()) {
                logout();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'email' => cleanString($_POST['email'])
                ];
                if (empty($data['email'])) {
                    $data['input_error'] = 'please_enter_email_address';
                } elseif (!email_exists($data['email'])) {
                    $data['input_error'] = 'email_address_does_not_exist';
                }
                if (empty($data['input_error'])) {
                    $user = $this->userModel->findUserByEmail($data['email']);
                    if (sendEmail('ForgotPassword', $data['email'], 'users', $user->user_id)) {
                        flash('user_message', 'users', 'password_reset_link_sent_to_email');
                    } else {
                        flash('user_message', 'users', 'forgot_password_request_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'users', $data['input_error'], 'danger');
                }
                redirect('users/forgot_password');
                die();
            }
            $data = [];
            $this->view('themes/' . getOption('active_theme') . '/forgot_password', $data);
        }
        
        public function reset_password()
        {
            if (isLoggedIn()) {
                logout();
            }
            if ((($_SERVER['REQUEST_METHOD'] == 'GET') && (isset($_GET['email'])) && (isset($_GET['passkey']))) || (($_SERVER['REQUEST_METHOD'] == 'POST') && (isset($_POST['email'])) && (isset($_POST['passkey'])))) {
                if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                    $_GET = sanitize_get();
                    $data = [
                        'email' => cleanString($_GET['email']),
                        'passkey' => cleanString($_GET['passkey'])
                    ];
                } else {
                    $_POST = sanitize_post();
                    $data = [
                        'email' => cleanString($_POST['email']),
                        'passkey' => cleanString($_POST['passkey'])
                    ];
                }
                if (($this->userModel->findUserByEmail($data['email'])) && ($this->userModel->findUserByPasskey($data['passkey']))) {
                    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                        $_POST = sanitize_post();
                        $data = [
                            'email' => cleanString($_POST['email']),
                            'passkey' => cleanString($_POST['passkey']),
                            'new_password' => cleanString($_POST['new_password']),
                            'confirm_password' => cleanString($_POST['confirm_password'])
                        ];
                        if (empty($data['new_password'])) {
                            $data['input_error'] = 'please_enter_password';
                        } elseif (strlen($data['new_password']) < 6) {
                            $data['input_error'] = 'password_must_be_at_least_6_characters';
                        }
                        if (empty($data['confirm_password'])) {
                            $data['input_error'] = 'please_enter_confirm_password';
                        }
                        if ($data['new_password'] != $data['confirm_password']) {
                            $data['input_error'] = 'confirm_password_does_not_match';
                        }
                        if (empty($data['input_error'])) {
                            $data['new_password'] = password_hash($data['new_password'], PASSWORD_DEFAULT);
                            if ($this->userModel->resetPassword($data)) {
                                $user = $this->userModel->findUserByEmail($data['email']);
                                sendEmail('PasswordChanged', $data['email'], 'users', $user->user_id);
                                flash('user_message', 'users', 'password_has_been_reset');
                                redirect('users/login');
                            } else {
                                flash('user_message', 'users', 'user_reset_password_failed', 'danger');
                                redirect('users/forgot_password');
                            }
                            die();
                        }
                        flash('user_message', 'users', $data['input_error'], 'danger');
                        redirect('users/reset_password/?email=' . $data['email'] . '&&passkey=' . $data['passkey']);
                        die();
                    }
                    $this->view('themes/' . getOption('active_theme') . '/reset_password', $data);
                } else {
                    flash('user_message', 'users', 'invalid_password_reset_link', 'danger');
                    redirect('users/forgot_password');
                }
            } else {
                redirect('users/forgot_password');
            }
        }
        
        public function show($id = NULL)
        {
            if (!isLoggedIn()) {
                logout();
            } elseif (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $user = user($id);
            if (empty($user)) {
                redirect(404);
            }
            $data = [
                'user' => $user
            ];
            $this->view('users/show', $data);
        }
        
        public function profile()
        {
            if (!isLoggedIn()) {
                logout();
            }
            $data = [
                'user' => user(user_id())
            ];
            $this->view('users/profile', $data);
        }
        
        public function update_profile()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'user_id' => cleanString($_POST['user_id']),
                    'role_id' => cleanString($_POST['role_id']),
                    'firstname' => cleanString($_POST['firstname']),
                    'lastname' => cleanString($_POST['lastname']),
                    'email' => cleanString($_POST['email']),
                    'calling_code' => cleanString($_POST['calling_code']),
                    'phone' => cleanString($_POST['phone']),
                    'country_id' => cleanString($_POST['country_id']),
                    'city' => cleanString($_POST['city']),
                    'state' => cleanString($_POST['state']),
                    'postal_code' => cleanString($_POST['postal_code']),
                    'zip_code' => cleanString($_POST['zip_code']),
                    'street' => cleanString($_POST['street'])
                ];
                if ($this->userModel->editUser($data)) {
                    addActivityLog('user', user_id(), 'user_profile_updated');
                    flash('user_message', 'users', 'user_profile_updated');
                } else {
                    flash('user_message', 'users', 'user_profile_update_failed', 'danger');
                }
            }
            redirect('users/profile');
            die();
        }
        
        public function update_language()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'user_id' => cleanString($_POST['user_id']),
                    'default_language' => cleanString($_POST['default_language'])
                ];
                if ($this->userModel->updateLanguage($data)) {
                    addActivityLog('user', user_id(), 'user_language_updated');
                    flash('user_message', 'users', 'user_language_updated');
                } else {
                    flash('user_message', 'users', 'user_language_update_failed', 'danger');
                }
            }
            redirect('users/profile');
            die();
        }
        
        
        public function update_image()
        {
            if (!isLoggedIn()) {
                logout();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'image' => fileExists(APPROOT . MEDIA_FOLDER, trim($_FILES['image']['name'])),
                    'current_image' => cleanString($_POST['current_image'])
                ];
                if (!upload_image($data['image'], 'media', $_FILES['image']['tmp_name'])) {
                    flash('user_message', 'users', 'update_failed', 'danger');
                } else {
                    if ($this->userModel->updateProfileImage($data)) {
                        if ($data['current_image'] != 'default_user.png') {
                            unlinkUploads('media', $data['current_image']);
                        }
                        addActivityLog('user', user_id(), 'profile_image_updated');
                        flash('user_message', 'users', 'profile_image_updated');
                    } else {
                        flash('user_message', 'users', 'profile_image_update_failed', 'danger');
                    }
                }
            }
            redirect('users/profile');
            die();
        }
        
        public function delete_profile_image()
        {
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $_GET = sanitize_get();
                if (user(user_id())->profile_image != 'default_user.png') {
                    unlinkUploads('media', user(user_id())->profile_image);
                }
                if ($this->userModel->deleteProfileImage()) {
                    addActivityLog('user', user_id(), 'profile_image_deleted');
                    flash('user_message', 'users', 'profile_image_deleted');
                } else {
                    flash('user_message', 'users', 'profile_image_delete_failed', 'danger');
                }
            }
            redirect('users/profile');
            die();
        }
        
        public function update_payout_settings()
        {
            if (!isLoggedIn()) {
                logout();
            }
            $user = $this->userModel->getUserById(user_id());
            if (empty($user)) {
                redirect(404);
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'user_id' => user_id(),
                    'payout_mode_id' => cleanString($_POST['payout_mode_id']),
                    'payout_address' => cleanString($_POST['payout_address']),
                    'user' => $user
                ];
                if ($this->userModel->updatePayoutSettings($data)) {
                    addActivityLog('user', $data['user_id'], 'payout_settings_updated');
                    flash('user_message', 'users', 'payout_settings_updated');
                } else {
                    flash('user_message', 'users', 'updating_payout_settings_failed', 'danger');
                }
                redirect('users/profile');
                die();
            }
            $data = [
                'user' => $user
            ];
            $this->view('users/profile', $data);
        }
        
        public function activate($id = NULL)
        {
            if (!isLoggedIn()) {
                logout();
            } elseif (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $user = $this->userModel->getUserById($id);
                if (empty($user)) {
                    redirect(404);
                }
                $data = [
                    'user_status_id' => 1,
                    'user_id' => $id
                ];
                if ($id == 1) {
                    flash('user_message', 'users', 'superadmin_cannot_be_activated', 'danger');
                } elseif ($id == user_id()) {
                    flash('user_message', 'users', 'cannot_self_activate', 'danger');
                } elseif ($this->userModel->updateUserStatus($data)) {
                    addActivityLog('user', $id, 'user_activated');
                    flash('user_message', 'users', 'user_activated');
                } else {
                    flash('user_message', 'users', 'user_activation_failed', 'danger');
                }
                redirect('users/show/' . $id);
                die();
            } else {
                redirect('users');
            }
        }
        
        public function deactivate($id = NULL)
        {
            if (!isLoggedIn()) {
                logout();
            } elseif (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                $user = $this->userModel->getUserById($id);
                if (empty($user)) {
                    redirect(404);
                }
                $data = [
                    'user_status_id' => 3,
                    'user_id' => $id
                ];
                if ($id == 1) {
                    flash('user_message', 'users', 'superadmin_cannot_be_deactivated', 'danger');
                } elseif ($id == user_id()) {
                    flash('user_message', 'users', 'cannot_self_deactivate', 'danger');
                } elseif ($this->userModel->updateUserStatus($data)) {
                    addActivityLog('user', $id, 'user_deactivated');
                    flash('user_message', 'users', 'user_deactivated');
                } else {
                    flash('user_message', 'users', 'user_deactivation_failed', 'danger');
                }
                redirect('users/show/' . $id);
                die();
            } else {
                redirect('users');
            }
        }
        
        public function adjust($id = NULL)
        {
            if (!isLoggedIn()) {
                logout();
            } elseif (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $user = $this->userModel->getUserById($id);
            if (empty($user)) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'user_id' => $id,
                    'type' => cleanString($_POST['type']),
                    'quantity' => cleanString($_POST['quantity']),
                    'transaction_type' => ($_POST['quantity'] > 0) ? 'in' : 'out'
                ];
                if (empty($data['type'])) {
                    $data['input_error'] = 'please_enter_item';
                }
                if (empty($data['quantity']) || $data['quantity'] == 0) {
                    $data['input_error'] = 'please_enter_quantity';
                }
                if (empty($data['input_error'])) {
                    $dataw = [
                        'user_id' => $id,
                        'user_remaining_words' => adjust_user_remaining_balances($id, (($data['type'] == 'words') ? $data['quantity'] : NULL), 'words'),
                        'user_remaining_chat_assistants' => adjust_user_remaining_balances($id, (($data['type'] == 'chat_assistants') ? $data['quantity'] : NULL), 'chat_assistants'),
                        'user_remaining_trained_models' => adjust_user_remaining_balances($id, (($data['type'] == 'trained_models') ? $data['quantity'] : NULL), 'trained_models'),
                        'user_remaining_chat_files' => adjust_user_remaining_balances($id, (($data['type'] == 'chat_files') ? $data['quantity'] : NULL), 'chat_files'),
                        'user_remaining_chat_links' => adjust_user_remaining_balances($id, (($data['type'] == 'chat_links') ? $data['quantity'] : NULL), 'chat_links'),
                        'user_remaining_chat_sitemaps' => adjust_user_remaining_balances($id, (($data['type'] == 'chat_sitemaps') ? $data['quantity'] : NULL), 'chat_sitemaps')
                    ];
                    
                    $out_type = 'user_remaining_' . $data['type'];
                    $dataz = [
                        'user_id' => $id,
                        'type' => $data['type'],
                        'transaction_type' => $data['transaction_type'],
                        'package_id' => NULL,
                        'old' => (int)user($id)->$out_type,
                        'current' => $dataw[$out_type],
                        'user_package_history_comment' => 'adjustment',
                        'user_package_history_added_by' => user_id()
                    ];
                    $this->userModel->addUserPackageHistory($dataz);
                    if ($this->userModel->editUserRemaining($dataw)) {
                        flash('user_message', 'users', 'user_balance_adjusted');
                    } else {
                        flash('user_message', 'users', 'user_balance_adjusting_failed', 'danger');
                    }
                } else {
                    flash('user_message', 'users', $data['input_error'], 'danger');
                }
                redirect('users/show/' . $id);
                die();
            }
        }
        
        public function createUserSession($user)
        {
            $_SESSION['user_id'] = $user->user_id;
            $_SESSION['email'] = $user->email;
            $_SESSION['username'] = $user->username;
            
            $_SESSION['session_access_token'] = base64_encode(bin2hex(openssl_random_pseudo_bytes(24) . time()));
            $_SESSION['session_refresh_token'] = base64_encode(bin2hex(openssl_random_pseudo_bytes(24) . time()));
            
            $this->userModel->addUserSession($_SESSION['user_id'], $_SESSION['session_access_token'], $_SESSION['session_refresh_token']);
            if ($_SESSION['user_id'] != 1) {
                if (isset($_SESSION['selected_package'])) {
                    redirect('packages/choose/' . $_SESSION['selected_package'][0]);
                } else {
                    redirect('dashboards');
                }
            } else {
                redirect('dashboards');
            }
            die();
        }
        
        public function logout()
        {
            logout();
        }
        
        
    }