<?php
    
    
    class Utilities extends Controller
    {
        public function __construct()
        {
            if (!isLoggedIn()) {
                logout();
            }
            require APPROOT . '/views/inc/load_models.php';
        }
        
        public function index()
        {
            redirect('dashboards');
        }
        
        public function logs()
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $data = [
                'activity_logs' => out_activities()
            ];
            $this->view('utilities/logs', $data);
        }
        
        public function backup()
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            $all_tables = $this->utilityModel->getAllTables();
            if (isset($_POST["table"])) {
                $data = [
                    'all_tables' => $_POST["table"]
                ];
                if ($file_name = $this->utilityModel->backupDatabase($data)) {
                    force_download($file_name, $file_name);
                    redirect('utilities/backup');
                    die();
                }
            } else {
                $data = [
                    'all_tables' => $all_tables
                ];
            }
            $this->view('utilities/backup', $data);
        }
        
        public function backup_settings()
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                $_POST = sanitize_post();
                $data = [
                    'backup_database_automatically' => trim($_POST['backup_database_automatically']),
                    'backup_database_automatically_after' => trim($_POST['backup_database_automatically_after']) . ' ' . trim($_POST['backup_database_automatically_after_duration_type']),
                    'delete_database_backups_older_than' => trim($_POST['delete_database_backups_older_than'])
                ];
                foreach ($data as $keys => $options) {
                    $data = [
                        'option_name' => $keys,
                        'option_value' => $options
                    ];
                    $insert = $this->settingModel->updateOptions($data);
                }
                if ($insert) {
                    addActivityLog('user', user_id(), 'backup settings_updated');
                    flash('user_message', 'settings', 'backup_settings_updated');
                } else {
                    flash('user_message', 'settings', 'backup_settings_update_failed', 'danger');
                }
            }
            redirect('utilities/backup');
            die();
        }
        
        public function delete_db_backup()
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (unlink(APPROOT . DATABASE_FOLDER . $_GET['backup'])) {
                    flash('user_message', 'settings', 'database_backup_deleted');
                } else {
                    flash('user_message', 'settings', 'delete_failed', 'danger');
                }
            }
            redirect('utilities/backup');
        }
        
        public function download_db_backup()
        {
            if (!isSuperAdmin()) {
                redirect(404);
                die();
            }
            if ($_SERVER['REQUEST_METHOD'] == 'GET') {
                if (force_download($_GET['backup'], APPROOT . DATABASE_FOLDER . $_GET['backup'])) {
                    flash('user_message', 'settings', 'database_backup_downloaded');
                } else {
                    flash('user_message', 'settings', 'database_backup_failed', 'danger');
                }
            }
            redirect('utilities/backup');
        }
        
        public function custom_errors()
        {
            $data = [
                'custom_errors' => view_custom_errors()
            ];
            $this->view('utilities/custom_errors', $data);
        }
        
    }
