<?php
    
    use SendGrid\Mail\Mail;
    
    function email_gateways()
    {
        $model = setting_model();
        return $model->getEmailGateways();
    }
    
    function email_gateways_names()
    {
        $model = setting_model();
        $results = $model->getEmailGateways();
        $final = array();
        foreach ($results as $result) {
            $final[] = strtolower($result->email_gateway_name) . '_live_enabled';
        }
        return $final;
    }
    
    function email_gateway($email_template_id)
    {
        $model = setting_model();
        return $model->getEmailGatewayById($email_template_id);
    }
    
    function php_mailer($from_company, $from_email, $to, $email_subject, $message)
    {
        // Load Composer's autoloader
        require APPROOT . '/third_party/PHPMailer/vendor/autoload.php';
        
        // Instantiation and passing `true` enables exceptions
        $mail = new PHPMailer\PHPMailer\PHPMailer(true);
        
        /* Set the mail sender. */
        $mail->setFrom($from_email, $from_company);
        
        /* Add a recipient. */
        $mail->addAddress($to);
        
        /* Set the subject. */
        $mail->Subject = $email_subject;
        
        /* Set the mail message body. */
        $mail->Body = $message;
        
        /* Finally send the mail. */
        if (!$mail->send()) {
            /* PHPMailer error. */
            //0echo $mail->ErrorInfo;
            return false;
        } else {
            return true;
        }
    }
    
    function php_mailer_smtp($from_company, $from_email, $to, $email_subject, $message)
    {
        $model = setting_model();
        $smtp_live_enabled = $model->getOptions('smtp_live_enabled');
        $smtp_settings = $model->getSMTPSettings($smtp_live_enabled->option_value);
        // Load Composer's autoloader
        require APPROOT . '/third_party/PHPMailer/vendor/autoload.php';
        
        // Instantiation and passing `true` enables exceptions
        $mail = new PHPMailer\PHPMailer\PHPMailer(true);
        
        try {
            
            $mail->setFrom($from_email, $from_company);
            $mail->addAddress($to);
            $mail->Subject = $email_subject;
            $mail->Body = $message;
            
            /* SMTP parameters. */
            
            /* Tells PHPMailer to use SMTP. */
            $mail->isSMTP();
            
            /* SMTP server address. */
            $mail->Host = $smtp_settings->smtp_host;
            
            /* Use SMTP authentication. */
            $mail->SMTPAuth = TRUE;
            
            /* Set the encryption system. */
            $mail->SMTPSecure = $smtp_settings->smtp_encryption;
            
            /* SMTP authentication username. */
            $mail->Username = $smtp_settings->smtp_username;
            
            /* SMTP authentication password. */
            $mail->Password = $smtp_settings->smtp_password;
            
            /* Set the SMTP port. */
            $mail->Port = $smtp_settings->smtp_port;
            
            /* Disable some SSL checks. */
            $mail->SMTPOptions = array(
                'ssl' => array(
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                    'allow_self_signed' => true
                )
            );
            
            /* Finally send the mail. */
            $mail->send();
        } catch (Exception $e) {
            echo $e->errorMessage();
        }
    }
    
    function sendgrid($from_company, $from_email, $to, $email_subject, $message)
    {
        $model = setting_model();
        $sendgrid_live_enabled = $model->getOptions('sendgrid_live_enabled');
        $sendgrid_settings = $model->getSendgridSettings($sendgrid_live_enabled->option_value);
        require APPROOT . '/third_party/sendgrid/vendor/autoload.php';
        $email = new Mail();
        $email->setFrom($from_email, $from_company);
        $email->setSubject($email_subject);
        $email->addTo($to);
        $email->addContent("text/html", $message);
        
        
        $sendgrid = new SendGrid(getenv($sendgrid_settings->sendgrid_api_key));
        
        try {
            $response = $sendgrid->send($email);
        } catch (Exception $e) {
            echo 'Caught exception: ' . $e->getMessage() . "\n";
        }
        if (isset($response) && $response->statusCode() == '200') {
            return true;
        } else {
            return false;
        }
    }
    
    function php_mail($from_company, $from_email, $to, $email_subject, $message)
    {
        
        $from = $from_company . " <" . $from_email . ">";
        $headers = "From: $from";

// boundary 
        $semi_rand = md5(time());
        $mime_boundary = "==Multipart_Boundary_x{$semi_rand}x";

// headers for attachment 
        $headers .= "\nMIME-Version: 1.0\n" . "Content-Type: multipart/mixed;\n" . " boundary=\"$mime_boundary\"";

// multipart boundary 
        $message = "--$mime_boundary\n" . "Content-Type: text/html; charset=\"UTF-8\"\n" .
            "Content-Transfer-Encoding: 7bit\n\n" . $message . "\n\n";
        
        $message .= "--$mime_boundary--";
        $returnpath = "-f" . $from;

//send email
        $mail = @mail($to, $email_subject, $message, $headers, $returnpath);

//function return true, if email sent, otherwise return fasle
        if ($mail) {
            return TRUE;
        } else {
            return FALSE;
        }
    }
    
    function sendEmail($template_type, $to, $item, $item_id, $user_id = NULL)
    {
        $template = email_template_by_type($template_type);
        if (isEmailTemplateEnabled($template->email_template_id)) {
            $message = template_helper($item, $item_id, $template->email_message);
// Send email
            $gateway = getOption('email_gateway');
            if ($gateway(company(1)->company_name, !empty(getOption('use_email')) ? getOption('use_email') : company(1)->company_email, $to, $template->email_subject, $message)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    function sendCustomEmail($to, $email_subject, $message)
    {
        
        $from = !empty(getOption('use_email')) ? company(1)->company_name . " <" . getOption('use_email') . ">" : company(1)->company_name . " <" . company(1)->company_email . ">";
        $headers = "From: $from";
        
        $gateway = getOption('email_gateway');
        if ($gateway(company(1)->company_name, !empty(getOption('use_email')) ? getOption('use_email') : company(1)->company_email, $to, $email_subject, $message)) {
            return TRUE;
        } else {
            return FALSE;
        }
    }