<?php
    function openai_model()
    {
        return new Openai;
    }
    
    function openai_tones()
    {
        $model = openai_model();
        return $model->getOpenAiTones();
    }
    
    function openai_models()
    {
        $model = openai_model();
        return $model->getOpenAiModels();
    }
    
    function openai_model_by_id($id)
    {
        $model = openai_model();
        return $model->getOpenAiModelById($id);
    }  
    
    function openai_model_by_model($openai_model)
    {
        $model = openai_model();
        return $model->getOpenAiModel($openai_model);
    }    
    
    function open_ai($user_id, $prompt, $temperature, $n, $tokens, $assistant = NULL)
    {
        $openai_model = openai_model_by_model(getOption('custom_package_model'));
        $available_words_prepaid = user($user_id)->user_remaining_words;
        $text = '';
        $finish_reason = '';
        $counter = 1;
        if($tokens>$openai_model->openai_model_tokens){
			$tokens = $openai_model->openai_model_tokens;
		}
        if (isset($assistant)) {
            $messages = array(
                0 =>
                    array(
                        'role' => 'system',
                        'content' => $assistant
                    ),
                1 =>
                    array(
                        'role' => 'user',
                        'content' => $prompt
                    )
            );
        } else {
            $messages = array(
                0 =>
                    array(
                        'role' => 'user',
                        'content' => $prompt
                    ),
            );
        }
        $ch = curl_init();
        $post = array(
            'model' => $openai_model->openai_model,
            'messages' => $messages,
            'temperature' => $temperature,
            'max_tokens' => $tokens,
            'n' => $n
        );
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/chat/completions');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
        
        $headers = array(
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            if (count($response['choices']) > 1) {
                foreach ($response['choices'] as $value) {
                    $text .= $counter . '. ' . trim($value['message']['content']) . "\r\n\r\n\r\n";
                    $finish_reason = $value['finish_reason'];
                    $counter++;
                }
            } else {
                $text .= $response['choices'][0]['message']['content'];
                $finish_reason = $response['choices'][0]['finish_reason'];
            }
            $tokens = $response['usage']['total_tokens'];
            
            $data['text'] = $text;
            $data['finish_reason'] = $finish_reason;
            $data['status'] = 'success';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = ($available_words_prepaid === -1) ? $available_words_prepaid : ($available_words_prepaid - $tokens);
            $data['tokens_used'] = $tokens;
            $data['chat_id'] = $response['id'];
            $data['message'] = '';
        } else {
            $data['finish_reason'] = NULL;
            $data['status'] = 'error';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = $available_words_prepaid;
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['finish_reason'] = NULL;
            $data['status'] = 'error';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = $available_words_prepaid;
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_upload_finetune_file($file)
    {
        $text = '';
        $post = array(
            "purpose" => "fine-tune",
            "file" => curl_file_create($file)
        );
        
        $json = ($post);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/files');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        
        
        $headers = array(
            "Authorization: Bearer " . getOption('openai_api_key'),
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['file_id'] = $response['id'];
            $data['bytes'] = $response['bytes'];
            $data['created_at'] = $response['created_at'];
            $data['filename'] = $response['filename'];
            $data['file_status'] = $response['status'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_retrieve_finetune_file($file_id)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/files/' . $file_id);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        curl_close($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['file_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['bytes'] = $response['bytes'];
            $data['created_at'] = $response['created_at'];
            $data['filename'] = $response['filename'];
            $data['purpose'] = $response['purpose'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_delete_finetune_file($file_id)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/files/' . $file_id);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        curl_close($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['file_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['deleted'] = $response['deleted'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_list_finetune_files()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/files');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['data'])) {
            $data['status'] = 'success';
            $data['data'] = $response['data'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_create_finetune_job($file)
    {
        $post = array(
            "model" => getOption('openai_default_finetune_model'),
            "training_file" => $file
        );
        
        $json = json_encode($post);
        
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, "https://api.openai.com/v1/fine_tuning/jobs");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        curl_setopt($ch, CURLOPT_POST, 1);
        
        $headers = array();
        $headers[] = "Content-Type: application/json";
        $headers[] = "Authorization: Bearer " . getOption('openai_api_key');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['finetune_job_id'] = $response['id'];
            $data['model'] = $response['model'];
            $data['created_at'] = $response['created_at'];
            $data['finetune_job_model'] = $response['fine_tuned_model'];
            $data['training_file'] = $response['training_file'];
            $data['finetune_job_status'] = $response['status'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_retrieve_finetune_job($finetuning_job_id)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/fine_tuning/jobs/' . $finetuning_job_id);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        curl_close($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id']) && !in_array($response['status'], array('failed','cancelled'))) {
            $data['status'] = 'success';
            $data['finetune_job_id'] = $response['id'];
            $data['model'] = $response['model'];
            $data['created_at'] = $response['created_at'];
            $data['finished_at'] = $response['finished_at'];
            $data['finetune_job_model'] = $response['fine_tuned_model'];
            $data['training_file'] = $response['training_file'];
            $data['finetune_job_status'] = $response['status'];
            $data['trained_tokens'] = $response['trained_tokens'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['code'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_cancel_finetune_job($finetuning_job_id)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/fine_tuning/jobs/' . $finetuning_job_id . '/cancel');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        curl_close($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['finetune_id'] = $response['id'];
            $data['model'] = $response['model'];
            $data['created_at'] = $response['created_at'];
            $data['finished_at'] = $response['finished_at'];
            $data['fine_tuned_model'] = $response['fine_tuned_model'];
            $data['training_file'] = $response['training_file'];
            $data['finetuning_status'] = $response['status'];
            $data['trained_tokens'] = $response['trained_tokens'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_list_finetune_jobs($limit)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/fine_tuning/jobs?limit=' . $limit);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . getOption('openai_api_key')
        ]);
        
        $result = curl_exec($ch);
        
        curl_close($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['data'])) {
            $data['status'] = 'success';
            $data['data'] = $response['data'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_upload_assistant_file($file)
    {
        $text = '';
        $post = array(
            "purpose" => "assistants",
            "file" => curl_file_create($file)
        );
        
        $json = ($post);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/files');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        
        
        $headers = array(
            "Authorization: Bearer " . getOption('openai_api_key'),
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['file_id'] = $response['id'];
            $data['bytes'] = $response['bytes'];
            $data['created_at'] = $response['created_at'];
            $data['filename'] = $response['filename'];
            $data['file_status'] = $response['status'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_assistant($file, $instructions)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $file = explode(',', $file);
        
        $data = [
            "instructions" => $instructions,
            "tools" => [
                ["type" => "retrieval"],
            ],
            "model" => getOption('openai_default_assistant_model'),
            "file_ids" => $file,
        ];
        
        $ch = curl_init("https://api.openai.com/v1/assistants");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['assistant_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['name'] = $response['name'];
            $data['instructions'] = $response['instructions'];
            $data['tools'] = $response['tools'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_assistant_file($assistant_id, $file_id)
    {
        $headers = [
            "Authorization: Bearer " . getOption('openai_api_key'),
            "Content-Type: application/json",
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $data = [
            "file_id" => $file_id,
        ];
        
        $ch = curl_init("https://api.openai.com/v1/assistants/$assistant_id/files");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['assistant_file_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['assistant_id'] = $response['assistant_id'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_thread($file_id, $message)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        if(getOption('assistant_version')=='v1'){
        
        	$file = explode(',', $file_id);
        	
	        $data = [
	            "messages" => [
	                [
	                    "role" => "user",
	                    "content" => $message,
	                    "file_ids" => $file,
	                ],
	            ],
	        ];
		}else{
	        $data = [
	            "messages" => [
	                [
	                    "role" => "user",
	                    "content" => $message,
	                    "attachments" => NULL,
	                ],
	            ],
	        ];
		}
        
        
        $ch = curl_init("https://api.openai.com/v1/threads");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['thread_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['metadata'] = $response['metadata'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_retrieve_thread($thread_id)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['thread_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['metadata'] = $response['metadata'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_thread_message($thread_id, $message)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $data = [
            "role" => "user",
            "content" => $message,
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id/messages");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['thread_id'] = $response['thread_id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['metadata'] = $response['metadata'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_run($thread_id, $assistant_id)
    {
        $headers = [
            "Authorization: Bearer " . getOption('openai_api_key'),
            "Content-Type: application/json",
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $data = [
            "assistant_id" => $assistant_id,
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id/runs");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['run_id'] = $response['id'];
            $data['thread_id'] = $response['thread_id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['metadata'] = $response['metadata'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_retrieve_run($thread_id, $run_id)
    {
        $headers = [
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id/runs/$run_id");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['run_status'] = $response['status'];
            $data['run_id'] = $response['id'];
            $data['thread_id'] = $response['thread_id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['metadata'] = $response['metadata'];
            $data['tokens_used'] = isset($response['usage']['total_tokens']) ? $response['usage']['total_tokens'] : 0;
            $data['last_error'] = $response['last_error'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_cancel_run($thread_id, $run_id)
    {
        $headers = [
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id/runs/$run_id/cancel");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['run_id'] = $response['id'];
            $data['thread_id'] = $response['thread_id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_list_messages($thread_id)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/threads/$thread_id/messages?order=desc");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        if (!empty($response) && isset($response['data'])) {
            $text = array();
            foreach ($response['data'] as $item) {
                if ($item['role'] == 'user') {
                    break;
                }
                if (isset($item['content']) && is_array($item['content'])) {
                    foreach ($item['content'] as $contentItem) {
                        if (isset($contentItem['text']) && is_array($contentItem['text'])) {
                            $text[] = remove_source($contentItem['text']['value']);
                        }
                    }
                }
            }
            if (!empty($text)) {
                $data['status'] = 'success';
                $data['object'] = $response['object'];
                $data['data'] = $text;
                $data['message'] = '';
            } else {
                $data['status'] = 'wait';
                $data['message'] = 'assistant_did_not_respond';
            }
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
        
        return $data;
    }
    
    function ai_files()
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/files");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
                
        if (!empty($response) && isset($response['data'])) {
            $data['status'] = 'success';
            $data['data'] = $response['data'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_delete_file($file)
    {
        $headers = [
            "Authorization: Bearer " . getOption('openai_api_key'),
        ];
        
        $ch = curl_init("https://api.openai.com/v1/files/$file");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && (!isset($response['error']))) {
            $data['status'] = 'success';
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_vector_store($file_id, $vector_store_name = NULL)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=v2",
        ];
        $file = explode(',', $file_id);
        
        $data = [
            "name" => $vector_store_name,
            "file_ids" => $file
        ];
        
        $ch = curl_init("https://api.openai.com/v1/vector_stores");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
                
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['vector_store_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['name'] = $response['name'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_vector_store_file($vector_store_id, $file)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=v2",
        ];
        
        $file = explode(',', $file);
        
        $data = [
            "file_id" => $file,
        ];

        $ch = curl_init("https://api.openai.com/v1/vector_stores/$vector_store_id/files");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['file_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['usage_bytes'] = $response['usage_bytes'];
            $data['vector_store_id'] = $response['vector_store_id'];
            $data['create_status'] = $response['status'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_create_assistant_v2($vector_store_id, $instructions)
    {
        $headers = [
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
            "OpenAI-Beta: assistants=" . getOption('assistant_version'),
        ];
        
        $data = [
		    "instructions" => $instructions,
		    "tools" => [
		        ["type" => "file_search"]
		    ],
		    "tool_resources" => [
		        "file_search" => [
		            "vector_store_ids" => [$vector_store_id]
		        ]
		    ],
		    "model" => getOption('openai_default_assistant_model')
		];
        
        $ch = curl_init("https://api.openai.com/v1/assistants");
        
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);

        if (!empty($response) && isset($response['id'])) {
            $data['status'] = 'success';
            $data['assistant_id'] = $response['id'];
            $data['object'] = $response['object'];
            $data['created_at'] = $response['created_at'];
            $data['name'] = $response['name'];
            $data['instructions'] = $response['instructions'];
            $data['tools'] = $response['tools'];
            $data['message'] = '';
        } else {
            $data['status'] = 'error';
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['status'] = 'error';
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function ai_user_trained_model_chat($user_id, $prompt, $temperature, $n, $tokens, $assistant, $trained_model)
    {
        $available_words_prepaid = user($user_id)->user_remaining_words;
        $text = '';
        $finish_reason = '';
        $counter = 1;
        $tokens = (int)getOption('custom_package_maximum_result_length');
        if (!empty($assistant)) {
            $messages = array(
                0 =>
                    array(
                        'role' => 'system',
                        'content' => $assistant
                    ),
                1 =>
                    array(
                        'role' => 'user',
                        'content' => $prompt
                    )
            );
        } else {
            $messages = array(
                0 =>
                    array(
                        'role' => 'user',
                        'content' => $prompt
                    ),
            );
        }
        $ch = curl_init();
        
        if (finetune_by_output_model($trained_model)->finetune_job_output_model_used == 'gpt-3.5-turbo') {
            $post = array(
                'model' => $trained_model,
                'messages' => $messages,
                'temperature' => $temperature,
                'max_tokens' => $tokens,
                'n' => $n
            );
        } else {
            $post = array(
                'model' => $trained_model,
                'prompt' => $prompt,
                'temperature' => $temperature,
                'max_tokens' => $tokens,
                'n' => $n
            );
        }
        curl_setopt($ch, CURLOPT_URL, 'https://api.openai.com/v1/completions');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
        
        $headers = array(
            "Content-Type: application/json",
            "Authorization: Bearer " . getOption('openai_api_key'),
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        
        $response = json_decode($result, true);
        
        if (!empty($response) && isset($response['id'])) {
            if (finetune_by_output_model($trained_model)->finetune_job_output_model_used == 'gpt-3.5-turbo') {
                if (count($response['choices']) > 1) {
                    foreach ($response['choices'] as $value) {
                        $text .= $counter . '. ' . trim($value['message']['content']) . "\r\n\r\n\r\n";
                        $finish_reason = $value['finish_reason'];
                        $counter++;
                    }
                } else {
                    $text .= $response['choices'][0]['message']['content'];
                    $finish_reason = $response['choices'][0]['finish_reason'];
                }
            } else {
                if (count($response['choices']) > 1) {
                    foreach ($response['choices'] as $value) {
                        $text .= $counter . '. ' . ltrim($value['text']) . "\r\n\r\n\r\n";
                        $finish_reason = $value['finish_reason'];
                        $counter++;
                    }
                } else {
                    $text .= $response['choices'][0]['text'];
                    $finish_reason = $response['choices'][0]['finish_reason'];
                }
            }
            $tokens = $response['usage']['total_tokens'];
            
            $data['text'] = $text;
            $data['finish_reason'] = $finish_reason;
            $data['status'] = 'success';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = $available_words_prepaid - $tokens;
            $data['tokens_used'] = $tokens;
            $data['chat_id'] = $response['id'];
            $data['message'] = '';
        } else {
            $data['finish_reason'] = NULL;
            $data['status'] = 'error';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = $available_words_prepaid;
            $data['message'] = isset($response['error']) ? $response['error']['type'] . ' - ' . $response['error']['message'] : label('unknown_error');
        }
        
        if (curl_errno($ch)) {
            $data['finish_reason'] = NULL;
            $data['status'] = 'error';
            $data['old_tokens'] = $available_words_prepaid;
            $data['current_tokens'] = $available_words_prepaid;
            $data['message'] = curl_error($ch);
        }
        
        curl_close($ch);
    
        return $data;
    }
    
    function is_assistant_file_supported($file)
    {
        $allowed = accepted_assistant_files();
        $ext = pathinfo($file, PATHINFO_EXTENSION);
        if (!in_array($ext, $allowed)) {
            return false;
        } else {
            return true;
        }
    }
    
    function accepted_assistant_files()
    {
        return array('c', 'cpp', 'csv', 'docx', 'html', 'java', 'json', 'md', 'pdf', 'php', 'pptx', 'py', 'rb', 'tex', 'txt');
    }

    function remove_source($string)
    {
        return preg_replace('/【\d+(:\d+)?+†source】/', '', $string);
    }