<?php
    function package_model()
    {
        return new Package;
    }
    
    function packages($group = NULL)
    {
        $model = package_model();
        if(isset($group)){
            $results = $model->getPackages($group);
        }else{
            $results = $model->getPackages();
        }
        return $results;
    }
    
    function package($package_id)
    {
        $model = package_model();
        return $model->getPackageById($package_id);
    }
    
    function free_trial_package()
    {
        $model = package_model();
        return $model->getFreeTrialPackage();
    }
    
    function package_cost($package_id, $package_group = NULL)
    {
        $package = package($package_id);
        if(isset($package_group) && in_array($package_group, array('monthly', 'yearly', 'lifetime'))){
        	$out_name = 'package_'.$package_group.'_cost';
			$package_cost = $package->$out_name;
		}else{
			$package_cost = $package->package_cost;
		}
        return $package_cost;
    }
    
    function package_duration($package_id, $package_group = NULL)
    {
        $package = package($package_id);
        if(isset($package_group) && in_array($package_group, array('monthly', 'yearly', 'lifetime'))){
        	if($package_group=='monthly'){
				$package_duration = '1';
				$package_duration_type = 'months';
			}elseif($package_group=='yearly'){
				$package_duration = '1';
				$package_duration_type = 'years';
			}elseif($package_group=='lifetime'){
				//32-bit = 2147483647
				//64-bit = 9223372036854775807
				$diff = 2147483647 - time();
				if (PHP_INT_SIZE === 4) {
				    $diff = 2147483647 - time();
				} elseif (PHP_INT_SIZE === 8) {
				    $diff = 9223372036854775807 - time();
				}
				$package_duration = $diff;
				$package_duration_type = 'seconds';
			}
		}else{
			$package_duration = $package->package_duration;
			$package_duration_type = $package->package_duration_type;
		}
        return array($package_duration, $package_duration_type);
    }
    
    function package_duration_name($id){
    	$package = package($id);
		if($package->is_free_trial){
			$out = '1 month';
		}elseif(!$package->is_free_trial && $package->package_duration && $package->package_duration_type){
			$out = $package->package_duration.' '.$package->package_duration_type;
		}else{
			$out = NULL;
		}
		return $out;
	}
    
    function package_cost_name($id){
    	$package = package($id);
		if($package->is_free_trial){
			$out = bold_currency_format(0);
		}elseif(!$package->is_free_trial && $package->package_cost && $package->is_custom){
			$out = bold_currency_format($package->package_cost);
		}else{
			$out = (!empty($package->package_monthly_cost) ? bold_currency_format($package->package_monthly_cost).' '.label('per_month') : NULL).' '.(!empty($package->package_yearly_cost) ? ' | '.bold_currency_format($package->package_yearly_cost).' '.label('per_year') : NULL).' '.(!empty($package->package_lifetime_cost) ? ' | '.bold_currency_format($package->package_lifetime_cost).' '.label('per_lifetime') : NULL);
		}
		return $out;
	}
    
    function package_features()
    {
        $model = package_model();
        return $model->getPackageFeatures();
    }
    
    function package_feature($package_feature_id)
    {
        $model = package_model();
        return $model->getPackageFeatureById($package_feature_id);
    }
    
    function package_items($package_id)
    {
        $model = package_model();
        return $model->getPackageItems($package_id);
    }
    
    function package_items_array($package_id)
    {
        $results = array();
        foreach (package_items($package_id) as $item) {
            $results[] = $item->package_feature_id;
        }
        return $results;
    }
    
    function duration_types()
    {
        $model = package_model();
        return $model->getDurationTypes();
    }
    
    function package_types()
    {
        return array('words', 'chat_assistants', 'trained_models', 'chat_files', 'chat_links', 'chat_sitemaps');
    }

    function package_groups()
    {
        return array('months'=>'monthly', 'years'=>'yearly', 'lifetime'=>'lifetime', 'others'=>'custom');
    }
    
    function highest_package_discount_percentage(){
		foreach(packages(0) as $package){
			if($package->package_monthly_cost>0 && $package->package_yearly_cost>0){
				$out[] = percentage((($package->package_monthly_cost*12)-$package->package_yearly_cost), ($package->package_monthly_cost*12), 0);
			}
		}
		return !empty($out) ? max($out) : NULL;
	}
    
    function private_package($package_id,$user_id)
    {
        $model = package_model();
        return $model->getPrivatePackage($package_id, $user_id);
    }
    
    function private_package_users($package_id)
    {
        $model = package_model();
        $out = array();
        foreach($model->getPrivatePackageUsers($package_id) as $users){
			$out[] = $users->user_id;
		}
		return $out;
    }