<?php
    function subscription_model()
    {
        return new Subscription;
    }
    
    function subscriptions()
    {
        $model = subscription_model();
        if (isSuperAdmin()) {
            $results = $model->getSubscriptions();
        } else {
            $results = $model->getSubscriptions(user_id());
        }
        return $results;
    }
    
    function subscription_totals()
    {
        $model = subscription_model();
        if (isSuperAdmin()) {
            $results = $model->getSubscriptionTotals();
        } else {
            $results = $model->getSubscriptionTotals(user_id());
        }
        return $results;
    }
    
    function free_trial_subscription($user_id, $payment_option_id)
    {
        $model = subscription_model();
        return $model->getFreeTrialSubscription($user_id, $payment_option_id);
    }
    
    function subscription_cost($plan_cost, $plan_duration, $duration_type)
    {
        if ($duration_type == 'hours') {
            $subscription_cost = $plan_cost * ($plan_duration / (date('t') * 24));
        } elseif ($duration_type == 'days') {
            $subscription_cost = $plan_cost * ($plan_duration / date('t'));
        } elseif ($duration_type == 'weeks') {
            $subscription_cost = $plan_cost * ($plan_duration / 4);
        } elseif ($duration_type == 'months') {
            $subscription_cost = $plan_cost * $plan_duration;
        } elseif ($duration_type == 'years') {
            $subscription_cost = $plan_cost * ($plan_duration * 12);
        } else {
            $subscription_cost = '0.00';
        }
        return round($subscription_cost, 2);
    }
    
    function subscription($subscription_id)
    {
        $model = subscription_model();
        return $model->getSubscriptionById($subscription_id);
    }
    
    function subscription_payment_by_transaction_id($transaction_id)
    {
        $model = subscription_model();
        return $model->getSubscriptionPaymentByTransactionId($transaction_id);
    }
    
    function subscription_payment_by_transaction_reference($transaction_reference)
    {
        $model = subscription_model();
        return $model->getSubscriptionPaymentByTransactionReference($transaction_reference);
    }
    
    function has_valid_subscription($user_id)
    {
        $model = subscription_model();
        $results = $model->getActiveSubscriptions($user_id, 'subscription_end_date');
        $sub_arr_id = array();
        if ($results) {
            foreach ($results as $user_subscription) {
                if ($user_subscription->subscription_status_id == 2 && $user_subscription->subscription_end_date > now()) {
                    $sub_arr_id[] = $user_subscription->subscription_id;
                }
            }
        } else {
            $sub_arr_id[] = FALSE;
        }
        return !empty($sub_arr_id) ? subscription($sub_arr_id[0]) : False;
    }
    
    function check_subscription($user_id, $controller)
    {
        if (!has_valid_subscription($user_id)) {
            flash('user_message', $controller, 'you_do_not_have_active_subscription', 'danger');
            redirect($controller);
            die();
        }
    }
    
    function subscription_end_date($subscription_id)
    {
        $subscription_end_date = subscription($subscription_id)->subscription_end_date;
        if ($subscription_end_date < now()) {
            $subscription_end_date = text_bold_red(dateTimeFormat($subscription_end_date));
        } else {
            $subscription_end_date = text_normal(dateTimeFormat($subscription_end_date));
        }
        return $subscription_end_date;
    }
    
    function minimum_payment_acceptable($amount)
    {
        if ($amount < getOption('minimum_payment_acceptable')) {
            $amount = getOption('minimum_payment_acceptable');
        }
        return number_format($amount, 2);
    }
    
    function subscriptions_between_dates($from_date, $to_date, $added_by = NULL)
    {
        $model = subscription_model();
        if (isSuperAdmin()) {
            $results = $model->getSubscriptionsBetweenDates($from_date, $to_date);
        } elseif (isset($added_by)) {
            $results = $model->getSubscriptionsBetweenDates($from_date, $to_date, $added_by);
        } else {
            $results = $model->getSubscriptionsBetweenDates($from_date, $to_date, user_id());
        }
        return $results;
    }
    
    function subscriptions_by_status($status_id, $added_by = NULL)
    {
        $model = subscription_model();
        if (isSuperAdmin()) {
            $results = $model->getSubscriptionsByStatus($status_id);
        } elseif (isset($added_by)) {
            $results = $model->getSubscriptionsByStatus($status_id, $added_by);
        } else {
            $results = $model->getSubscriptionsByStatus($status_id, user_id());
        }
        return $results;
    }
    
    function total_subscriptions_income_between_dates($from_date, $to_date)
    {
        $model = subscription_model();
        return $model->getSubscriptionsBetweenDatesTotals($from_date, $to_date);
    }
    
    function subscriptions_totals()
    {
        $model = subscription_model();
        return $model->getSubscriptionsTotals();
    }
    
    function subscription_statuses($subscription_status_id = NULL)
    {
        $model = subscription_model();
        if (isset($subscription_status_id)) {
            $results = $model->getSubscriptionStatusById($subscription_status_id);
        } else {
            $results = $model->getSubscriptionStatus();
        }
        return $results;
    }