<?php
  /*
   * PDO Database Class
   * Connect to database
   * Create prepared statements
   * Bind values
   * Return rows and results
   */
  class Database {
    private $host = DB_HOST;
    private $user = DB_USER;
    private $pass = DB_PASS;
    private $dbname = DB_NAME;

    public $dbh;
    public $stmt;
    private $error;

    public function __construct(){
      // Set DSN
      $dsn = 'mysql:host=' . $this->host . ';dbname=' . $this->dbname . ';charset=utf8mb4';
      $options = array(
        PDO::ATTR_PERSISTENT => true,
        PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION
      );

      // Create PDO instance
      try{
        $this->dbh = new PDO($dsn, $this->user, $this->pass, $options);
      } catch(PDOException $e){
        $this->error = $e->getMessage();
        echo $this->error;
      }
    }


      public function insert($table, $fields = array()){
          if(count($fields)){
              $keys = array_keys($fields);
              $values = '';
              $x=1;

              foreach($fields as $field){
                  $values .= '?';
                  if($x < count($fields)){
                      $values .= ', ';
                  }
                  $x++;
              }

              $sql = "INSERT INTO {$table} (`" . implode('`, `', $keys). "`) VALUES({$values})";
              if(!$this->query($sql, $fields)->error()){
                  return true;
              }
          }
          return false;
      }

    // Prepare statement with query
    public function query($sql){
      $this->stmt = $this->dbh->prepare($sql);
    }

    // Bind values
    public function bind($param, $value, $type = null){
      if(is_null($type)){
        switch(true){
          case is_int($value):
            $type = PDO::PARAM_INT;
            break;
          case is_bool($value):
            $type = PDO::PARAM_BOOL;
            break;
          case is_null($value):
            $type = PDO::PARAM_NULL;
            break;
          default:
            $type = PDO::PARAM_STR;
        }
      }

      $this->stmt->bindValue($param, $value, $type);
    }

    // Execute the prepared statement
    public function execute(){
		try {
		    return $this->stmt->execute();
		} catch (PDOException $e) {
		    return $e;
		}
    }

    // Get result set as array of objects
    public function resultSet(){
      $this->execute();
      return $this->stmt->fetchAll(PDO::FETCH_OBJ);
    }

    // Get single record as object
    public function single(){
      $this->execute();
      return $this->stmt->fetch(PDO::FETCH_OBJ);
    }
    
    // Get result set as array of objects
    public function resultSetAssoc(){
      $this->execute();
      return $this->stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    // Get result set as array of objects
    public function resultSetNum(){
      $this->execute();
      return $this->stmt->fetchAll(PDO::FETCH_NUM);
    }
    
    // Get single num record as object
    public function singleNum(){
      $this->stmt->execute();
      return $this->stmt->fetch(PDO::FETCH_NUM);
    }
    
    // Get single assoc record as object
    public function singleAssoc(){
      $this->stmt->execute();
      return $this->stmt->fetch(PDO::FETCH_ASSOC);
    }
    
    // Get column count
    public function columnCount(){
      $this->stmt->execute();
      return $this->stmt->columnCount();
    }
    
    // Get row count
    public function rowCount(){
      return $this->stmt->rowCount();
    }
    
    // Get field count
    public function fieldCount(){
      return $this->stmt->fieldCount();
    }
    
    // Get last insert id
    public function lastInsertId(){
      return $this->dbh->lastInsertId();
    }
  }