<?php
    
    class Assistant
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getChatAssistants($user_id = NULL)
        {
            $query = "SELECT * FROM chat_assistants ";
            if (isset($user_id)) {
                $query .= " WHERE chat_assistant_added_by = :chat_assistant_added_by";
            }
            $query .= " ORDER BY chat_assistant_id = 1 Desc, chat_assistant_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_assistant_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getSelectedChatAssistants($ids, $user_id = NULL)
        {
            $ids = implode(', ', $ids);
            $query = "SELECT * FROM chat_assistants WHERE chat_assistant_id IN (" . $ids . ") ";
            if (isset($user_id)) {
                $query .= " AND chat_assistant_added_by = :chat_assistant_added_by ";
            }
            $query .= " ORDER BY chat_assistant_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_assistant_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function searchChatAssistants($search_term)
        {
            $this->db->query("SELECT * FROM chat_assistants WHERE chat_assistant_name LIKE '%" . $search_term . "%' OR chat_assistant_profession LIKE '%" . $search_term . "%'");
            return $this->db->resultSet();
        }
        
        public function getChatAssistantById($id)
        {
            $this->db->query('SELECT * FROM chat_assistants
WHERE chat_assistant_id = :chat_assistant_id');
            $this->db->bind(':chat_assistant_id', $id);
    
            return $this->db->single();
        }
        
        public function getChatAssistantByUid($id)
        {
            $this->db->query('SELECT * FROM chat_assistants
WHERE chat_assistant_uid = :chat_assistant_uid');
            $this->db->bind(':chat_assistant_uid', $id);
    
            return $this->db->single();
        }
        
        public function getChatAssistantByName($name)
        {
            $this->db->query('SELECT * FROM chat_assistants
WHERE chat_assistant_name = :chat_assistant_name');
            $this->db->bind(':chat_assistant_name', $name);
    
            return $this->db->single();
        }
        
        
        public function addChatAssistant($data)
        {
            $this->db->query('INSERT INTO chat_assistants(chat_assistant_uid,chat_assistant_added_by,chat_assistant_name,chat_assistant_logo,chat_assistant_profession,chat_assistant_prompt,chat_assistant_status_id,chat_assistant_date_added) VALUES(:chat_assistant_uid,:chat_assistant_added_by,:chat_assistant_name,:chat_assistant_logo,:chat_assistant_profession,:chat_assistant_prompt,:chat_assistant_status_id,:chat_assistant_date_added)');
// Bind values
            $this->db->bind(':chat_assistant_uid', uid());
            $this->db->bind(':chat_assistant_added_by', user_id());
            $this->db->bind(':chat_assistant_name', $data['chat_assistant_name']);
            $this->db->bind(':chat_assistant_logo', $data['chat_assistant_logo']);
            $this->db->bind(':chat_assistant_profession', $data['chat_assistant_profession']);
            $this->db->bind(':chat_assistant_prompt', $data['chat_assistant_prompt']);
            $this->db->bind(':chat_assistant_status_id', $data['chat_assistant_status_id']);
            $this->db->bind(':chat_assistant_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updateChatAssistant($data)
        {
            $this->db->query('UPDATE chat_assistants SET chat_assistant_name = :chat_assistant_name, chat_assistant_profession = :chat_assistant_profession, chat_assistant_prompt = :chat_assistant_prompt, chat_assistant_status_id = :chat_assistant_status_id WHERE chat_assistant_id = :chat_assistant_id');
// Bind values
            $this->db->bind(':chat_assistant_name', $data['chat_assistant_name']);
            $this->db->bind(':chat_assistant_profession', $data['chat_assistant_profession']);
            $this->db->bind(':chat_assistant_prompt', $data['chat_assistant_prompt']);
            $this->db->bind(':chat_assistant_status_id', $data['chat_assistant_status_id']);
            $this->db->bind(':chat_assistant_id', $data['chat_assistant_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateChatAssistantStatus($data)
        {
            $this->db->query('UPDATE chat_assistants SET chat_assistant_status_id = :chat_assistant_status_id WHERE chat_assistant_id = :chat_assistant_id');
// Bind values
            $this->db->bind(':chat_assistant_status_id', $data['chat_assistant_status_id']);
            $this->db->bind(':chat_assistant_id', $data['chat_assistant_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateChatAssistantImage($data)
        {
            $this->db->query('UPDATE chat_assistants SET chat_assistant_logo = :chat_assistant_logo WHERE chat_assistant_id = :chat_assistant_id');
// Bind values
            $this->db->bind(':chat_assistant_logo', $data['chat_assistant_logo']);
            $this->db->bind(':chat_assistant_id', $data['chat_assistant_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function deleteChatAssistant($id)
        {
            $this->db->query('DELETE FROM chat_assistants WHERE chat_assistant_id = :chat_assistant_id');
// Bind values
            $this->db->bind(':chat_assistant_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
        //FILES
        public function getAssistantFiles($user_id = NULL)
        {
            $query = "SELECT * FROM assistant_files ";
            if (isset($user_id)) {
                $query .= " WHERE assistant_file_added_by = :assistant_file_added_by";
            }
            $query .= " ORDER BY assistant_file_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':assistant_file_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addAssistantFile($data)
        {
            $this->db->query('INSERT INTO assistant_files(assistant_file,assistant_file_size,assistant_file_added_by,assistant_file_date_added) VALUES(:assistant_file,:assistant_file_size,:assistant_file_added_by,:assistant_file_date_added)');
// Bind values
            $this->db->bind(':assistant_file', $data['assistant_file']);
            $this->db->bind(':assistant_file_size', $data['assistant_file_size']);
            $this->db->bind(':assistant_file_added_by', user_id());
            $this->db->bind(':assistant_file_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updateAssistantFile($data)
        {
            $this->db->query('UPDATE assistant_files SET assistant_id = :assistant_id, assistant_file_output_id = :assistant_file_output_id, assistant_file_output_status = :assistant_file_output_status WHERE assistant_file_id = :assistant_file_id');
// Bind values
            $this->db->bind(':assistant_id', $data['assistant_id']);
            $this->db->bind(':assistant_file_output_id', $data['assistant_file_output_id']);
            $this->db->bind(':assistant_file_output_status', $data['assistant_file_output_status']);
            $this->db->bind(':assistant_file_id', $data['assistant_file_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getAssistantFileById($id)
        {
            $this->db->query('SELECT * FROM assistant_files
WHERE assistant_file_id = :assistant_file_id');
            $this->db->bind(':assistant_file_id', $id);
    
            return $this->db->single();
        }
        
        public function getAssistantFileByOutputId($id)
        {
            $this->db->query('SELECT * FROM assistant_files
WHERE assistant_file_output_id = :assistant_file_output_id');
            $this->db->bind(':assistant_file_output_id', $id);
    
            return $this->db->single();
        }
        
    }
    
