<?php
    
    class Chat
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getChats($user_id = NULL)
        {
            $query = "SELECT * FROM chats ";
            if (isset($user_id)) {
                $query .= " WHERE chat_added_by = :chat_added_by ";
            }
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getWhatsappChats($user_id)
        {
            $query = "SELECT * FROM chats ";
            $query .= " WHERE chat_added_by = :chat_added_by ";
            $query .= " AND source = :source ";
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
            $this->db->bind(':chat_added_by', $user_id);
            $this->db->bind(':source', 'whatsapp');
            return $this->db->resultSet();
        }
        
        public function addChat($data)
        {
            $this->db->query('INSERT INTO chats (chat_uid, chat_subject, source, source_id, chat_file_id, visit_id, chat_ip_address, chat_added_by, chat_date_added) VALUES(:chat_uid, :chat_subject, :source, :source_id, :chat_file_id, :visit_id, :chat_ip_address, :chat_added_by, :chat_date_added)');
// Bind values
            $this->db->bind(':chat_uid', getOption('chat_prefix') . '-' . uid());
            $this->db->bind(':chat_subject', $data['chat_subject']);
            $this->db->bind(':source', $data['source'] ?? NULL);
            $this->db->bind(':source_id', $data['source_id'] ?? NULL);
            $this->db->bind(':chat_file_id', $data['chat_file_id'] ?? NULL);
            $this->db->bind(':visit_id', $data['visit_id']);
            $this->db->bind(':chat_ip_address', $data['chat_ip_address'] ?? NULL);
            $this->db->bind(':chat_added_by', $data['chat_added_by']);
            $this->db->bind(':chat_date_added', now());
// Execute

            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function nextChatId()
        {
            $this->db->query("SHOW TABLE STATUS LIKE 'chats'");
            $row = $this->db->single();
            return $row->Auto_increment;
        }
        
        public function updateChat($data)
        {
            $this->db->query('UPDATE chats SET chat_subject = :chat_subject WHERE chat_id = :chat_id');
// Bind values
            $this->db->bind(':chat_subject', $data['chat_subject']);
            $this->db->bind(':chat_id', $data['chat_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function endChatSession($data)
        {
            $this->db->query('UPDATE chats SET chat_date_ended = :chat_date_ended WHERE chat_id = :chat_id');
// Bind values
            $this->db->bind(':chat_date_ended', now());
            $this->db->bind(':chat_id', $data['chat_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function searchChat($data)
        {
            $this->db->query('SELECT * FROM chats WHERE chat_subject LIKE "%' . $data['term'] . '%" ORDER BY chat_id Desc');
            return $this->db->resultSet();
            
        }
        
        public function getLastChat($is_bot = NULL)
        {
            $this->db->query('SELECT * FROM chats ORDER BY chat_id Desc LIMIT 1');
    
            return $this->db->single();
            
        }
        
        public function getChatById($id)
        {
            $this->db->query('SELECT * FROM chats
WHERE chat_id = :chat_id');
            $this->db->bind(':chat_id', $id);
    
            return $this->db->single();
            
        }
        
        public function getChatBySubject($chat_subject, $user_id)
        {
            $this->db->query('SELECT * FROM chats WHERE chat_subject = :chat_subject AND chat_added_by = :chat_added_by');
            $this->db->bind(':chat_subject', $chat_subject);
            $this->db->bind(':chat_added_by', $user_id);
            return $this->db->single();
        }
        
        public function getChatsByVisit($visit_id = NULL)
        {
            $query = "SELECT * FROM chats WHERE visit_id = :visit_id";
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':visit_id', $visit_id);
            return $this->db->resultSet();
        }
        
        public function getChatsBetweenDates($from_date, $to_date, $user_id = NULL)
        {
            $query = "SELECT * FROM chats WHERE chat_date_added BETWEEN :from_date AND :to_date";
            if (isset($user_id)) {
                $query .= " AND chat_added_by = :chat_added_by";
            }
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            if (isset($user_id)) {
                $this->db->bind(':chat_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getChatAssistantsUsage()
        {
            $query = "SELECT COUNT(source_id) as assistant_id_count, source_id FROM chats WHERE source_id != 0 AND source = 'assistant'";
            $query .= " GROUP BY source_id ORDER BY assistant_id_count DESC ";
            $this->db->query($query);
            return $this->db->resultSet();
        }
        
        public function deleteChat($id)
        {
            $this->db->query('DELETE FROM chats WHERE chat_id = :chat_id');
// Bind values
            $this->db->bind(':chat_id', $id);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getChatMessages($chat_id = NULL)
        {
            $query = "SELECT * FROM chat_messages ";
            if (isset($chat_id)) {
                $query .= " WHERE chat_id = :chat_id ";
            }
            $query .= " ORDER BY chat_message_id Asc";
            $this->db->query($query);
            if (isset($chat_id)) {
                $this->db->bind(':chat_id', $chat_id);
            }
            return $this->db->resultSet();
        }
        
        public function addChatMessage($data)
        {
            $this->db->query('INSERT INTO chat_messages (chat_id, is_bot, assistant_file_id, chat_message, chat_message_added_by, chat_message_date_added) VALUES(:chat_id, :is_bot, :assistant_file_id, :chat_message, :chat_message_added_by, :chat_message_date_added)');
// Bind values
            $this->db->bind(':chat_id', $data['chat_id']);
            $this->db->bind(':is_bot', $data['is_bot']);
            $this->db->bind(':assistant_file_id', $data['assistant_file_id']);
            $this->db->bind(':chat_message', $data['message']);
            $this->db->bind(':chat_message_added_by', $data['chat_message_added_by']);
            $this->db->bind(':chat_message_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getLastChatMessage($is_bot = NULL)
        {
            $query = "SELECT * FROM chat_messages ";
            if (isset($is_bot)) {
                $query .= " WHERE is_bot = 1";
            }
            $query .= " ORDER BY chat_message_id Desc LIMIT 1";
            $this->db->query($query);
            return $this->db->single();
        }
        
        
        public function getLastChatBySource($user_id, $source, $source_id)
        {
            $this->db->query('SELECT * FROM chats
WHERE chat_added_by = :chat_added_by AND source = :source AND source_id = :source_id ORDER BY chat_id Desc LIMIT 1');
            $this->db->bind(':chat_added_by', $user_id);
            $this->db->bind(':source', $source);
            $this->db->bind(':source_id', $source_id);
            return $this->db->single();
        }
        
        
        public function getLastChatBySourceOwner($user_id, $source, $source_id)
        {
            $this->db->query('SELECT * FROM chats
WHERE chat_added_by = :chat_added_by AND source = :source AND source_id = :source_id AND visit_id IS NULL ORDER BY chat_id Desc LIMIT 1');
            $this->db->bind(':chat_added_by', $user_id);
            $this->db->bind(':source', $source);
            $this->db->bind(':source_id', $source_id);
            return $this->db->single();
        }
        
        public function getLastChatByIpAddress($ip_address, $source = NULL, $source_id = NULL)
        {
            $query = 'SELECT * FROM chats WHERE chat_ip_address = :chat_ip_address ';
            $query .= ' AND chat_date_ended IS NULL ';
            if (isset($source) && isset($source_id)) {
                $query .= ' AND source = :source AND source_id = :source_id ';
            }
            $query .= ' ORDER BY chat_id Desc LIMIT 1';
            $this->db->query($query);
            $this->db->bind(':chat_ip_address', $ip_address);
            if (isset($source) && isset($source_id)) {
                $this->db->bind(':source', $source);
                $this->db->bind(':source_id', $source_id);
            }
            return $this->db->single();
        }
        
        public function getSourceChatsByIpAddress($ip_address, $source = NULL, $source_id = NULL)
        {
            $query = "SELECT * FROM chats ";
            $query .= " WHERE chat_ip_address = :chat_ip_address ";
            if (isset($source) && isset($source_id)) {
                $query .= ' AND source = :source AND source_id = :source_id ';
            }
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
            $this->db->bind(':chat_ip_address', $ip_address);
            if (isset($source) && isset($source_id)) {
                $this->db->bind(':source', $source);
                $this->db->bind(':source_id', $source_id);
            }
            return $this->db->resultSet();
        }
        
        public function getSourceChats($user_id, $source, $source_id, $external_chat = NULL)
        {
            $query = "SELECT * FROM chats ";
            $query .= " WHERE chat_added_by = :chat_added_by ";
            $query .= " AND source = :source ";
            $query .= " AND source_id = :source_id ";
            if (!isset($external_chat)) {
                $query .= " AND chat_ip_address IS NULL ";
            }
            $query .= " ORDER BY chat_id Desc";
            $this->db->query($query);
            $this->db->bind(':chat_added_by', $user_id);
            $this->db->bind(':source', $source);
            $this->db->bind(':source_id', $source_id);
            return $this->db->resultSet();
        }
    }