<?php
    
    class Chatfile
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getChatFiles($user_id = NULL)
        {
            $query = "SELECT * FROM chat_files WHERE chat_link IS NULL AND is_sitemap IS NULL";
            if (isset($user_id)) {
                $query .= " AND chat_file_added_by = :chat_file_added_by ";
            }
            $query .= " ORDER BY chat_file_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_file_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addChatFile($data)
        {
            $this->db->query('INSERT INTO chat_files (chat_file, chat_file_size, chat_link, is_sitemap, chat_file_added_by, chat_file_date_added) VALUES(:chat_file, :chat_file_size, :chat_link, :is_sitemap, :chat_file_added_by, :chat_file_date_added)');
// Bind values
            $this->db->bind(':chat_file', $data['chat_file']);
            $this->db->bind(':chat_file_size', $data['chat_file_size']);
            $this->db->bind(':chat_link', $data['chat_link'] ?? NULL);
            $this->db->bind(':is_sitemap', $data['is_sitemap'] ?? NULL);
            $this->db->bind(':chat_file_added_by', user_id());
            $this->db->bind(':chat_file_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updateChatFileOutput($data)
        {
            $this->db->query('UPDATE chat_files SET chat_filename = :chat_filename, chat_file_output_id = :chat_file_output_id, chat_file_output_filename = :chat_file_output_filename, chat_file_output_size = :chat_file_output_size, chat_file_output_status = :chat_file_output_status WHERE chat_file_id = :chat_file_id');
// Bind values
            $this->db->bind(':chat_filename', $data['chat_filename'] ?? NULL);
            $this->db->bind(':chat_file_output_id', $data['chat_file_output_id']);
            $this->db->bind(':chat_file_output_filename', $data['chat_file_output_filename']);
            $this->db->bind(':chat_file_output_size', $data['chat_file_output_size']);
            $this->db->bind(':chat_file_output_status', $data['chat_file_output_status']);
            $this->db->bind(':chat_file_id', $data['chat_file_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateChatJobOutput($data)
        {
            $this->db->query('UPDATE chat_files SET chat_job_output_id = :chat_job_output_id, chat_job_output_status = :chat_job_output_status, chat_job_output_model = :chat_job_output_model, chat_job_output_tokens = :chat_job_output_tokens WHERE chat_id = :chat_id');
// Bind values
            $this->db->bind(':chat_job_output_id', $data['chat_job_output_id']);
            $this->db->bind(':chat_job_output_status', $data['chat_job_output_status']);
            $this->db->bind(':chat_job_output_model', $data['chat_job_output_model']);
            $this->db->bind(':chat_job_output_tokens', $data['chat_job_output_tokens']);
            $this->db->bind(':chat_id', $data['chat_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getChatFileById($id)
        {
            $this->db->query('SELECT * FROM chat_files
WHERE chat_file_id = :chat_file_id');
            $this->db->bind(':chat_file_id', $id);
    
            return $this->db->single();
            
        }
        
        public function getChatFileByLink($name)
        {
            $this->db->query('SELECT * FROM chat_files
WHERE chat_link = :chat_link');
            $this->db->bind(':chat_link', $name);
    
            return $this->db->single();
            
        }
        
        public function deleteChatFile($id)
        {
            $this->db->query('DELETE FROM chat_files WHERE chat_file_id = :chat_file_id');
// Bind values
            $this->db->bind(':chat_file_id', $id);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getChatFileByOutputId($id)
        {
            $this->db->query('SELECT * FROM chat_files
WHERE chat_file_output_id = :chat_file_output_id');
            $this->db->bind(':chat_file_output_id', $id);
    
            return $this->db->single();
        }
        
        
        //Chat Links
        public function getChatLinks($user_id = NULL)
        {
            $query = "SELECT * FROM chat_files WHERE chat_link IS NOT NULL AND is_sitemap IS NULL";
            if (isset($user_id)) {
                $query .= " AND chat_file_added_by = :chat_file_added_by ";
            }
            $query .= " ORDER BY chat_file_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_file_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        
        //Chat Sitemaps
        public function getChatSitemaps($user_id = NULL)
        {
            $query = "SELECT * FROM chat_files WHERE chat_link IS NOT NULL AND is_sitemap IS NOT NULL";
            if (isset($user_id)) {
                $query .= " AND chat_file_added_by = :chat_file_added_by ";
            }
            $query .= " ORDER BY chat_file_id Desc";
            
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':chat_file_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addChatSitemapLink($data)
        {
            $this->db->query('INSERT INTO chat_sitemap_links (chat_file_id, chat_sitemap_link, chat_sitemap_file_name, chat_sitemap_link_size) VALUES(:chat_file_id, :chat_sitemap_link, :chat_sitemap_file_name, :chat_sitemap_link_size)');
// Bind values
            $this->db->bind(':chat_file_id', $data['chat_file_id']);
            $this->db->bind(':chat_sitemap_link', $data['chat_sitemap_link']);
            $this->db->bind(':chat_sitemap_file_name', $data['chat_sitemap_file_name']);
            $this->db->bind(':chat_sitemap_link_size', $data['chat_sitemap_link_size']);
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getChatSitemapLinks($chat_file_id)
        {
            $query = "SELECT * FROM chat_sitemap_links WHERE chat_file_id = :chat_file_id ";
            $query .= " ORDER BY chat_sitemap_link_id Asc";
            $this->db->query($query);
            $this->db->bind(':chat_file_id', $chat_file_id);
            return $this->db->resultSet();
        }
        
        public function updateChatSitemapLinkOutput($data)
        {
            $this->db->query('UPDATE chat_sitemap_links SET chat_sitemap_link_output_id = :chat_sitemap_link_output_id, chat_sitemap_link_output_status = :chat_sitemap_link_output_status, chat_sitemap_link_output_status_date_added = :chat_sitemap_link_output_status_date_added WHERE chat_sitemap_link_id = :chat_sitemap_link_id');
// Bind values
            $this->db->bind(':chat_sitemap_link_output_id', $data['chat_sitemap_link_output_id']);
            $this->db->bind(':chat_sitemap_link_output_status', $data['chat_sitemap_link_output_status']);
            $this->db->bind(':chat_sitemap_link_output_status_date_added', now());
            $this->db->bind(':chat_sitemap_link_id', $data['chat_sitemap_link_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getChatSitemapLinkById($id)
        {
            $this->db->query('SELECT * FROM chat_sitemap_links
WHERE chat_sitemap_link_id = :chat_sitemap_link_id');
            $this->db->bind(':chat_sitemap_link_id', $id);
    
            return $this->db->single();
        }
        
        public function cancelChatSitemapLink($id)
        {
            $this->db->query('UPDATE chat_sitemap_links SET chat_sitemap_link_output_id = :chat_sitemap_link_output_id, chat_sitemap_link_output_status = :chat_sitemap_link_output_status, chat_sitemap_link_output_status_date_added = :chat_sitemap_link_output_status_date_added WHERE chat_sitemap_link_id = :chat_sitemap_link_id');
// Bind values
            $this->db->bind(':chat_sitemap_link_output_id', NULL);
            $this->db->bind(':chat_sitemap_link_output_status', NULL);
            $this->db->bind(':chat_sitemap_link_output_status_date_added', NULL);
            $this->db->bind(':chat_sitemap_link_id', $id);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function deleteChatSitemapLink($id)
        {
            $this->db->query('DELETE FROM chat_sitemap_links WHERE chat_sitemap_link_id = :chat_sitemap_link_id');
// Bind values
            $this->db->bind(':chat_sitemap_link_id', $id);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
    }