<?php
    
    class Finetune
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getFineTunes($user_id = NULL)
        {
            $query = "SELECT * FROM finetunes ";
            if (isset($user_id)) {
                $query .= " WHERE finetune_file_added_by = :finetune_file_added_by ";
            }
            $query .= " ORDER BY finetune_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':finetune_file_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addFineTuneFile($data)
        {
            $this->db->query('INSERT INTO finetunes (finetune_file, finetune_file_size, finetune_file_added_by, finetune_file_date_added) VALUES(:finetune_file, :finetune_file_size, :finetune_file_added_by, :finetune_file_date_added)');
// Bind values
            $this->db->bind(':finetune_file', $data['finetune_file']);
            $this->db->bind(':finetune_file_size', $data['finetune_file_size']);
            $this->db->bind(':finetune_file_added_by', user_id());
            $this->db->bind(':finetune_file_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updateFineTuneFileOutput($data)
        {
            $this->db->query('UPDATE finetunes SET finetune_file_output_id = :finetune_file_output_id, finetune_file_output_filename = :finetune_file_output_filename, finetune_file_output_size = :finetune_file_output_size, finetune_file_output_status = :finetune_file_output_status WHERE finetune_id = :finetune_id');
// Bind values
            $this->db->bind(':finetune_file_output_id', $data['finetune_file_output_id']);
            $this->db->bind(':finetune_file_output_filename', $data['finetune_file_output_filename']);
            $this->db->bind(':finetune_file_output_size', $data['finetune_file_output_size']);
            $this->db->bind(':finetune_file_output_status', $data['finetune_file_output_status']);
            $this->db->bind(':finetune_id', $data['finetune_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateFineTuneJobOutput($data)
        {
            $this->db->query('UPDATE finetunes SET finetune_job_output_id = :finetune_job_output_id, finetune_job_output_status = :finetune_job_output_status, finetune_job_output_model = :finetune_job_output_model, finetune_job_output_tokens = :finetune_job_output_tokens, finetune_job_output_model_used = :finetune_job_output_model_used WHERE finetune_id = :finetune_id');
// Bind values
            $this->db->bind(':finetune_job_output_id', $data['finetune_job_output_id']);
            $this->db->bind(':finetune_job_output_status', $data['finetune_job_output_status']);
            $this->db->bind(':finetune_job_output_model', $data['finetune_job_output_model']);
            $this->db->bind(':finetune_job_output_tokens', $data['finetune_job_output_tokens']);
            $this->db->bind(':finetune_job_output_model_used', $data['finetune_job_output_model_used']);
            $this->db->bind(':finetune_id', $data['finetune_id']);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getFineTuneById($id)
        {
            $this->db->query('SELECT * FROM finetunes
WHERE finetune_id = :finetune_id');
            $this->db->bind(':finetune_id', $id);
    
            return $this->db->single();
            
        }
        
        public function getFinetuneByOutputModel($finetune_job_output_model)
        {
            $this->db->query('SELECT * FROM finetunes WHERE finetune_job_output_model = :finetune_job_output_model');
            $this->db->bind(':finetune_job_output_model', $finetune_job_output_model);
            return $this->db->single();
        }
        
        public function getFinetunesBetweenDates($from_date, $to_date, $user_id = NULL)
        {
            $query = "SELECT * FROM finetunes WHERE finetune_date_added BETWEEN :from_date AND :to_date";
            if (isset($user_id)) {
                $query .= " AND finetune_added_by = :finetune_added_by";
            }
            $query .= " ORDER BY finetune_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            if (isset($user_id)) {
                $this->db->bind(':finetune_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function deleteFinetune($id)
        {
            $this->db->query('DELETE FROM finetunes WHERE finetune_id = :finetune_id');
// Bind values
            $this->db->bind(':finetune_id', $id);

// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
    }