<?php
    
    class Notification
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getNotifications()
        {
            $query = "SELECT * FROM notifications";
            $query .= " WHERE to_user_id = :to_user_id";
            $query .= " ORDER BY notification_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':to_user_id', user_id());
            return $this->db->resultSet();
        }
        
        public function getNotificationById($id)
        {
            $this->db->query('SELECT * FROM notifications WHERE notification_id = :notification_id');
            $this->db->bind(':notification_id', $id);
            return $this->db->single();
        }
        
        public function addNotification($notification, $to_user, $link, $is_whatsapp)
        {
            $this->db->query('INSERT INTO notifications (notification_uid, notification_date_added, notification_description, notification_added_by, to_user_id, is_whatsapp, link) VALUES(:notification_uid, :notification_date_added, :notification_description, :notification_added_by, :to_user_id, :is_whatsapp, :link)');
// Bind values
            $this->db->bind(':notification_uid', uid());
            $this->db->bind(':notification_date_added', now());
            $this->db->bind(':notification_description', $notification);
            $this->db->bind(':notification_added_by', !empty(user_id()) ? user_id() : NULL);
            $this->db->bind(':to_user_id', $to_user);
            $this->db->bind(':is_whatsapp', isset($is_whatsapp) ? 1 : NULL);
            $this->db->bind(':link', $link);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getUnreadNotifications($is_whatsapp=NULL)
        {
            $query = "SELECT * FROM notifications
            WHERE to_user_id = :to_user_id 
            AND is_notification_read = :is_notification_read ";
            if(isset($is_whatsapp)){
                $query .= " AND is_whatsapp = :is_whatsapp ";
            }
            $query .= " ORDER BY notification_id Desc ";
            $this->db->query($query);
//Bind values
            $this->db->bind(':to_user_id', user_id());
            $this->db->bind(':is_notification_read', 0);
            if(isset($is_whatsapp)){
                $this->db->bind(':is_whatsapp', 1);
            }
            return $this->db->resultSet();
        }
        
        public function markNotificationAsRead($notification_id)
        {
            $this->db->query('UPDATE notifications SET is_notification_read = :is_notification_read WHERE notification_id = :notification_id AND to_user_id = :to_user_id');
// Bind values
            $this->db->bind(':is_notification_read', 1);
            $this->db->bind(':notification_id', $notification_id);
            $this->db->bind(':to_user_id', user_id());
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
    }