<?php
    
    class Package
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getPackages($is_custom = NULL)
        {
            $query = "SELECT * FROM packages ";
            if(isset($is_custom)){
            	$query .= "  WHERE is_custom = :is_custom ";
			}
            $query .= " ORDER BY CASE WHEN is_free_trial = 0 THEN 0 ELSE 1 END, is_free_trial desc, package_cost, package_monthly_cost, package_yearly_cost, package_lifetime_cost Asc";
            $this->db->query($query);
            if(isset($is_custom)){
                $this->db->bind(':is_custom', $is_custom);
            }
            return $this->db->resultSet();
        }
        
        public function getGroupedPackages($group = NULL)
        {
            $query = "SELECT * FROM packages ";
            if(isset($group)){
                if($group=='others'){
                    $query .= "  WHERE package_duration > :package_duration ";
                }else{
                    $query .= "  WHERE package_duration = :package_duration ";
                    $query .= "  AND package_duration_type = '$group' ";
                }
            }
            $query .= " ORDER BY CASE WHEN package_cost = 0 THEN 0 ELSE 1 END, package_cost Asc";
            $this->db->query($query);
            if(isset($group)){
                $this->db->bind(':package_duration', 1);
            }
            return $this->db->resultSet();
        }
        
        public function addPackage($data)
        {
            $this->db->query('INSERT INTO packages (package_name, package_duration, package_duration_type, package_chat_assistants, package_trained_models, package_chat_files, package_chat_links, package_chat_sitemaps, package_words, accept_custom_branding, package_cost, package_monthly_cost, package_yearly_cost, package_lifetime_cost, is_custom, is_free_trial, is_private) VALUES(:package_name, :package_duration, :package_duration_type, :package_chat_assistants, :package_trained_models, :package_chat_files, :package_chat_links, :package_chat_sitemaps, :package_words, :accept_custom_branding, :package_cost, :package_monthly_cost, :package_yearly_cost, :package_lifetime_cost, :is_custom, :is_free_trial, :is_private)');
// Bind values
            $this->db->bind(':package_name', $data['package_name']);
            $this->db->bind(':package_duration', $data['package_duration']);
            $this->db->bind(':package_duration_type', $data['package_duration_type']);
            $this->db->bind(':package_chat_assistants', $data['package_chat_assistants']);
            $this->db->bind(':package_trained_models', $data['package_trained_models']);
            $this->db->bind(':package_chat_files', $data['package_chat_files']);
            $this->db->bind(':package_chat_links', $data['package_chat_links']);
            $this->db->bind(':package_chat_sitemaps', $data['package_chat_sitemaps']);
            $this->db->bind(':package_words', $data['package_words']);
            $this->db->bind(':accept_custom_branding', $data['accept_custom_branding']);
            $this->db->bind(':package_cost', $data['package_cost']);
            $this->db->bind(':package_monthly_cost', $data['package_monthly_cost']);
            $this->db->bind(':package_yearly_cost', $data['package_yearly_cost']);
            $this->db->bind(':package_lifetime_cost', $data['package_lifetime_cost']);
            $this->db->bind(':is_custom', $data['is_custom']);
            $this->db->bind(':is_free_trial', $data['is_free_trial']);
            $this->db->bind(':is_private', $data['is_private']);
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updatePackage($data)
        {
            $this->db->query('UPDATE packages SET package_name = :package_name, package_duration = :package_duration, package_duration_type = :package_duration_type, package_chat_assistants = :package_chat_assistants, package_trained_models = :package_trained_models, package_chat_files = :package_chat_files, package_chat_links = :package_chat_links, package_chat_sitemaps = :package_chat_sitemaps, package_words = :package_words, accept_custom_branding = :accept_custom_branding, is_private  =  :is_private, package_cost = :package_cost, package_monthly_cost = :package_monthly_cost, package_yearly_cost = :package_yearly_cost, package_lifetime_cost = :package_lifetime_cost WHERE package_id = :package_id');
// Bind values
            $this->db->bind(':package_name', $data['package_name']);
            $this->db->bind(':package_duration', $data['package_duration']);
            $this->db->bind(':package_duration_type', $data['package_duration_type']);
            $this->db->bind(':package_chat_assistants', $data['package_chat_assistants']);
            $this->db->bind(':package_trained_models', $data['package_trained_models']);
            $this->db->bind(':package_chat_files', $data['package_chat_files']);
            $this->db->bind(':package_chat_links', $data['package_chat_links']);
            $this->db->bind(':package_chat_sitemaps', $data['package_chat_sitemaps']);
            $this->db->bind(':package_words', $data['package_words']);
            $this->db->bind(':accept_custom_branding', $data['accept_custom_branding']);
            $this->db->bind(':is_private', $data['is_private']);
            $this->db->bind(':package_cost', $data['package_cost']);
            $this->db->bind(':package_monthly_cost', $data['package_monthly_cost']);
            $this->db->bind(':package_yearly_cost', $data['package_yearly_cost']);
            $this->db->bind(':package_lifetime_cost', $data['package_lifetime_cost']);
            $this->db->bind(':package_id', $data['package_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePackageFeatured($data)
        {
            $this->db->query('UPDATE packages SET package_featured = :package_featured WHERE package_id = :package_id');
// Bind values
            $this->db->bind(':package_featured', $data['package_featured']);
            $this->db->bind(':package_id', $data['package_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPackageById($id)
        {
            $this->db->query('SELECT * FROM packages
WHERE package_id = :package_id');
            $this->db->bind(':package_id', $id);
            return $this->db->single();
        }
        
        public function getFreeTrialPackage()
        {
            $this->db->query('SELECT * FROM packages
WHERE is_free_trial = :is_free_trial');
            $this->db->bind(':is_free_trial', 1);
            return $this->db->single();
        }
        
        public function deletePackage($id)
        {
            $this->db->query('DELETE FROM packages WHERE package_id = :package_id');
// Bind values
            $this->db->bind(':package_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPackageFeatures()
        {
            $this->db->query("SELECT * FROM package_features ORDER BY package_feature_id Asc");
            return $this->db->resultSet();
        }
        
        public function addPackageFeature($data)
        {
            $this->db->query('INSERT INTO package_features (package_feature_description) VALUES(:package_feature_description)');
// Bind values
            $this->db->bind(':package_feature_description', $data['package_feature_description']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePackageFeature($data)
        {
            $this->db->query('UPDATE package_features SET package_feature_description = :package_feature_description WHERE package_feature_id = :package_feature_id');
// Bind values
            $this->db->bind(':package_feature_description', $data['package_feature_description']);
            $this->db->bind(':package_feature_id', $data['package_feature_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPackageFeatureById($id)
        {
            $this->db->query('SELECT * FROM package_features
WHERE package_feature_id = :package_feature_id');
            $this->db->bind(':package_feature_id', $id);
            return $this->db->single();
        }
        
        public function deletePackageFeature($id)
        {
            $this->db->query('DELETE FROM package_features WHERE package_feature_id=:package_feature_id');
// Bind values
            $this->db->bind(':package_feature_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPackageItems($package_id)
        {
            $this->db->query("SELECT * FROM package_items WHERE package_id = :package_id ORDER BY package_item_id Asc");
            $this->db->bind(':package_id', $package_id);
            return $this->db->resultSet();
        }
        
        public function addPackageItem($data)
        {
            $this->db->query('INSERT INTO package_items (package_id, package_feature_id) VALUES(:package_id, :package_feature_id)');
// Bind values
            $this->db->bind(':package_id', $data['package_id']);
            $this->db->bind(':package_feature_id', $data['package_feature_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function deletePackageItem($data)
        {
            $this->db->query('DELETE FROM package_items WHERE package_id=:package_id AND package_feature_id=:package_feature_id');
// Bind values
            $this->db->bind(':package_id', $data['package_id']);
            $this->db->bind(':package_feature_id', $data['package_feature_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getDurationTypes()
        {
            $this->db->query("SELECT * FROM duration_types");
            return $this->db->resultSet();
        }
        
        public function getPrivatePackage($package_id, $user_id)
        {
            $this->db->query('SELECT * FROM private_packages
WHERE package_id = :package_id AND user_id  = :user_id');
            $this->db->bind(':package_id', $package_id);
            $this->db->bind(':user_id', $user_id);
            return $this->db->single();
        }
        
        public function getPrivatePackageUsers($package_id)
        {
            $this->db->query('SELECT * FROM private_packages
WHERE package_id = :package_id');
            $this->db->bind(':package_id', $package_id);
            return $this->db->resultSet();
        }
        
        public function addPrivatePackageUser($data)
        {
            $this->db->query('INSERT INTO private_packages (package_id, user_id) VALUES(:package_id, :user_id)');
// Bind values
            $this->db->bind(':package_id', $data['package_id']);
            $this->db->bind(':user_id', $data['user_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function deletePrivatePackageUser($data)
        {
            $this->db->query('DELETE FROM private_packages WHERE package_id=:package_id AND user_id=:user_id');
// Bind values
            $this->db->bind(':package_id', $data['package_id']);
            $this->db->bind(':user_id', $data['user_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
    }