<?php
    
    class Payment
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function addPaymentOption($data)
        {
            $this->db->query('INSERT INTO payment_options (payment_option, payment_option_name, payment_option_description, payment_option_online, payment_option_added_by, payment_option_date_added) VALUES(:payment_option, :payment_option_name, :payment_option_description, :payment_option_online, :payment_option_added_by, :payment_option_date_added)');
// Bind values
            $this->db->bind(':payment_option', $data['payment_option']);
            $this->db->bind(':payment_option_name', $data['payment_option_name']);
            $this->db->bind(':payment_option_description', $data['payment_option_description']);
            $this->db->bind(':payment_option_online', $data['payment_option_online']);
            $this->db->bind(':payment_option_added_by', $data['payment_option_added_by']);
            $this->db->bind(':payment_option_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updatePaymentOption($data)
        {
            $this->db->query('UPDATE payment_options SET payment_option_name = :payment_option_name, payment_option_description = :payment_option_description WHERE payment_option_id =:payment_option_id');
// Bind values
            $this->db->bind(':payment_option_name', $data['payment_option_name']);
            $this->db->bind(':payment_option_description', $data['payment_option_description']);
            $this->db->bind(':payment_option_id', $data['payment_option_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePaymentOptionCurrencies($data)
        {
            $this->db->query('UPDATE payment_options SET payment_option_currencies = :payment_option_currencies, payment_option_zero_decimal_currencies = :payment_option_zero_decimal_currencies WHERE payment_option_id =:payment_option_id');
// Bind values
            $this->db->bind(':payment_option_currencies', $data['payment_option_currencies']);
            $this->db->bind(':payment_option_zero_decimal_currencies', $data['payment_option_zero_decimal_currencies']);
            $this->db->bind(':payment_option_id', $data['payment_option_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaymentOptions($payment_option_online = NULL)
        {
            $query = 'SELECT * FROM payment_options';
            if (isset($payment_option_online)) {
                $query .= ' WHERE payment_option_online = :payment_option_online ';
            }
            $query .= ' ORDER BY payment_option_id Desc';
            $this->db->query($query);
            if (isset($payment_option_online)) {
                $this->db->bind(':payment_option_online', $payment_option_online);
            }
            return $this->db->resultSet();
        }
        
        public function getCompanyPaymentOptions($company_id)
        {
            $query = 'SELECT * FROM payment_options';
            $query .= ' WHERE company_id = :company_id ';
            $query .= ' ORDER BY payment_option_id Desc';
            $this->db->query($query);
            $this->db->bind(':company_id', $company_id);
            return $this->db->resultSet();
        }
        
        public function getPaymentOptionById($payment_option_id)
        {
            $this->db->query('SELECT * FROM payment_options WHERE payment_option_id = :payment_option_id');
// Bind value
            $this->db->bind(':payment_option_id', $payment_option_id);
            return $this->db->single();
        }
        
        public function getPaymentOptionBySlug($payment_option)
        {
            $this->db->query('SELECT * FROM payment_options WHERE payment_option = :payment_option');
// Bind value
            $this->db->bind(':payment_option', $payment_option);
            return $this->db->single();
        }
        
        public function findPaymentOptionByName($payment_option_name)
        {
            $this->db->query('SELECT * FROM payment_options WHERE payment_option_name = :payment_option_name');
// Bind value
            $this->db->bind(':payment_option_name', $payment_option_name);
            return $this->db->single();
        }
        
        public function deletePaymentOption($id)
        {
            $this->db->query('DELETE from payment_options WHERE payment_option_id = :payment_option_id');
// Bind values
            $this->db->bind(':payment_option_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
    }