<?php
    
    class Payout
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getPayouts($added_by = NULL)
        {
            $query = "SELECT * FROM payouts ";
            if (isset($added_by)) {
                $query .= " WHERE user_id = :user_id";
            }
            $query .= " ORDER BY payout_id Desc";
            $this->db->query($query);
            if (isset($added_by)) {
// Bind values
                $this->db->bind(':user_id', $added_by);
            }
            return $this->db->resultSet();
        }
        
        public function getPayoutsTotal($payout_status_id, $from_date = NULL, $to_date = NULL)
        {
            $query = 'SELECT SUM(payout_amount) as payout_amount FROM payouts ';
            $query .= ' WHERE payout_status_id = :payout_status_id ';
            if (isset($from_date) && isset($to_date)) {
                $query .= ' AND payout_date_added BETWEEN :from_date AND :to_date';
            }
            $query .= ' ORDER BY payout_id Desc';
            $this->db->query($query);
// Bind value
            $this->db->bind(':payout_status_id', $payout_status_id);
            if (isset($from_date) && isset($to_date)) {
                $this->db->bind(':from_date', $from_date);
                $this->db->bind(':to_date', $to_date);
            }
            return $this->db->single();
        }
        
        public function getUserPayoutsTotal($user_id, $payout_status_id = NULL)
        {
            $query = 'SELECT SUM(payout_amount) as payout_amount
FROM payouts 
WHERE user_id = :user_id ';
            if (isset($payout_status_id)) {
                $query .= ' AND payout_status_id = :payout_status_id ';
            }
            $query .= ' ORDER BY payout_id Desc';
            $this->db->query($query);
// Bind value
            $this->db->bind(':user_id', $user_id);
            if (isset($payout_status_id)) {
                $this->db->bind(':payout_status_id', $payout_status_id);
            }
            return $this->db->single();
            
        }
        
        public function getPayoutsByStatus($payout_status_id, $added_by = NULL)
        {
            $query = "SELECT * FROM payouts
WHERE payout_status_id = :payout_status_id";
            if (isset($added_by)) {
                $query .= " AND user_id = :user_id";
            }
            $query .= " ORDER BY payout_id Desc";
            $this->db->query($query);
            if (isset($added_by)) {
// Bind values
                $this->db->bind(':user_id', $added_by);
            }
            $this->db->bind(':payout_status_id', $payout_status_id);
            return $this->db->resultSet();
        }
        
        public function requestPayout($data)
        {
            $this->db->query('INSERT INTO payouts (payout_uid, payout_name, user_id, payout_mode_id, payout_address, payout_amount, payout_date_added) VALUES(:payout_uid, :payout_name, :user_id, :payout_mode_id, :payout_address, :payout_amount, :payout_date_added)');
// Bind values
            $this->db->bind(':payout_uid', uid());
            $this->db->bind(':payout_name', $data['payout_name']);
            $this->db->bind(':user_id', $data['user_id']);
            $this->db->bind(':payout_mode_id', $data['payout_mode_id']);
            $this->db->bind(':payout_address', $data['payout_address']);
            $this->db->bind(':payout_amount', $data['payout_amount']);
            $this->db->bind(':payout_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function payRequestedPayout($data)
        {
            $this->db->query('UPDATE payouts SET payout_transaction_id=:payout_transaction_id, payout_note = :payout_note, payout_status_id = :payout_status_id, payout_date_paid = :payout_date_paid WHERE payout_id=:payout_id');
// Bind values
            $this->db->bind(':payout_transaction_id', $data['payout_transaction_id']);
            $this->db->bind(':payout_note', $data['payout_note']);
            $this->db->bind(':payout_status_id', $data['payout_status_id']);
            $this->db->bind(':payout_date_paid', now());
            $this->db->bind(':payout_id', $data['payout_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePayoutStatus($data)
        {
            $this->db->query('UPDATE payouts SET payout_status_id = :payout_status_id WHERE payout_id=:payout_id');
// Bind values
            $this->db->bind(':payout_status_id', $data['payout_status_id']);
            $this->db->bind(':payout_id', $data['payout_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPayoutById($id)
        {
            $this->db->query('SELECT * FROM payouts
WHERE payout_id = :payout_id');
            $this->db->bind(':payout_id', $id);
            return $this->db->single();
        }
        
        public function getPayoutByTransactionId($transaction_id)
        {
            $this->db->query('SELECT * FROM payouts
WHERE payout_transaction_id = :payout_transaction_id');
            $this->db->bind(':payout_transaction_id', $transaction_id);
            return $this->db->single();
        }
        
        public function nextPayoutId()
        {
            $this->db->query("SHOW TABLE STATUS LIKE 'payouts'");
            $row = $this->db->single();
            return $row->Auto_increment;
        }
        
        public function addPayoutMode($data)
        {
            $this->db->query('INSERT INTO payout_modes (payout_mode_name, payout_mode_minimum) VALUES(:payout_mode_name, :payout_mode_minimum)');
// Bind values
            $this->db->bind(':payout_mode_name', $data['payout_mode_name']);
            $this->db->bind(':payout_mode_minimum', $data['payout_mode_minimum']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePayoutMode($data)
        {
            $this->db->query('UPDATE payout_modes SET payout_mode_name = :payout_mode_name, payout_mode_minimum = :payout_mode_minimum WHERE payout_mode_id=:payout_mode_id');
// Bind values
            $this->db->bind(':payout_mode_name', $data['payout_mode_name']);
            $this->db->bind(':payout_mode_minimum', $data['payout_mode_minimum']);
            $this->db->bind(':payout_mode_id', $data['payout_mode_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updatePayoutModeStatus($data)
        {
            $this->db->query('UPDATE payout_modes SET payout_mode_status = :payout_mode_status WHERE payout_mode_id=:payout_mode_id');
// Bind values
            $this->db->bind(':payout_mode_status', $data['payout_mode_status']);
            $this->db->bind(':payout_mode_id', $data['payout_mode_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPayoutModes($payout_mode_status = NULL)
        {
            $query = "SELECT * FROM payout_modes ";
            if (isset($payout_mode_status)) {
                $query .= " WHERE payout_mode_status = :payout_mode_status";
            }
            $query .= " ORDER BY payout_mode_id Desc";
            $this->db->query($query);
            if (isset($payout_mode_status)) {
// Bind values
                $this->db->bind(':payout_mode_status', $payout_mode_status);
            }
            return $this->db->resultSet();
        }
        
        public function getPayoutModeById($id)
        {
            $this->db->query('SELECT *
FROM payout_modes 
WHERE payout_mode_id = :payout_mode_id');
// Bind value
            $this->db->bind(':payout_mode_id', $id);
            return $this->db->single();
        }
        
        public function getPayoutModeByName($payout_mode)
        {
            $this->db->query('SELECT *
FROM payout_modes 
WHERE payout_mode_name = :payout_mode_name');
// Bind value
            $this->db->bind(':payout_mode_name', $payout_mode);
            return $this->db->single();
        }
        
        public function deletePayoutMode($id)
        {
            $this->db->query('DELETE FROM payout_modes WHERE payout_mode_id = :payout_mode_id');
// Bind values
            $this->db->bind(':payout_mode_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPayoutStatus()
        {
            $this->db->query('SELECT * FROM payout_status');
            return $this->db->resultSet();
        }
        
        public function getPayoutStatusById($id)
        {
            $this->db->query('SELECT * FROM payout_status WHERE payout_status_id = :payout_status_id');
            $this->db->bind(':payout_status_id', $id);
            return $this->db->single();
        }
        
        
    }