<?php
    
    class Post
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getPosts($post_status_id = NULL)
        {
            $query = "SELECT * FROM posts ";
            if (isset($post_status_id)) {
                $query .= " WHERE post_status_id = :post_status_id ";
            }
            $query .= " ORDER BY post_id Desc";
            $this->db->query($query);
            if (isset($post_status_id)) {
                $this->db->bind(':post_status_id', $post_status_id);
            }
            return $this->db->resultSet();
        }
        
        public function addPost($data)
        {
            $this->db->query('INSERT INTO posts (post_added_by, post_title, post_slug, post_category_id, post_featured_image, post_description, post_status_id, post_date_added) VALUES(:post_added_by, :post_title, :post_slug, :post_category_id, :post_featured_image, :post_description, :post_status_id, :post_date_added)');
// Bind values
            $this->db->bind(':post_added_by', user_id());
            $this->db->bind(':post_title', $data['post_title']);
            $this->db->bind(':post_slug', generate_seo_url($data['post_title']));
            $this->db->bind(':post_category_id', $data['post_category_id']);
            $this->db->bind(':post_featured_image', $data['post_featured_image']);
            $this->db->bind(':post_description', $data['post_description']);
            $this->db->bind(':post_status_id', $data['post_status_id']);
            $this->db->bind(':post_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updatePost($data)
        {
            $this->db->query('UPDATE posts SET post_title=:post_title, post_category_id=:post_category_id, post_description=:post_description, post_status_id=:post_status_id WHERE post_id=:post_id');
// Bind values
            $this->db->bind(':post_title', $data['post_title']);
            $this->db->bind(':post_category_id', $data['post_category_id']);
            $this->db->bind(':post_description', $data['post_description']);
            $this->db->bind(':post_status_id', $data['post_status_id']);
            $this->db->bind(':post_id', $data['post_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateFeaturedImage($data)
        {
            $this->db->query('UPDATE posts SET post_featured_image=:post_featured_image WHERE post_id=:post_id');
// Bind values
            $this->db->bind(':post_featured_image', $data['post_featured_image']);
            $this->db->bind(':post_id', $data['post_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPostById($id)
        {
            $this->db->query('SELECT * FROM posts WHERE post_id = :post_id');
            $this->db->bind(':post_id', $id);
            return $this->db->single();
        }
        
        public function deletePost($id)
        {
            $this->db->query('DELETE FROM posts WHERE post_id = :post_id');
            $this->db->bind(':post_id', $id);
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getNextPostById($id)
        {
            $this->db->query('SELECT * FROM posts WHERE post_id = (select min(post_id) from posts WHERE post_id > :post_id)');
            $this->db->bind(':post_id', $id);
            return $this->db->single();
        }
        
        public function getPreviousPostById($id)
        {
            $this->db->query('SELECT * FROM posts WHERE post_id = (select max(post_id) from posts WHERE post_id < :post_id)');
            $this->db->bind(':post_id', $id);
            return $this->db->single();
        }
        
        public function getPostBySlug($post_slug)
        {
            $this->db->query('SELECT * FROM posts WHERE post_slug = :post_slug');
            $this->db->bind(':post_slug', $post_slug);
            return $this->db->single();
        }
        
        public function addPostCategory($data)
        {
            $this->db->query('INSERT INTO post_categories (post_category_added_by, post_category_name, post_category_slug, post_category_status_id, post_category_date_added) VALUES(:post_category_added_by, :post_category_name, :post_category_slug, :post_category_status_id, :post_category_date_added)');
// Bind values
            $this->db->bind(':post_category_added_by', user_id());
            $this->db->bind(':post_category_name', $data['post_category_name']);
            $this->db->bind(':post_category_slug', generate_seo_url($data['post_category_name']));
            $this->db->bind(':post_category_status_id', $data['post_category_status_id']);
            $this->db->bind(':post_category_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function updatePostCategory($data)
        {
            $this->db->query('UPDATE post_categories SET post_category_name=:post_category_name, post_category_status_id=:post_category_status_id WHERE post_category_id=:post_category_id');
// Bind values
            $this->db->bind(':post_category_name', $data['post_category_name']);
            $this->db->bind(':post_category_status_id', $data['post_category_status_id']);
            $this->db->bind(':post_category_id', $data['post_category_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPostCategories()
        {
            $this->db->query("SELECT * FROM post_categories ORDER BY post_category_id Desc");
            return $this->db->resultSet();
        }
        
        public function getPostCategoryById($id)
        {
            $this->db->query('SELECT * FROM post_categories WHERE post_category_id = :post_category_id');
            $this->db->bind(':post_category_id', $id);
            return $this->db->single();
        }
        
        public function getPostCategoryBySlug($post_category_slug)
        {
            $this->db->query('SELECT * FROM post_categories WHERE post_category_slug = :post_category_slug');
            $this->db->bind(':post_category_slug', $post_category_slug);
            return $this->db->single();
        }
        
        public function deletePostCategory($id)
        {
            $this->db->query('DELETE FROM post_categories WHERE post_category_id = :post_category_id');
            $this->db->bind(':post_category_id', $id);
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPostComments($post_id)
        {
            $this->db->query("SELECT * FROM post_comments WHERE post_id = :post_id AND post_comment_parent_id IS NULL ORDER BY post_comment_id Asc");
            $this->db->bind(':post_id', $post_id);
            return $this->db->resultSet();
        }
        
        public function getPostCommentById($id)
        {
            $this->db->query('SELECT * FROM post_comments WHERE post_comment_id = :post_comment_id');
            $this->db->bind(':post_comment_id', $id);
            return $this->db->single();
        }
        
        public function getPostCommentReplies($comment_id)
        {
            $this->db->query('SELECT * FROM post_comments WHERE post_comment_parent_id = :post_comment_parent_id');
            $this->db->bind(':post_comment_parent_id', $comment_id);
            return $this->db->resultSet();
        }
        
        public function addComment($data)
        {
            $this->db->query('INSERT INTO post_comments (post_comment_added_by, post_id, post_comment_parent_id, post_comment, post_comment_status_id, post_comment_date_added) VALUES(:post_comment_added_by, :post_id, :post_comment_parent_id, :post_comment, :post_comment_status_id, :post_comment_date_added)');
// Bind values
            $this->db->bind(':post_comment_added_by', $data['post_comment_added_by']);
            $this->db->bind(':post_id', $data['post_id']);
            $this->db->bind(':post_comment_parent_id', $data['post_comment_parent_id']);
            $this->db->bind(':post_comment', $data['post_comment']);
            $this->db->bind(':post_comment_status_id', $data['post_comment_status_id']);
            $this->db->bind(':post_comment_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getPostsByCategoryId($post_category_id, $post_status_id = NULL)
        {
            $query = "SELECT * FROM posts WHERE post_category_id = :post_category_id ";
            if (isset($post_status_id)) {
                $query .= " AND post_status_id = :post_status_id ";
            }
            $query .= " ORDER BY post_id Desc";
            $this->db->query($query);
            $this->db->bind(':post_category_id', $post_category_id);
            if (isset($post_status_id)) {
                $this->db->bind(':post_status_id', $post_status_id);
            }
            return $this->db->resultSet();
        }
        
        
    }