<?php
    
    class Setting
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getOptions($option_name)
        {
            $this->db->query('SELECT * FROM options WHERE option_name = :option_name');
            $this->db->bind(':option_name', $option_name);
            return $this->db->single();
        }
        
        public function updateOptions($data)
        {
            $this->db->query("UPDATE options SET option_value = :option_value WHERE option_name = :option_name");
// Bind values
            $this->db->bind(':option_value', $data['option_value']);
            $this->db->bind(':option_name', $data['option_name']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getItemStatuses()
        {
            $this->db->query("SELECT * FROM item_status ORDER BY item_status_id Desc");
            return $this->db->resultSet();
        }
        
        public function getItemStatusById($id)
        {
            $this->db->query('SELECT * FROM item_status WHERE item_status_id = :item_status_id');
            $this->db->bind(':item_status_id', $id);
            return $this->db->single();
        }
        
        public function getNavigationByName($navigation_name)
        {
            $this->db->query('SELECT * FROM navigation WHERE navigation_name = :navigation_name');
            $this->db->bind(':navigation_name', $navigation_name);
            return $this->db->single();
        }
        
        public function updateOptionsAutoload($data)
        {
            $this->db->query("UPDATE options SET option_autoload = :option_autoload WHERE option_name = :option_name");
// Bind values
            $this->db->bind(':option_autoload', $data['option_autoload']);
            $this->db->bind(':option_name', $data['option_name']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaypalStandardSettings($status)
        {
            $this->db->query('SELECT * FROM paypal_standard_settings WHERE paypal_standard_status = :paypal_standard_status');
            $this->db->bind(':paypal_standard_status', $status);
            return $this->db->single();
        }
        
        public function updatePaypalStandardSettings($data)
        {
            $this->db->query("UPDATE paypal_standard_settings SET paypal_standard_email = :paypal_standard_email WHERE paypal_standard_status = :paypal_standard_status");
// Bind values
            $this->db->bind(':paypal_standard_email', $data['paypal_standard_email']);
            $this->db->bind(':paypal_standard_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaypalProSettings($status)
        {
            $this->db->query('SELECT * FROM paypal_pro_settings WHERE paypal_pro_status = :paypal_pro_status');
            $this->db->bind(':paypal_pro_status', $status);
            return $this->db->single();
        }
        
        public function updatePaypalProSettings($data)
        {
            $this->db->query("UPDATE paypal_pro_settings SET paypal_pro_username = :paypal_pro_username, paypal_pro_password = :paypal_pro_password, paypal_pro_signature = :paypal_pro_signature WHERE paypal_pro_status = :paypal_pro_status");
// Bind values
            $this->db->bind(':paypal_pro_username', $data['paypal_pro_username']);
            $this->db->bind(':paypal_pro_password', $data['paypal_pro_password']);
            $this->db->bind(':paypal_pro_signature', $data['paypal_pro_signature']);
            $this->db->bind(':paypal_pro_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getStripeSettings($status)
        {
            $this->db->query('SELECT * FROM stripe_settings WHERE stripe_status = :stripe_status');
            $this->db->bind(':stripe_status', $status);
            return $this->db->single();
        }
        
        public function updateStripeSettings($data)
        {
            $this->db->query("UPDATE stripe_settings SET stripe_secret_key = :stripe_secret_key, stripe_publishable_key = :stripe_publishable_key WHERE stripe_status = :stripe_status");
// Bind values
            $this->db->bind(':stripe_secret_key', $data['stripe_secret_key']);
            $this->db->bind(':stripe_publishable_key', $data['stripe_publishable_key']);
            $this->db->bind(':stripe_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getAuthorizeNetSettings($status)
        {
            $this->db->query('SELECT * FROM authorize_net_settings WHERE authorize_net_status = :authorize_net_status');
            $this->db->bind(':authorize_net_status', $status);
            return $this->db->single();
        }
        
        public function updateAuthorizeNetSettings($data)
        {
            $this->db->query("UPDATE authorize_net_settings SET authorize_net_login_id = :authorize_net_login_id, authorize_net_transaction_key = :authorize_net_transaction_key WHERE authorize_net_status = :authorize_net_status");
// Bind values
            $this->db->bind(':authorize_net_login_id', $data['authorize_net_login_id']);
            $this->db->bind(':authorize_net_transaction_key', $data['authorize_net_transaction_key']);
            $this->db->bind(':authorize_net_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPesapalSettings($status)
        {
            $this->db->query('SELECT * FROM pesapal_settings WHERE pesapal_status = :pesapal_status');
            $this->db->bind(':pesapal_status', $status);
            return $this->db->single();
        }
        
        public function updatePesapalSettings($data)
        {
            $this->db->query("UPDATE pesapal_settings SET pesapal_consumer_key = :pesapal_consumer_key, pesapal_consumer_secret = :pesapal_consumer_secret, pesapal_notification_id = :pesapal_notification_id WHERE pesapal_status = :pesapal_status");
// Bind values
            $this->db->bind(':pesapal_consumer_key', $data['pesapal_consumer_key']);
            $this->db->bind(':pesapal_consumer_secret', $data['pesapal_consumer_secret']);
            $this->db->bind(':pesapal_notification_id', $data['pesapal_notification_id']);
            $this->db->bind(':pesapal_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getFlutterwaveSettings($status)
        {
            $this->db->query('SELECT * FROM flutterwave_settings WHERE flutterwave_status = :flutterwave_status');
            $this->db->bind(':flutterwave_status', $status);
            return $this->db->single();
        }
        
        public function updateFlutterwaveSettings($data)
        {
            $this->db->query("UPDATE flutterwave_settings SET flutterwave_public_key = :flutterwave_public_key, flutterwave_secret_key = :flutterwave_secret_key, flutterwave_encryption_key = :flutterwave_encryption_key WHERE flutterwave_status = :flutterwave_status");
// Bind values
            $this->db->bind(':flutterwave_public_key', $data['flutterwave_public_key']);
            $this->db->bind(':flutterwave_secret_key', $data['flutterwave_secret_key']);
            $this->db->bind(':flutterwave_encryption_key', $data['flutterwave_encryption_key']);
            $this->db->bind(':flutterwave_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getSslcommerzSettings($status)
        {
            $this->db->query('SELECT * FROM sslcommerz_settings WHERE sslcommerz_status = :sslcommerz_status');
            $this->db->bind(':sslcommerz_status', $status);
            return $this->db->single();
        }
        
        public function updateSslcommerzSettings($data)
        {
            $this->db->query("UPDATE sslcommerz_settings SET sslcommerz_store_id = :sslcommerz_store_id, sslcommerz_password = :sslcommerz_password WHERE sslcommerz_status = :sslcommerz_status");
// Bind values
            $this->db->bind(':sslcommerz_store_id', $data['sslcommerz_store_id']);
            $this->db->bind(':sslcommerz_password', $data['sslcommerz_password']);
            $this->db->bind(':sslcommerz_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getEsewaSettings($status)
        {
            $this->db->query('SELECT * FROM esewa_settings WHERE esewa_status = :esewa_status');
            $this->db->bind(':esewa_status', $status);
            return $this->db->single();
        }
        
        public function updateEsewaSettings($data)
        {
            $this->db->query("UPDATE esewa_settings SET esewa_merchant_code = :esewa_merchant_code WHERE esewa_status = :esewa_status");
// Bind values
            $this->db->bind(':esewa_merchant_code', $data['esewa_merchant_code']);
            $this->db->bind(':esewa_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getInstamojoSettings($status)
        {
            $this->db->query('SELECT * FROM instamojo_settings WHERE instamojo_status = :instamojo_status');
            $this->db->bind(':instamojo_status', $status);
            return $this->db->single();
        }
        
        public function updateInstamojoSettings($data)
        {
            $this->db->query("UPDATE instamojo_settings SET instamojo_api_key = :instamojo_api_key, instamojo_auth_token = :instamojo_auth_token WHERE instamojo_status = :instamojo_status");
// Bind values
            $this->db->bind(':instamojo_api_key', $data['instamojo_api_key']);
            $this->db->bind(':instamojo_auth_token', $data['instamojo_auth_token']);
            $this->db->bind(':instamojo_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaystackSettings($status)
        {
            $this->db->query('SELECT * FROM paystack_settings WHERE paystack_status = :paystack_status');
            $this->db->bind(':paystack_status', $status);
            return $this->db->single();
        }
        
        public function updatePaystackSettings($data)
        {
            $this->db->query("UPDATE paystack_settings SET paystack_secret_key = :paystack_secret_key, paystack_public_key = :paystack_public_key WHERE paystack_status = :paystack_status");
// Bind values
            $this->db->bind(':paystack_secret_key', $data['paystack_secret_key']);
            $this->db->bind(':paystack_public_key', $data['paystack_public_key']);
            $this->db->bind(':paystack_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getAamarpaySettings($status)
        {
            $this->db->query('SELECT * FROM aamarpay_settings WHERE aamarpay_status = :aamarpay_status');
            $this->db->bind(':aamarpay_status', $status);
            return $this->db->single();
        }
        
        public function updateAamarpaySettings($data)
        {
            $this->db->query("UPDATE aamarpay_settings SET aamarpay_store_id = :aamarpay_store_id, aamarpay_signature_key = :aamarpay_signature_key WHERE aamarpay_status = :aamarpay_status");
// Bind values
            $this->db->bind(':aamarpay_store_id', $data['aamarpay_store_id']);
            $this->db->bind(':aamarpay_signature_key', $data['aamarpay_signature_key']);
            $this->db->bind(':aamarpay_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getToyyibpaySettings($status)
        {
            $this->db->query('SELECT * FROM toyyibpay_settings WHERE toyyibpay_status = :toyyibpay_status');
            $this->db->bind(':toyyibpay_status', $status);
            return $this->db->single();
        }
        
        public function updateToyyibpaySettings($data)
        {
            $this->db->query("UPDATE toyyibpay_settings SET toyyibpay_secret_key = :toyyibpay_secret_key, toyyibpay_category = :toyyibpay_category WHERE toyyibpay_status = :toyyibpay_status");
// Bind values
            $this->db->bind(':toyyibpay_secret_key', $data['toyyibpay_secret_key']);
            $this->db->bind(':toyyibpay_category', $data['toyyibpay_category']);
            $this->db->bind(':toyyibpay_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getHyperpaySettings($status)
        {
            $this->db->query('SELECT * FROM hyperpay_settings WHERE hyperpay_status = :hyperpay_status');
            $this->db->bind(':hyperpay_status', $status);
            return $this->db->single();
        }
        
        public function updateHyperpaySettings($data)
        {
            $this->db->query("UPDATE hyperpay_settings SET hyperpay_access_token = :hyperpay_access_token, hyperpay_entity_id = :hyperpay_entity_id, hyperpay_brands = :hyperpay_brands WHERE hyperpay_status = :hyperpay_status");
// Bind values
            $this->db->bind(':hyperpay_access_token', $data['hyperpay_access_token']);
            $this->db->bind(':hyperpay_entity_id', $data['hyperpay_entity_id']);
            $this->db->bind(':hyperpay_brands', $data['hyperpay_brands']);
            $this->db->bind(':hyperpay_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaygateSettings($status)
        {
            $this->db->query('SELECT * FROM paygate_settings WHERE paygate_status = :paygate_status');
            $this->db->bind(':paygate_status', $status);
            return $this->db->single();
        }
        
        public function updatePaygateSettings($data)
        {
            $this->db->query("UPDATE paygate_settings SET paygate_paygate_id = :paygate_paygate_id, paygate_encryption_key = :paygate_encryption_key WHERE paygate_status = :paygate_status");
// Bind values
            $this->db->bind(':paygate_paygate_id', $data['paygate_paygate_id']);
            $this->db->bind(':paygate_encryption_key', $data['paygate_encryption_key']);
            $this->db->bind(':paygate_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPaddleSettings($status)
        {
            $this->db->query('SELECT * FROM paddle_settings WHERE paddle_status = :paddle_status');
            $this->db->bind(':paddle_status', $status);
            return $this->db->single();
        }
        
        public function updatePaddleSettings($data)
        {
            $this->db->query("UPDATE paddle_settings SET paddle_api_key = :paddle_api_key, paddle_client_side_token = :paddle_client_side_token WHERE paddle_status = :paddle_status");
// Bind values
            $this->db->bind(':paddle_api_key', $data['paddle_api_key']);
            $this->db->bind(':paddle_client_side_token', $data['paddle_client_side_token']);
            $this->db->bind(':paddle_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getPayTRSettings($status)
        {
            $this->db->query('SELECT * FROM paytr_settings WHERE paytr_status = :paytr_status');
            $this->db->bind(':paytr_status', $status);
            return $this->db->single();
        }
        
        public function updatePayTRSettings($data)
        {
            $this->db->query("UPDATE paytr_settings SET paytr_merchant_id = :paytr_merchant_id, paytr_merchant_key = :paytr_merchant_key, paytr_merchant_salt = :paytr_merchant_salt WHERE paytr_status = :paytr_status");
// Bind values
            $this->db->bind(':paytr_merchant_id', $data['paytr_merchant_id']);
            $this->db->bind(':paytr_merchant_key', $data['paytr_merchant_key']);
            $this->db->bind(':paytr_merchant_salt', $data['paytr_merchant_salt']);
            $this->db->bind(':paytr_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getIyzicoSettings($status)
        {
            $this->db->query('SELECT * FROM iyzico_settings WHERE iyzico_status = :iyzico_status');
            $this->db->bind(':iyzico_status', $status);
            return $this->db->single();
        }
        
        public function updateIyzicoSettings($data)
        {
            $this->db->query("UPDATE iyzico_settings SET iyzico_api_key = :iyzico_api_key, iyzico_secret_key = :iyzico_secret_key WHERE iyzico_status = :iyzico_status");
// Bind values
            $this->db->bind(':iyzico_api_key', $data['iyzico_api_key']);
            $this->db->bind(':iyzico_secret_key', $data['iyzico_secret_key']);
            $this->db->bind(':iyzico_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getTapSettings($status)
        {
            $this->db->query('SELECT * FROM tap_settings WHERE tap_status = :tap_status');
            $this->db->bind(':tap_status', $status);
            return $this->db->single();
        }
        
        public function updateTapSettings($data)
        {
            $this->db->query("UPDATE tap_settings SET tap_secret_key = :tap_secret_key, tap_publishable_key = :tap_publishable_key, tap_encryption_key = :tap_encryption_key WHERE tap_status = :tap_status");
// Bind values
            $this->db->bind(':tap_secret_key', $data['tap_secret_key']);
            $this->db->bind(':tap_publishable_key', $data['tap_publishable_key']);
            $this->db->bind(':tap_encryption_key', $data['tap_encryption_key']);
            $this->db->bind(':tap_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }

		public function getTwoCheckoutSettings($status){
			$this->db->query('SELECT * FROM twocheckout_settings WHERE twocheckout_status = :twocheckout_status');
			$this->db->bind(':twocheckout_status', $status);
			$row = $this->db->single();
			return $row;
		}
		
		public function updateTwoCheckoutSettings($data){
			$this->db->query("UPDATE twocheckout_settings SET twocheckout_publishable_key = :twocheckout_publishable_key, twocheckout_private_key = :twocheckout_private_key, twocheckout_secret_key = :twocheckout_secret_key, twocheckout_merchant_code = :twocheckout_merchant_code WHERE twocheckout_status = :twocheckout_status");
			// Bind values
			$this->db->bind(':twocheckout_publishable_key', $data['twocheckout_publishable_key']);
			$this->db->bind(':twocheckout_private_key', $data['twocheckout_private_key']);
			$this->db->bind(':twocheckout_secret_key', $data['twocheckout_secret_key']);
			$this->db->bind(':twocheckout_merchant_code', $data['twocheckout_merchant_code']);
			$this->db->bind(':twocheckout_status', $data['option_value']);
			// Execute
			if($this->db->execute()){
				return true;
			} else{
				return false;
			}
		}
        
        public function getMidtransSettings($status)
        {
            $this->db->query('SELECT * FROM midtrans_settings WHERE midtrans_status = :midtrans_status');
            $this->db->bind(':midtrans_status', $status);
            $row = $this->db->single();
            return $row;
        }
        
        public function updateMidtransSettings($data)
        {
            $this->db->query("UPDATE midtrans_settings SET midtrans_client_id = :midtrans_client_id, midtrans_server_key = :midtrans_server_key WHERE midtrans_status = :midtrans_status");
// Bind values
            $this->db->bind(':midtrans_client_id', $data['midtrans_client_id']);
            $this->db->bind(':midtrans_server_key', $data['midtrans_server_key']);
            $this->db->bind(':midtrans_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getTripaySettings($status)
        {
            $this->db->query('SELECT * FROM tripay_settings WHERE tripay_status = :tripay_status');
            $this->db->bind(':tripay_status', $status);
            $row = $this->db->single();
            return $row;
        }
        
        public function updateTripaySettings($data)
        {
            $this->db->query("UPDATE tripay_settings SET tripay_private_key = :tripay_private_key, tripay_api_key = :tripay_api_key, tripay_merchant_code = :tripay_merchant_code, tripay_payment_method = :tripay_payment_method WHERE tripay_status = :tripay_status");
// Bind values
            $this->db->bind(':tripay_private_key', $data['tripay_private_key']);
            $this->db->bind(':tripay_api_key', $data['tripay_api_key']);
            $this->db->bind(':tripay_merchant_code', $data['tripay_merchant_code']);
            $this->db->bind(':tripay_payment_method', $data['tripay_payment_method']);
            $this->db->bind(':tripay_status', $data['option_value']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getSMTPSettings($status)
        {
            $this->db->query('SELECT * FROM smtp_settings WHERE smtp_id = :smtp_id');
            $this->db->bind(':smtp_id', 1);
            return $this->db->single();
        }
        
        public function updateSMTPSettings($data)
        {
            $this->db->query("UPDATE smtp_settings SET smtp_host = :smtp_host, smtp_encryption = :smtp_encryption, smtp_username = :smtp_username, smtp_password = :smtp_password, smtp_port = :smtp_port WHERE smtp_id = :smtp_id");
// Bind values
            $this->db->bind(':smtp_host', $data['smtp_host']);
            $this->db->bind(':smtp_encryption', $data['smtp_encryption']);
            $this->db->bind(':smtp_username', $data['smtp_username']);
            $this->db->bind(':smtp_password', $data['smtp_password']);
            $this->db->bind(':smtp_port', $data['smtp_port']);
            $this->db->bind(':smtp_id', 1);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getSendgridSettings($status)
        {
            $this->db->query('SELECT * FROM sendgrid_settings WHERE sendgrid_id = :sendgrid_id');
            $this->db->bind(':sendgrid_id', 1);
            return $this->db->single();
        }
        
        public function updateSendgridSettings($data)
        {
            $this->db->query("UPDATE sendgrid_settings SET sendgrid_api_key = :sendgrid_api_key WHERE sendgrid_id = :sendgrid_id");
// Bind values
            $this->db->bind(':sendgrid_api_key', $data['sendgrid_api_key']);
            $this->db->bind(':sendgrid_id', 1);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getGoogleRecaptchaSettings()
        {
            $this->db->query('SELECT * FROM google_recaptcha_settings WHERE google_recaptcha_id = :google_recaptcha_id');
            $this->db->bind(':google_recaptcha_id', 1);
            return $this->db->single();
        }
        
        public function updateGoogleRecaptchaSettings($data)
        {
            $this->db->query("UPDATE google_recaptcha_settings SET google_recaptcha_site_key = :google_recaptcha_site_key, google_recaptcha_secret_key = :google_recaptcha_secret_key WHERE google_recaptcha_id = :google_recaptcha_id");
// Bind values
            $this->db->bind(':google_recaptcha_site_key', $data['google_recaptcha_site_key']);
            $this->db->bind(':google_recaptcha_secret_key', $data['google_recaptcha_secret_key']);
            $this->db->bind(':google_recaptcha_id', 1);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getLanguages()
        {
            $this->db->query("SELECT * FROM languages");
            return $this->db->resultSet();
        }
        
        public function getActiveLanguages()
        {
            $this->db->query("SELECT * FROM languages WHERE is_active = :is_active");
            $this->db->bind(':is_active', 1);
            return $this->db->resultSet();
        }
        
        public function getLanguageById($id)
        {
            $this->db->query("SELECT * FROM languages WHERE language_id = :language_id");
            $this->db->bind(':language_id', $id);
            return $this->db->single();
        }
        
        public function getLanguageByIso($iso)
        {
            $this->db->query("SELECT * FROM languages WHERE language_iso = :language_iso");
            $this->db->bind(':language_iso', $iso);
            return $this->db->single();
        }
        
        public function getSupportedCurrencies()
        {
            $this->db->query("SELECT * FROM supported_currencies");
            return $this->db->resultSet();
        }
        
        public function getSupportedCurrencyByIso($iso)
        {
            $this->db->query('SELECT * FROM supported_currencies WHERE supported_currency_iso = :supported_currency_iso');
            $this->db->bind(':supported_currency_iso', $iso);
            return $this->db->single();
        }
        
        public function getEmailTemplateById($email_template_id)
        {
            $this->db->query('SELECT * FROM email_templates WHERE email_template_id = :email_template_id');
            $this->db->bind(':email_template_id', $email_template_id);
            return $this->db->single();
        }
        
        public function getEmailTemplateByType($email_type)
        {
            $this->db->query('SELECT * FROM email_templates WHERE email_type = :email_type');
            $this->db->bind(':email_type', $email_type);
            return $this->db->single();
        }
        
        public function getEmailTemplates()
        {
            $this->db->query("SELECT * FROM email_templates");
            return $this->db->resultSet();
        }
        
        public function getEmailTemplatesByItem($item)
        {
            $this->db->query("SELECT * FROM email_templates WHERE item = :item");
            $this->db->bind(':item', $item);
            return $this->db->resultSet();
        }
        
        public function updateEmailTemplate($data)
        {
            $this->db->query('UPDATE email_templates SET email_subject=:email_subject, email_message=:email_message WHERE email_template_id = :email_template_id');
// Bind values
            $this->db->bind(':email_subject', $data['email_subject']);
            $this->db->bind(':email_message', $data['email_message']);
            $this->db->bind(':email_template_id', $data['email_template_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function enableTemplate($id)
        {
            $this->db->query('UPDATE email_templates SET email_template_status=:email_template_status WHERE email_template_id = :email_template_id');
// Bind values
            $this->db->bind(':email_template_status', 1);
            $this->db->bind(':email_template_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function disableTemplate($id)
        {
            $this->db->query('UPDATE email_templates SET email_template_status=:email_template_status WHERE email_template_id = :email_template_id');
// Bind values
            $this->db->bind(':email_template_status', 0);
            $this->db->bind(':email_template_id', $id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getEmailGateways()
        {
            $this->db->query("SELECT * FROM email_gateways");
            return $this->db->resultSet();
        }
        
        public function getEmailGatewayById($email_gateway_id)
        {
            $this->db->query("SELECT * FROM email_gateways WHERE email_gateway_id = :email_gateway_id");
            $this->db->bind(':email_gateway_id', $email_gateway_id);
            return $this->db->single();
        }
        
        public function getPaymentGateways()
        {
            $this->db->query("SELECT * FROM payment_gateways");
            return $this->db->resultSet();
        }
        
        public function getPaymentGatewayById($payment_gateway_id)
        {
            $this->db->query("SELECT * FROM payment_gateways WHERE payment_gateway_id = :payment_gateway_id");
            $this->db->bind(':payment_gateway_id', $payment_gateway_id);
            return $this->db->single();
        }
        
        public function getCountries()
        {
            $this->db->query('SELECT * FROM countries');
            return $this->db->resultSet();
        }
        
        public function getCountryById($country_id)
        {
            $this->db->query('SELECT * FROM countries WHERE country_id=:country_id');
            $this->db->bind(':country_id', $country_id);
            return $this->db->single();
        }
        
        public function getCountryByName($country_name)
        {
            $this->db->query('SELECT * FROM countries WHERE short_name=:short_name');
            $this->db->bind(':short_name', $country_name);
            return $this->db->single();
        }
        
        public function getCurrencies()
        {
            $this->db->query('SELECT * FROM currencies');
            return $this->db->resultSet();
        }
        
        public function getCurrencyById($currency_id)
        {
            $this->db->query('SELECT * FROM currencies WHERE currency_id=:currency_id');
            $this->db->bind(':currency_id', $currency_id);
            return $this->db->single();
        }
        
        public function getCompanyCurrencyId($currency_iso)
        {
            $this->db->query('SELECT * FROM currencies WHERE currency_iso=:currency_iso');
            $this->db->bind(':currency_iso', $currency_iso);
            return $this->db->single();
        }
        
        public function getCreditCardTypes()
        {
            $this->db->query('SELECT * FROM credit_card_types');
            return $this->db->resultSet();
        }
        
        public function getSupportedDateTimeFormats()
        {
            $this->db->query('SELECT * FROM supported_datetime_formats');
            return $this->db->resultSet();
        }
        
        public function getSupportedDateTimeFormatById($id)
        {
            $this->db->query('SELECT * FROM supported_datetime_formats WHERE supported_datetime_format_id=:supported_datetime_format_id');
            $this->db->bind(':supported_datetime_format_id', $id);
            return $this->db->single();
        }
        
        public function getSupportedDateFormats()
        {
            $this->db->query('SELECT * FROM supported_date_formats');
            return $this->db->resultSet();
        }
        
        public function getSupportedDateFormatById($id)
        {
            $this->db->query('SELECT * FROM supported_date_formats WHERE supported_date_format_id=:supported_date_format_id');
            $this->db->bind(':supported_date_format_id', $id);
            return $this->db->single();
        }
        
        public function getTCPFFonts()
        {
            $this->db->query("SELECT * FROM tcpdf_fonts");
            return $this->db->resultSet();
        }
        
        public function getTimezones()
        {
            $this->db->query("SELECT * FROM timezones");
            return $this->db->resultSet();
        }
        
        public function getSocialProfiles()
        {
            $this->db->query('SELECT * FROM social_profiles ORDER BY social_profile_id Desc');
            return $this->db->resultSet();
        }
        
        public function addSocialProfile($data)
        {
            $this->db->query('INSERT INTO social_profiles (social_profile_added_by, social_profile_name, social_profile_link, social_profile_icon, social_profile_status_id, social_profile_date_added) VALUES(:social_profile_added_by, :social_profile_name, :social_profile_link, :social_profile_icon, :social_profile_status_id, :social_profile_date_added)');
            $this->db->bind(':social_profile_added_by', $data['social_profile_added_by']);
            $this->db->bind(':social_profile_name', $data['social_profile_name']);
            $this->db->bind(':social_profile_link', $data['social_profile_link']);
            $this->db->bind(':social_profile_icon', $data['social_profile_icon']);
            $this->db->bind(':social_profile_status_id', $data['social_profile_status_id']);
            $this->db->bind(':social_profile_date_added', now());
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateSocialProfile($data)
        {
            $this->db->query('UPDATE social_profiles SET social_profile_name = :social_profile_name, social_profile_link = :social_profile_link, social_profile_icon = :social_profile_icon, social_profile_status_id = :social_profile_status_id WHERE social_profile_id = :social_profile_id');
// Bind values
            $this->db->bind(':social_profile_name', $data['social_profile_name']);
            $this->db->bind(':social_profile_link', $data['social_profile_link']);
            $this->db->bind(':social_profile_icon', $data['social_profile_icon']);
            $this->db->bind(':social_profile_status_id', $data['social_profile_status_id']);
            $this->db->bind(':social_profile_id', $data['social_profile_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function deleteSocialProfile($social_profile_id)
        {
            $this->db->query('DELETE FROM social_profiles WHERE social_profile_id = :social_profile_id');
            $this->db->bind(':social_profile_id', $social_profile_id);
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getProgrammingLanguages()
        {
            $this->db->query('SELECT * FROM programming_languages ORDER BY programming_language_id Desc');
            return $this->db->resultSet();
        }
        
        public function getItemSettings()
        {
            $this->db->query('SELECT * FROM item_settings ORDER BY item_id Desc');
            return $this->db->resultSet();
        }
        
        public function getItemByName($item_name)
        {
            $this->db->query('SELECT * FROM item_settings WHERE item_name = :item_name');
            $this->db->bind(':item_name', $item_name);
            return $this->db->single();
        }
        
        public function updateItemSettings($data)
        {
            $this->db->query('UPDATE item_settings SET item_hex_colour = :item_hex_colour, item_rgb_colour = :item_rgb_colour WHERE item_id = :item_id');
// Bind values
            $this->db->bind(':item_hex_colour', $data['item_hex_colour']);
            $this->db->bind(':item_rgb_colour', $data['item_rgb_colour']);
            $this->db->bind(':item_id', $data['item_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getChatIconAnimations()
        {
            $this->db->query("SELECT * FROM chat_icon_animations ORDER BY chat_icon_animation_id Desc");
            return $this->db->resultSet();
        }
    }