<?php
    
    class Subscription
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getFreeTrialSubscription($user_id, $payment_option_id)
        {
            $this->db->query("SELECT * 
FROM subscriptions 
WHERE user_id = :user_id AND payment_option_id = :payment_option_id");
            $this->db->bind(':user_id', $user_id);
            $this->db->bind(':payment_option_id', $payment_option_id);
            return $this->db->single();
        }
        
        public function addSubscriptionPayment($data)
        {
            if (getOption('approve_subscription_automatically') && payment_option($data['payment_option_id'])->payment_option_online == 1) {
                $is_approve_subscription_automatically_enabled = 2;
                $subscription_date_activated = now();
            } else {
                $is_approve_subscription_automatically_enabled = 1;
                $subscription_date_activated = NULL;
            }
            $next_subscription_name = getOption('subscription_prefix') . '-' . time();
            $this->db->query('INSERT INTO subscriptions (subscription_uid, user_id, subscription_added_by, subscription_name, package_id, order_code, payment_option_id, transaction_id, transaction_reference, subscription_items, subscription_cost, subscription_duration, subscription_duration_type, subscription_start_date, subscription_end_date, subscription_status_id, subscription_date_activated,subscription_date_added) VALUES(:subscription_uid, :user_id, :subscription_added_by, :subscription_name, :package_id, :order_code, :payment_option_id, :transaction_id, :transaction_reference, :subscription_items, :subscription_cost, :subscription_duration, :subscription_duration_type, :subscription_start_date, :subscription_end_date, :subscription_status_id, :subscription_date_activated, :subscription_date_added)');
// Bind values
            $this->db->bind(':subscription_uid', $data['subscription_uid']);
            $this->db->bind(':user_id', $data['user_id'] ?? $data['subscription_added_by']);
            $this->db->bind(':subscription_added_by', $data['subscription_added_by']);
            $this->db->bind(':subscription_name', $next_subscription_name);
            $this->db->bind(':package_id', $data['package_id']);
            $this->db->bind(':order_code', getOption('order_prefix').'-'.generate_otp(getOption('order_code_length')));
            $this->db->bind(':payment_option_id', $data['payment_option_id']);
            $this->db->bind(':transaction_id', $data['transaction_id']);
            $this->db->bind(':transaction_reference', $data['transaction_reference']);
            $this->db->bind(':subscription_items', $data['subscription_items']);
            $this->db->bind(':subscription_cost', $data['subscription_cost']);
            $this->db->bind(':subscription_duration', $data['package_duration']);
            $this->db->bind(':subscription_duration_type', $data['package_duration_type']);
            $this->db->bind(':subscription_start_date', now());
            $this->db->bind(':subscription_end_date', last_day($data['package_duration'], $data['package_duration_type']));
            $this->db->bind(':subscription_status_id', $is_approve_subscription_automatically_enabled);
            $this->db->bind(':subscription_date_activated', $subscription_date_activated);
            $this->db->bind(':subscription_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getTempSubscriptionPayment($reference_number, $session_id)
        {
            $this->db->query('SELECT * FROM subscriptions WHERE transaction_reference = :transaction_reference AND transaction_session_id = :transaction_session_id');
// Bind value
            $this->db->bind(':transaction_reference', $reference_number);
            $this->db->bind(':transaction_session_id', $session_id);
            return $this->db->single();
        }
        
        public function getSubscriptionPaymentByTransactionId($id)
        {
            $this->db->query('SELECT * FROM subscriptions WHERE transaction_id = :transaction_id');
            $this->db->bind(':transaction_id', $id);
            return $this->db->single();
        }
        
        public function getSubscriptionPaymentByTransactionReference($id)
        {
            $this->db->query('SELECT * FROM subscriptions WHERE transaction_reference = :transaction_reference');
            $this->db->bind(':transaction_reference', $id);
            return $this->db->single();
        }
        
        public function getSubscriptionPaymentByOrderCode($id)
        {
            $this->db->query('SELECT * FROM subscriptions WHERE order_code LIKE :order_code');
            $this->db->bind(':order_code', '%'.$id.'%');
            return $this->db->resultSet();
        }
        
        private function nextSubscriptionId()
        {
            $this->db->query("SHOW TABLE STATUS LIKE 'subscriptions'");
            $row = $this->db->single();
            return $row->Auto_increment;
        }
        
        public function getSubscriptions($user_id = NULL)
        {
            $query = "SELECT * FROM subscriptions ";
            if (isset($user_id)) {
                $query .= " WHERE user_id = :user_id";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
// Bind values
                $this->db->bind(':user_id', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getActiveSubscriptions($subscriber_id, $order_by = NULL)
        {
            $query = "SELECT * FROM subscriptions
WHERE user_id = :user_id 
AND subscription_end_date > :subscription_end_date ";
            if (isset($order_by)) {
                $query .= " ORDER BY $order_by Asc";
            } else {
                $query .= " ORDER BY subscription_id Desc";
            }
            $this->db->query($query);
            $this->db->bind(':user_id', $subscriber_id);
            $this->db->bind(':subscription_end_date', now());
            return $this->db->resultSet();
        }
        
        public function getSubscriptionsByExpiry($expiry, $plan_type = NULL)
        {
            $query = "SELECT *
FROM subscriptions ";
            if ($expiry == 'expired') {
                $query .= " WHERE subscription_end_date < :subscription_end_date ";
            } elseif ($expiry == 'active') {
                $query .= " WHERE subscription_end_date > :subscription_end_date ";
            }
            if (isset($plan_type)) {
                $query .= " AND plan_type = :plan_type";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
            if (isset($plan_type)) {
// Bind values
                $this->db->bind(':plan_type', $plan_type);
            }
            $this->db->bind(':subscription_end_date', now());
            return $this->db->resultSet();
        }
        
        public function getSubscriptionById($id)
        {
            $this->db->query('SELECT *
FROM subscriptions 
WHERE subscription_id = :subscription_id');
            $this->db->bind(':subscription_id', $id);
            return $this->db->single();
        }
        
        public function updateSubscriptionStatus($data)
        {
            $this->db->query('UPDATE subscriptions SET subscription_status_id = :subscription_status_id WHERE subscription_id = :subscription_id');
// Bind values
            $this->db->bind(':subscription_status_id', $data['subscription_status_id']);
            $this->db->bind(':subscription_id', $data['subscription_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function updateSubscriptionDateActivated($data)
        {
            $this->db->query('UPDATE subscriptions SET subscription_date_activated = :subscription_date_activated WHERE subscription_id = :subscription_id');
// Bind values
            $this->db->bind(':subscription_date_activated', $data['subscription_date_activated']);
            $this->db->bind(':subscription_id', $data['subscription_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function Editsubscription($data)
        {
            $this->db->query('UPDATE subscriptions SET payment_mode = :payment_mode, transaction_id = :transaction_id, amount_paid = :amount_paid WHERE subscription_id =:subscription_id');
// Bind values
            $this->db->bind(':payment_mode', $data['payment_mode']);
            $this->db->bind(':transaction_id', $data['transaction_id']);
            $this->db->bind(':amount_paid', $data['amount_added']);
            $this->db->bind(':subscription_id', $data['subscription']->subscription_id);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getSubscriptionTotals($whose_subscription = NULL)
        {
            $query = "SELECT
SUM(subscription_cost) as subscription_cost FROM subscriptions ";
            if (isset($whose_subscription)) {
                $query .= " WHERE user_id = :user_id ";
            }
            $this->db->query($query);

// Bind values
            if (isset($whose_subscription)) {
                $this->db->bind(':user_id', $whose_subscription);
            }
            return $this->db->single();
        }
        
        public function getSubscriptionMonthlyTotals($whose_subscription = NULL)
        {
            if (isset($whose_subscription)) {
                $this->db->query("SELECT
SUM(amount_paid) as amount_paid FROM subscriptions 
GROUP BY YEAR(subscription_date_added), 
MONTH(subscription_date_added) 
WHERE user_id = :user_id");
// Bind values
                $this->db->bind(':user_id', $whose_subscription);
            } else {
                $this->db->query("SELECT * FROM subscriptions
GROUP BY YEAR(subscription_date_added), 
MONTH(subscription_date_added)");
            }
            return $this->db->ResultSet();
        }
        
        public function getDailySubscriptions($day)
        {
            $this->db->query("SELECT
SUM(amount_paid) as amount_paid 
FROM subscriptions 
WHERE DATE(subscription_date_added) = :day");
// Bind values
            $this->db->bind(':day', $day);
            return $this->db->single();
        }
        
        public function getWeeklySubscriptions()
        {
            $this->db->query("SELECT
SUM(amount_paid) as amount_paid 
FROM subscriptions 
GROUP BY WEEK(subscription_date_added)");
            return $this->db->single();
        }


//-----------------------
        
        public function getSubscriptionsBetweenDates($from_date, $to_date, $user_id = NULL)
        {
            $query = "SELECT * FROM subscriptions
WHERE subscription_date_added BETWEEN :from_date AND :to_date ";
            if (isset($user_id)) {
                $query .= " AND user_id = :user_id ";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            if (isset($user_id)) {
                $this->db->bind(':user_id', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getSubscriptionsBetweenDatesTotals($from_date, $to_date)
        {
            $query = "SELECT SUM(subscription_cost) as subscription_cost FROM subscriptions WHERE subscription_date_added BETWEEN :from_date AND :to_date";
            $this->db->query($query);
// Bind values
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            return $this->db->single();
        }
        
        public function getSubscriptionsTotals()
        {
            $query = "SELECT SUM(subscription_cost) as subscription_cost FROM subscriptions ";
            $this->db->query($query);
// Bind values
            return $this->db->single();
        }
        
        public function getSubscriptionsByStatus($subscription_status_id, $added_by = NULL)
        {
            $query = "SELECT *
FROM subscriptions WHERE subscription_status_id = :subscription_status_id";
            if (isset($added_by)) {
                $query .= " AND subscription_added_by = :subscription_added_by";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
            if (isset($added_by)) {
// Bind values
                $this->db->bind(':subscription_added_by', $added_by);
            }
            $this->db->bind(':subscription_status_id', $subscription_status_id);
            return $this->db->resultSet();
        }
        
        public function getSubscriptionsByStatusAndType($subscription_status_id, $plan_type, $company_id = NULL)
        {
            $query = "SELECT *
FROM subscriptions 
WHERE subscription_status_id = :subscription_status_id";
            $query .= " AND plan_type = :plan_type";
            if (isset($company_id)) {
                $query .= " AND company_id = :company_id";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':plan_type', $plan_type);
            $this->db->bind(':subscription_status_id', $subscription_status_id);
            if (isset($company_id)) {
                $this->db->bind(':company_id', $company_id);
            }
            return $this->db->resultSet();
        }
        
        public function getSubscriptionsByType($plan_type, $company_id = NULL)
        {
            $query = "SELECT *
FROM subscriptions ";
            $query .= " WHERE plan_type = :plan_type";
            if (isset($company_id)) {
                $query .= " AND company_id = :company_id";
            }
            $query .= " ORDER BY subscription_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':plan_type', $plan_type);
            if (isset($company_id)) {
                $this->db->bind(':company_id', $company_id);
            }
            return $this->db->resultSet();
        }
        
        public function getSubscriptionStatus()
        {
            $this->db->query('SELECT * FROM subscription_status');
            return $this->db->resultSet();
        }
        
        public function getSubscriptionStatusById($id)
        {
            $this->db->query('SELECT * FROM subscription_status WHERE subscription_status_id = :subscription_status_id');
            $this->db->bind(':subscription_status_id', $id);
            return $this->db->single();
        }
        
    }