<?php
    
    class Ticket
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getTickets($user_id = NULL)
        {
            $query = "SELECT * FROM tickets ";
            if (isset($user_id)) {
                $query .= " WHERE ticket_added_by = :ticket_added_by ";
            }
            $query .= " ORDER BY ticket_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':ticket_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addTicket($data)
        {
            $this->db->query('INSERT INTO tickets (ticket_added_by, ticket_priority_id, ticket_subject, ticket_description, ticket_date_added) VALUES(:ticket_added_by, :ticket_priority_id, :ticket_subject, :ticket_description, :ticket_date_added)');
            // Bind values
            $this->db->bind(':ticket_added_by', user_id());
            $this->db->bind(':ticket_priority_id', $data['ticket_priority_id']);
            $this->db->bind(':ticket_subject', $data['ticket_subject']);
            $this->db->bind(':ticket_description', $data['ticket_description']);
            $this->db->bind(':ticket_date_added', now());
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getTicketById($id)
        {
            $this->db->query('SELECT * FROM tickets WHERE ticket_id = :ticket_id');
            $this->db->bind(':ticket_id', $id);
            return $this->db->single();
        }
        
        public function getTicketStatuses()
        {
            $this->db->query('SELECT * FROM ticket_status');
            return $this->db->resultSet();
        }
        
        public function getTicketStatusById($id)
        {
            $this->db->query('SELECT * FROM ticket_status WHERE ticket_status_id = :ticket_status_id');
            $this->db->bind(':ticket_status_id', $id);
            return $this->db->single();
        }
        
        public function getTicketPriorities()
        {
            $this->db->query('SELECT * FROM ticket_priority');
            return $this->db->resultSet();
        }
        
        public function getTicketPriorityById($id)
        {
            $this->db->query('SELECT * FROM ticket_priority WHERE ticket_priority_id = :ticket_priority_id');
            $this->db->bind(':ticket_priority_id', $id);
            return $this->db->single();
        }
        
        public function updateTicketStatus($data)
        {
            $this->db->query('UPDATE tickets SET ticket_status_id = :ticket_status_id WHERE ticket_id = :ticket_id');
            $this->db->bind(':ticket_status_id', $data['ticket_status_id']);
            $this->db->bind(':ticket_id', $data['ticket_id']);
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getTicketsByStatus($ticket_status_id)
        {
            $this->db->query('SELECT * FROM tickets
                        LEFT JOIN ticket_status 
                        ON tickets.ticket_status_id = ticket_status.ticket_status_id 
                        WHERE tickets.ticket_status_id = :ticket_status_id ORDER BY tickets.ticket_id Desc');
            $this->db->bind(':ticket_status_id', $ticket_status_id);
    
    
            return $this->db->resultSet();
        }
        
        public function getTicketReplies($id)
        {
            $this->db->query('SELECT * FROM ticket_replies WHERE ticket_id = :ticket_id ORDER BY ticket_reply_id DESC');
            $this->db->bind(':ticket_id', $id);
            return $this->db->resultSet();
        }
        
        public function addTicketReply($data)
        {
            $this->db->query('INSERT INTO ticket_replies (ticket_reply_uid, ticket_id, ticket_reply_added_by, ticket_reply_date_added, ticket_reply, ticket_reply_ip) VALUES(:ticket_reply_uid, :ticket_id, :ticket_reply_added_by, :ticket_reply_date_added, :ticket_reply, :ticket_reply_ip)');
// Bind values
            $this->db->bind(':ticket_reply_uid', uid());
            $this->db->bind(':ticket_id', $data['ticket_id']);
            $this->db->bind(':ticket_reply_added_by', $data['ticket_reply_added_by']);
            $this->db->bind(':ticket_reply_date_added', now());
            $this->db->bind(':ticket_reply', $data['ticket_reply']);
            $this->db->bind(':ticket_reply_ip', ip_address());
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        
    }