<?php
    
    class Visit
    {
        private Database $db;
        
        public function __construct()
        {
            $this->db = new Database;
        }
        
        public function getVisits($user_id = NULL)
        {
            $query = "SELECT * FROM visits ";
            if (isset($user_id)) {
                $query .= " WHERE visit_added_by = :visit_added_by";
            }
            $query .= " ORDER BY visit_id Desc";
            $this->db->query($query);
            if (isset($user_id)) {
                $this->db->bind(':visit_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function getOpenVisits()
        {
            $query = "SELECT * FROM visits ";
            $query .= " WHERE visit_date_ended IS NULL ";
            $query .= " ORDER BY visit_id Desc";
            $this->db->query($query);
            return $this->db->resultSet();
        }
        
        public function addVisit($data)
        {
            $this->db->query('INSERT INTO visits (widget_id, visit_uid, visit_added_by, visit_location, visit_ip_address, visit_platform, visit_last_activity, visit_date_added) VALUES(:widget_id, :visit_uid, :visit_added_by, :visit_location, :visit_ip_address, :visit_platform, :visit_last_activity, :visit_date_added)');
// Bind values
            $this->db->bind(':widget_id', $data['widget_id'] ?? NULL);
            $this->db->bind(':visit_uid', getOption('visit_prefix') . '-' . uid());
            $this->db->bind(':visit_added_by', $data['visit_added_by']);
            $this->db->bind(':visit_location', $data['visit_location']);
            $this->db->bind(':visit_ip_address', $data['visit_ip_address']);
            $this->db->bind(':visit_platform', $data['visit_platform']);
            $this->db->bind(':visit_last_activity', $data['visit_last_activity']);
            $this->db->bind(':visit_date_added', now());
// Execute
            if ($this->db->execute()) {
                return $this->db->lastInsertId();
            } else {
                return false;
            }
        }
        
        public function getVisitById($id)
        {
            $this->db->query('SELECT * FROM visits
WHERE visit_id = :visit_id');
            $this->db->bind(':visit_id', $id);
            return $this->db->single();
        }
        
        public function getLastVisitByIpAddress($ip_address, $closed = NULL)
        {
            $query = "SELECT * FROM visits ";
            $query .= "WHERE visit_ip_address = :visit_ip_address ";
            if (isset($closed)) {
                $query .= "AND visit_date_ended IS NOT NULL ";
            } else {
                $query .= "AND visit_date_ended IS NULL ";
            }
            $query .= "ORDER BY visit_id DESC LIMIT 1";
            $this->db->query($query);
            $this->db->bind(':visit_ip_address', $ip_address);
            return $this->db->single();
        }
        
        public function updateVisitLastActivity($data)
        {
            $this->db->query('UPDATE visits SET visit_last_activity = :visit_last_activity WHERE visit_id = :visit_id');
// Bind values
            $this->db->bind(':visit_last_activity', $data['visit_last_activity']);
            $this->db->bind(':visit_id', $data['visit_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function endVisit($data)
        {
            $this->db->query('UPDATE visits SET visit_date_ended = :visit_date_ended WHERE visit_id = :visit_id');
// Bind values
            $this->db->bind(':visit_date_ended', $data['visit_date_ended']);
            $this->db->bind(':visit_id', $data['visit_id']);
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getVisitsBetweenDates($from_date, $to_date, $user_id = NULL)
        {
            $query = "SELECT * FROM visits WHERE visit_date_added BETWEEN :from_date AND :to_date";
            if (isset($user_id)) {
                $query .= " AND visit_added_by = :visit_added_by";
            }
            $query .= " ORDER BY visit_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            if (isset($user_id)) {
                $this->db->bind(':visit_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
        public function addVisitTimeline($data)
        {
            $this->db->query('INSERT INTO visit_timelines (visit_id, visit_timeline_action, visit_timeline, visit_timeline_date_added) VALUES(:visit_id, :visit_timeline_action, :visit_timeline, :visit_timeline_date_added)');
// Bind values
            $this->db->bind(':visit_id', $data['visit_id']);
            $this->db->bind(':visit_timeline_action', $data['visit_timeline_action']);
            $this->db->bind(':visit_timeline', $data['visit_timeline']);
            $this->db->bind(':visit_timeline_date_added', now());
// Execute
            if ($this->db->execute()) {
                return true;
            } else {
                return false;
            }
        }
        
        public function getVisitTimelines($visit_id)
        {
            $query = "SELECT * FROM visit_timelines ";
            $query .= " WHERE visit_id = :visit_id ";
            $query .= " ORDER BY visit_timeline_id Desc";
            $this->db->query($query);
            $this->db->bind(':visit_id', $visit_id);
            return $this->db->resultSet();
        }
        
        public function getLastVisitTimelineByIpAddressAndType($ip_address, $type)
        {
            $query = "SELECT * FROM visit_timelines ";
            $query .= "WHERE visit_timeline_action = :visit_timeline_action ";
            $query .= "ORDER BY visit_timeline_id DESC LIMIT 1";
            $this->db->query($query);
            $this->db->bind(':visit_timeline_action', $type);
            return $this->db->single();
        }
        
        public function getVisitTimelinesBetweenDates($from_date, $to_date, $user_id = NULL)
        {
            $query = "SELECT visit_timeline_id,a.visit_id,visit_timeline_action,visit_timeline,visit_timeline_date_added,visit_added_by FROM visit_timelines a LEFT JOIN visits b ON a.visit_id = b.visit_id WHERE visit_timeline_date_added BETWEEN :from_date AND :to_date AND visit_timeline_action = :visit_timeline_action ";
            if (isset($user_id)) {
                $query .= " AND visit_added_by = :visit_added_by";
            }
            $query .= " ORDER BY visit_timeline_id Desc";
            $this->db->query($query);
// Bind values
            $this->db->bind(':visit_timeline_action', 'visited');
            $this->db->bind(':from_date', $from_date);
            $this->db->bind(':to_date', $to_date);
            if (isset($user_id)) {
                $this->db->bind(':visit_added_by', $user_id);
            }
            return $this->db->resultSet();
        }
        
    }